@@ -2,7 +2,7 @@
  * audio_output.h : audio output interface
  *****************************************************************************
  * Copyright (C) 2002 VideoLAN
- * $Id: audio_output.h,v 1.81 2003/08/03 23:11:21 gbazin Exp $
+ * $Id: audio_output.h,v 1.86 2003/11/20 22:10:55 fenrir Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -20,33 +20,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
+#ifndef _VLC_AUDIO_OUTPUT_H
+#define _VLC_AUDIO_OUTPUT_H 1
 
-/*****************************************************************************
- * audio_sample_format_t
- *****************************************************************************
- * This structure defines a format for audio samples.
- *****************************************************************************/
-struct audio_sample_format_t
-{
-    vlc_fourcc_t        i_format;
-    unsigned int        i_rate;
-    /* Describes the channels configuration of the samples (ie. number of
-     * channels which are available in the buffer, and positions). */
-    uint32_t            i_physical_channels;
-    /* Describes from which original channels, before downmixing, the
-     * buffer is derived. */
-    uint32_t            i_original_channels;
-    /* Optional - for A/52, SPDIF and DTS types : */
-    /* Bytes used by one compressed frame, depends on bitrate. */
-    unsigned int        i_bytes_per_frame;
-    /* Number of sampleframes contained in one compressed frame. */
-    unsigned int        i_frame_length;
-    /* Please note that it may be completely arbitrary - buffers are not
-     * obliged to contain a integral number of so-called "frames". It's
-     * just here for the division :
-     * buffer_size = i_nb_samples * i_bytes_per_frame / i_frame_length
-     */
-};
+#include "vlc_es.h"
 
 #define AOUT_FMTS_IDENTICAL( p_first, p_second ) (                          \
     ((p_first)->i_format == (p_second)->i_format)                           \
@@ -125,11 +102,12 @@ typedef int32_t vlc_fixed_t;
 /* Values used for the audio-device and audio-channels object variables */
 #define AOUT_VAR_MONO               1
 #define AOUT_VAR_STEREO             2
-#define AOUT_VAR_2F2R               3
-#define AOUT_VAR_5_1                4
-#define AOUT_VAR_6_1                5
-#define AOUT_VAR_7_1                6
-#define AOUT_VAR_SPDIF              7
+#define AOUT_VAR_2F2R               4
+#define AOUT_VAR_3F2R               5
+#define AOUT_VAR_5_1                6
+#define AOUT_VAR_6_1                7
+#define AOUT_VAR_7_1                8
+#define AOUT_VAR_SPDIF              10
 
 #define AOUT_VAR_CHAN_STEREO        1
 #define AOUT_VAR_CHAN_RSTEREO       2
@@ -207,3 +185,4 @@ VLC_EXPORT( int, aout_Restart, ( aout_instance_t * p_aout ) );
 VLC_EXPORT( int, aout_FindAndRestart, ( vlc_object_t *, const char *, vlc_value_t, vlc_value_t, void * ) );
 VLC_EXPORT( int, aout_ChannelsRestart, ( vlc_object_t *, const char *, vlc_value_t, vlc_value_t, void * ) );
 
+#endif /* _VLC_AUDIO_OUTPUT_H */
