@@ -3,7 +3,7 @@
  * Collection of useful common types and macros definitions
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: vlc_common.h,v 1.96 2003/12/06 22:45:53 jpsaman Exp $
+ * $Id: vlc_common.h,v 1.110 2004/02/29 13:08:56 gbazin Exp $
  *
  * Authors: Samuel Hocevar <sam@via.ecp.fr>
  *          Vincent Seguin <seguin@via.ecp.fr>
@@ -43,6 +43,8 @@
 #   undef HAVE_STDINT_H
 #   undef HAVE_INTTYPES_H
 #   undef off_t
+#elif defined( _MSC_VER )
+#   pragma warning( disable : 4244 )
 #endif
 
 #include "vlc_config.h"
@@ -190,6 +192,9 @@ typedef struct msg_subscription_t msg_subscription_t;
 typedef struct playlist_t playlist_t;
 typedef struct playlist_item_t playlist_item_t;
 typedef struct playlist_group_t playlist_group_t;
+typedef struct item_info_t item_info_t;
+typedef struct item_info_category_t item_info_category_t;
+typedef struct playlist_export_t playlist_export_t;
 
 /* Modules */
 typedef struct module_bank_t module_bank_t;
@@ -219,16 +224,17 @@ typedef struct es_sys_t es_sys_t;
 typedef struct pgrm_descriptor_t pgrm_descriptor_t;
 typedef struct pgrm_sys_t pgrm_sys_t;
 typedef struct stream_descriptor_t stream_descriptor_t;
-typedef struct stream_sys_t stream_sys_t;
 
 /* Format */
 typedef struct audio_format_t audio_format_t;
 typedef struct video_format_t video_format_t;
 typedef struct subs_format_t subs_format_t;
-typedef struct es_format_t  es_format_t;
+typedef struct es_format_t es_format_t;
+typedef struct video_palette_t video_palette_t;
 
 /* NInput */
-typedef struct stream_t stream_t;
+typedef struct stream_sys_t stream_sys_t;
+typedef struct stream_t     stream_t;
 typedef struct es_out_t     es_out_t;
 typedef struct es_out_id_t  es_out_id_t;
 typedef struct es_out_sys_t es_out_sys_t;
@@ -450,12 +456,11 @@ typedef int ( * vlc_callback_t ) ( vlc_object_t *,      /* variable's object */
 #define TAB_APPEND( count, tab, p )             \
     if( (count) > 0 )                           \
     {                                           \
-        (*(void **)(&tab)) =                    \
-            realloc( tab, sizeof( void ** ) * ( (count) + 1 ) ); \
+        (void *)(tab) = realloc( tab, sizeof( void ** ) * ( (count) + 1 ) ); \
     }                                           \
     else                                        \
     {                                           \
-        (*(void **)(&tab)) = malloc( sizeof( void ** ) );    \
+        (void *)(tab) = malloc( sizeof( void ** ) );    \
     }                                           \
     ((void**)(tab))[count] = (void*)(p);        \
     (count)++
@@ -499,18 +504,18 @@ typedef int ( * vlc_callback_t ) ( vlc_object_t *,      /* variable's object */
  * MSB, and should be used for both network communications and files. Note that
  * byte orders other than little and big endians are not supported, but only
  * the VAX seems to have such exotic properties. */
-static inline uint16_t U16_AT( void * _p )
+static inline uint16_t U16_AT( void const * _p )
 {
     uint8_t * p = (uint8_t *)_p;
     return ( ((uint16_t)p[0] << 8) | p[1] );
 }
-static inline uint32_t U32_AT( void * _p )
+static inline uint32_t U32_AT( void const * _p )
 {
     uint8_t * p = (uint8_t *)_p;
     return ( ((uint32_t)p[0] << 24) | ((uint32_t)p[1] << 16)
               | ((uint32_t)p[2] << 8) | p[3] );
 }
-static inline uint64_t U64_AT( void * _p )
+static inline uint64_t U64_AT( void const * _p )
 {
     uint8_t * p = (uint8_t *)_p;
     return ( ((uint64_t)p[0] << 56) | ((uint64_t)p[1] << 48)
@@ -519,18 +524,18 @@ static inline uint64_t U64_AT( void * _p )
               | ((uint64_t)p[6] << 8) | p[7] );
 }
 
-static inline uint16_t GetWLE( void * _p )
+static inline uint16_t GetWLE( void const * _p )
 {
     uint8_t * p = (uint8_t *)_p;
     return ( ((uint16_t)p[1] << 8) | p[0] );
 }
-static inline uint32_t GetDWLE( void * _p )
+static inline uint32_t GetDWLE( void const * _p )
 {
     uint8_t * p = (uint8_t *)_p;
     return ( ((uint32_t)p[3] << 24) | ((uint32_t)p[2] << 16)
               | ((uint32_t)p[1] << 8) | p[0] );
 }
-static inline uint64_t GetQWLE( void * _p )
+static inline uint64_t GetQWLE( void const * _p )
 {
     uint8_t * p = (uint8_t *)_p;
     return ( ((uint64_t)p[7] << 56) | ((uint64_t)p[6] << 48)
@@ -543,6 +548,29 @@ static inline uint64_t GetQWLE( void * _p )
 #define GetDWBE( p )    U32_AT( p )
 #define GetQWBE( p )    U64_AT( p )
 
+/* Helper writer functions */
+
+#define SetWLE( p, v ) _SetWLE( (uint8_t*)p, v)
+static inline void _SetWLE( uint8_t *p, uint16_t i_dw )
+{
+    p[1] = ( i_dw >>  8 )&0xff;
+    p[0] = ( i_dw       )&0xff;
+}
+
+#define SetDWLE( p, v ) _SetDWLE( (uint8_t*)p, v)
+static inline void _SetDWLE( uint8_t *p, uint32_t i_dw )
+{
+    p[3] = ( i_dw >> 24 )&0xff;
+    p[2] = ( i_dw >> 16 )&0xff;
+    p[1] = ( i_dw >>  8 )&0xff;
+    p[0] = ( i_dw       )&0xff;
+}
+#define SetQWLE( p, v ) _SetQWLE( (uint8_t*)p, v)
+static inline void _SetQWLE( uint8_t *p, uint64_t i_qw )
+{
+    SetDWLE( p,   i_qw&0xffffffff );
+    SetDWLE( p+4, ( i_qw >> 32)&0xffffffff );
+}
 
 #if WORDS_BIGENDIAN
 #   define hton16(i)   ( i )
@@ -610,6 +638,20 @@ static inline uint64_t GetQWLE( void * _p )
 #   define vlc_strdup NULL
 #endif
 
+#if !defined(HAVE_VASPRINTF) || defined(SYS_DARWIN) || defined(SYS_BEOS)
+#   define vasprintf vlc_vasprintf
+    VLC_EXPORT( int, vlc_vasprintf, (char **, const char *, va_list ) );
+#elif !defined(__PLUGIN__)
+#   define vlc_vasprintf NULL
+#endif
+
+#if !defined(HAVE_ASPRINTF) || defined(SYS_DARWIN) || defined(SYS_BEOS)
+#   define asprintf vlc_asprintf
+    VLC_EXPORT( int, vlc_asprintf, (char **, const char *, ... ) );
+#elif !defined(__PLUGIN__)
+#   define vlc_asprintf NULL
+#endif
+
 #ifndef HAVE_STRNDUP
 #   if defined(STRNDUP_IN_GNOME_H) && \
         (defined(MODULE_NAME_IS_gnome)||defined(MODULE_NAME_IS_gnome_main)||\
@@ -692,8 +734,10 @@ static inline uint64_t GetQWLE( void * _p )
 /* 64 bits integer constant suffix */
 #if defined( __MINGW32__ ) || (!defined(WIN32) && !defined(UNDER_CE))
 #   define I64C(x)         x##LL
+#   define UI64C(x)        x##ULL
 #else
 #   define I64C(x)         x##i64
+#   define UI64C(x)        x##ui64
 #endif /* defined(WIN32)||defined(UNDER_CE) */
 
 #if defined(WIN32) || defined(UNDER_CE)
@@ -777,6 +821,7 @@ VLC_EXPORT( char *, vlc_wraptext, ( const char *, int, vlc_bool_t ) );
 #define CPU_CAPABILITY_3DNOW   (1<<4)
 #define CPU_CAPABILITY_MMXEXT  (1<<5)
 #define CPU_CAPABILITY_SSE     (1<<6)
+#define CPU_CAPABILITY_SSE2    (1<<7)
 #define CPU_CAPABILITY_ALTIVEC (1<<16)
 #define CPU_CAPABILITY_FPU     (1<<31)
 
