@@ -1,8 +1,8 @@
 /*****************************************************************************
- * vlc_es.h
+ * vlc_es.h: Elementary stream formats descriptions
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: vlc_es.h,v 1.5 2003/11/24 02:35:50 fenrir Exp $
+ * $Id: vlc_es.h,v 1.10 2004/02/07 00:33:08 gbazin Exp $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -25,25 +25,42 @@
 #define _VLC_ES_H 1
 
 /**
- * Description of a audio frame
+ * \file
+ * This file defines the elementary streams format types
+ */
+
+/**
+ * video palette data
+ * \see viedo_format_t
+ * \see subs_format_t
+ */
+struct video_palette_t
+{
+    int i_dummy; /**< to keep the compatibility with ffmpeg's palette */
+
+    uint32_t palette[256]; /**< 4-byte ARGB palette entries, stored in native
+                            * byte order */
+};
+
+/**
+ * audio format description
  */
 struct audio_format_t
 {
-    vlc_fourcc_t        i_format;
-
-    unsigned int        i_rate;
+    vlc_fourcc_t i_format;                          /**< audio format fourcc */
+    unsigned int i_rate;                              /**< audio sample-rate */
 
     /* Describes the channels configuration of the samples (ie. number of
      * channels which are available in the buffer, and positions). */
-    uint32_t            i_physical_channels;
+    uint32_t     i_physical_channels;
 
     /* Describes from which original channels, before downmixing, the
      * buffer is derived. */
-    uint32_t            i_original_channels;
+    uint32_t     i_original_channels;
 
     /* Optional - for A/52, SPDIF and DTS types : */
     /* Bytes used by one compressed frame, depends on bitrate. */
-    unsigned int        i_bytes_per_frame;
+    unsigned int i_bytes_per_frame;
 
     /* Number of sampleframes contained in one compressed frame. */
     unsigned int        i_frame_length;
@@ -60,7 +77,7 @@ struct audio_format_t
 };
 
 /**
- * Description of a video frame
+ * video format description
  */
 struct video_format_t
 {
@@ -78,10 +95,12 @@ struct video_format_t
 
     unsigned int i_frame_rate;                     /**< frame rate numerator */
     unsigned int i_frame_rate_base;              /**< frame rate denominator */
+
+    video_palette_t *p_palette;              /**< video palette from demuxer */
 };
 
 /**
- * Description of subs
+ * subtitles format description
  */
 struct subs_format_t
 {
@@ -107,6 +126,8 @@ struct es_format_t
     int             i_cat;
     vlc_fourcc_t    i_codec;
 
+    int             i_id;       /* -1: let the core mark the right id
+                                   >=0: valid id */
     int             i_group;    /* -1 : standalone
                                    >= 0 then a "group" (program) is created
                                         for each value */
@@ -140,6 +161,7 @@ static inline void es_format_Init( es_format_t *fmt,
 {
     fmt->i_cat                  = i_cat;
     fmt->i_codec                = i_codec;
+    fmt->i_id                   = -1;
     fmt->i_group                = 0;
     fmt->i_priority             = 0;
     fmt->psz_language           = NULL;
@@ -157,6 +179,10 @@ static inline void es_format_Init( es_format_t *fmt,
 static inline void es_format_Copy( es_format_t *dst, es_format_t *src )
 {
     memcpy( dst, src, sizeof( es_format_t ) );
+    if( src->psz_language )
+         dst->psz_language = strdup( src->psz_language );
+    if( src->psz_description )
+        dst->psz_description = strdup( src->psz_description );
     if( src->i_extra > 0 )
     {
         dst->p_extra = malloc( src->i_extra );
@@ -170,5 +196,16 @@ static inline void es_format_Copy( es_format_t *dst, es_format_t *src )
     }
 }
 
-#endif
+static inline void es_format_Clean( es_format_t *fmt )
+{
+    if( fmt->psz_language ) free( fmt->psz_language );
+    fmt->psz_language = 0;
+    if( fmt->psz_description ) free( fmt->psz_description );
+    fmt->psz_description = 0;
+    if( fmt->i_extra > 0 ) free( fmt->p_extra );
+    fmt->i_extra = 0; fmt->p_extra = 0;
+    if( fmt->video.p_palette ) free( fmt->video.p_palette );
+    fmt->video.p_palette = 0;
+}
 
+#endif
