@@ -4,7 +4,7 @@
  * interface, such as message output.
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: vlc_interface.h,v 1.4 2003/07/20 10:38:49 gbazin Exp $
+ * $Id: vlc_interface.h,v 1.10 2004/03/01 18:31:12 gbazin Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -25,12 +25,25 @@
 
 typedef struct intf_dialog_args_t intf_dialog_args_t;
 
+/**
+ * \file
+ * This file contains structures and function prototypes for
+ * interface management in vlc
+ */
+
+
 /*****************************************************************************
  * intf_thread_t: describe an interface thread
  *****************************************************************************
  * This struct describes all interface-specific data of the main (interface)
  * thread.
  *****************************************************************************/
+
+/**
+ * \defgroup vlc_interface Interface
+ * These functions and structures are for interface management
+ * @{
+ */
 struct intf_thread_t
 {
     VLC_COMMON_MEMBERS
@@ -39,14 +52,14 @@ struct intf_thread_t
     vlc_bool_t          b_block;
 
     /* Specific interfaces */
-    intf_console_t *    p_console;                                /* console */
-    intf_sys_t *        p_sys;                           /* system interface */
+    intf_console_t *    p_console;                               /** console */
+    intf_sys_t *        p_sys;                          /** system interface */
 
-    /* Interface module */
+    /** Interface module */
     module_t *   p_module;
-    void      ( *pf_run )    ( intf_thread_t * );
+    void      ( *pf_run )    ( intf_thread_t * ); /** Run function */
 
-    /* Specific for dialogs providers */
+    /** Specific for dialogs providers */
     void ( *pf_show_dialog ) ( intf_thread_t *, int, int,
                                intf_dialog_args_t * );
 
@@ -54,6 +67,9 @@ struct intf_thread_t
     vlc_mutex_t  change_lock;
     vlc_bool_t   b_menu_change;
     vlc_bool_t   b_menu;
+
+    /* Provides the ability to switch an interface on the fly */
+    char *psz_switch_intf;
 };
 
 /*****************************************************************************
@@ -92,6 +108,8 @@ VLC_EXPORT( int,               intf_RunThread,  ( intf_thread_t * ) );
 VLC_EXPORT( void,              intf_StopThread, ( intf_thread_t * ) );
 VLC_EXPORT( void,              intf_Destroy,    ( intf_thread_t * ) );
 
+/*@}*/
+
 /*****************************************************************************
  * Macros
  *****************************************************************************/
@@ -117,6 +135,8 @@ VLC_EXPORT( void,              intf_Destroy,    ( intf_thread_t * ) );
 #define INTF_DIALOG_NET         4
 #define INTF_DIALOG_SAT         5
 
+#define INTF_DIALOG_STREAMWIZARD 8
+
 #define INTF_DIALOG_PLAYLIST   10
 #define INTF_DIALOG_MESSAGES   11
 #define INTF_DIALOG_FILEINFO   12
@@ -127,3 +147,14 @@ VLC_EXPORT( void,              intf_Destroy,    ( intf_thread_t * ) );
 #define INTF_DIALOG_FILE_GENERIC 30
 
 #define INTF_DIALOG_EXIT       99
+
+/* Useful text messages shared by interfaces */
+#define INTF_ABOUT_MSG \
+    _( "VLC is an open-source and cross-platform multimedia " \
+       "player for various audio and video formats (MPEG-1, MPEG-2, MPEG-4, " \
+       "DivX, mp3, Ogg, ...) as well as DVDs, VCDs, CD audio, and various " \
+       "streaming protocols.\n\n" \
+       "VLC is also a streaming server with transcoding capabilities " \
+       "(UDP unicast and multicast, HTTP, ...) mainly designed for " \
+       "high-bandwidth networks.\n\n"\
+       "For more information, have a look at the web site." )
