@@ -2,7 +2,7 @@
  * cddax.c : CD digital audio input module for vlc using libcdio
  *****************************************************************************
  * Copyright (C) 2000,2003 VideoLAN
- * $Id: cdda.c,v 1.15 2003/12/22 02:24:51 sam Exp $
+ * $Id: cdda.c,v 1.18 2004/02/14 17:36:05 gbazin Exp $
  *
  * Authors: Rocky Bernstein <rocky@panix.com>
  *          Laurent Aimar <fenrir@via.ecp.fr>
@@ -29,7 +29,6 @@
 
 #include <vlc/vlc.h>
 
-
 /*****************************************************************************
  * prototypes
  *****************************************************************************/
@@ -38,8 +37,6 @@ void E_(Close)        ( vlc_object_t * );
 
 int  E_(OpenIntf)     ( vlc_object_t * );
 void E_(CloseIntf)    ( vlc_object_t * );
-int  E_(DemuxOpen)    ( vlc_object_t * p_this);
-void E_(DemuxClose)   ( vlc_object_t * p_this);
 
 int  E_(DebugCB)      ( vlc_object_t *p_this, const char *psz_name,
                         vlc_value_t oldval, vlc_value_t val,
@@ -114,15 +111,13 @@ vlc_module_begin();
     add_shortcut( "cddax" );
 
     /* Configuration options */
-    add_category_hint( N_("CDX"), NULL, VLC_TRUE );
-
     add_integer ( MODULE_STRING "-debug", 0, E_(DebugCB),
-                  N_("set debug mask for additional debugging."),
+                  N_("If nonzero, this gives additional debug information."),
                   DEBUG_LONGTEXT, VLC_TRUE );
 
     add_integer( MODULE_STRING "-caching",
                  DEFAULT_PTS_DELAY / 1000, NULL,
-                 N_("Caching value in ms"),
+                 N_("Caching value in microseconds"),
                  CACHING_LONGTEXT, VLC_TRUE );
 
     add_string( MODULE_STRING "-author-format",
@@ -186,10 +181,4 @@ vlc_module_begin();
 
 #endif
 
-    add_submodule();
-        set_description( _("Audio CD demux") );
-        set_capability( "demux", 0 );
-        set_callbacks( E_(DemuxOpen), E_(DemuxClose) );
-        add_shortcut( "cdda" );
-
 vlc_module_end();
