@@ -2,7 +2,7 @@
  * filter.c : DirectShow access module for vlc
  *****************************************************************************
  * Copyright (C) 2002 VideoLAN
- * $Id: filter.cpp,v 1.10 2003/12/15 00:47:18 gbazin Exp $
+ * $Id: filter.cpp,v 1.13 2004/02/16 13:33:10 gbazin Exp $
  *
  * Author: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -32,6 +32,11 @@
 #include <vlc/input.h>
 #include <vlc/vout.h>
 
+#ifndef _MSC_VER
+    /* Work-around a bug in w32api-2.5 */
+#   define QACONTAINERFLAGS QACONTAINERFLAGS_SOMETHINGELSE
+#endif
+
 #include "filter.h"
 
 #define DEBUG_DSHOW 1
@@ -294,10 +299,10 @@ STDMETHODIMP CapturePin::Disconnect()
     msg_Dbg( p_input, "CapturePin::Disconnect" );
 #endif
 
+#if 0 // FIXME: This does seem to create crashes sometimes
     VLCMediaSample vlc_sample;
     access_sys_t *p_sys = p_input->p_access_data;
 
-#if 0 // FIXME: This does seem to create crashes sometimes
     vlc_mutex_lock( &p_sys->lock );
     while( samples_queue.size() )
     {
@@ -396,21 +401,21 @@ STDMETHODIMP CapturePin::EndOfStream( void )
 #ifdef DEBUG_DSHOW
     msg_Dbg( p_input, "CapturePin::EndOfStream" );
 #endif
-    return E_NOTIMPL;
+    return S_OK;
 }
 STDMETHODIMP CapturePin::BeginFlush( void )
 {
 #ifdef DEBUG_DSHOW
     msg_Dbg( p_input, "CapturePin::BeginFlush" );
 #endif
-    return E_NOTIMPL;
+    return S_OK;
 }
 STDMETHODIMP CapturePin::EndFlush( void )
 {
 #ifdef DEBUG_DSHOW
     msg_Dbg( p_input, "CapturePin::EndFlush" );
 #endif
-    return E_NOTIMPL;
+    return S_OK;
 }
 STDMETHODIMP CapturePin::NewSegment( REFERENCE_TIME tStart,
                                      REFERENCE_TIME tStop,
@@ -419,7 +424,7 @@ STDMETHODIMP CapturePin::NewSegment( REFERENCE_TIME tStart,
 #ifdef DEBUG_DSHOW
     msg_Dbg( p_input, "CapturePin::NewSegment" );
 #endif
-    return E_NOTIMPL;
+    return S_OK;
 }
 
 /* IMemInputPin methods */
@@ -510,7 +515,7 @@ STDMETHODIMP CapturePin::ReceiveCanBlock( void )
 
 CaptureFilter::CaptureFilter( input_thread_t * _p_input, AM_MEDIA_TYPE mt )
   : p_input( _p_input ), p_pin( new CapturePin( _p_input, this, mt ) ),
-    media_type( mt ), i_ref( 1 )
+    media_type( mt ), state( State_Stopped ), i_ref( 1 )
 {
 }
 
@@ -591,9 +596,11 @@ STDMETHODIMP CaptureFilter::GetClassID(CLSID *pClsID)
 STDMETHODIMP CaptureFilter::GetState(DWORD dwMSecs, FILTER_STATE *State)
 {
 #ifdef DEBUG_DSHOW
-    msg_Dbg( p_input, "CaptureFilter::GetState" );
+    msg_Dbg( p_input, "CaptureFilter::GetState %i", state );
 #endif
-    return E_NOTIMPL;
+
+    *State = state;
+    return S_OK;
 };
 STDMETHODIMP CaptureFilter::SetSyncSource(IReferenceClock *pClock)
 {
@@ -601,7 +608,7 @@ STDMETHODIMP CaptureFilter::SetSyncSource(IReferenceClock *pClock)
     msg_Dbg( p_input, "CaptureFilter::SetSyncSource" );
 #endif
 
-    return NOERROR;
+    return S_OK;
 };
 STDMETHODIMP CaptureFilter::GetSyncSource(IReferenceClock **pClock)
 {
@@ -617,6 +624,8 @@ STDMETHODIMP CaptureFilter::Stop()
 #ifdef DEBUG_DSHOW
     msg_Dbg( p_input, "CaptureFilter::Stop" );
 #endif
+
+    state = State_Stopped;
     return S_OK;
 };
 STDMETHODIMP CaptureFilter::Pause()
@@ -624,6 +633,8 @@ STDMETHODIMP CaptureFilter::Pause()
 #ifdef DEBUG_DSHOW
     msg_Dbg( p_input, "CaptureFilter::Pause" );
 #endif
+
+    state = State_Paused;
     return S_OK;
 };
 STDMETHODIMP CaptureFilter::Run(REFERENCE_TIME tStart)
@@ -631,6 +642,8 @@ STDMETHODIMP CaptureFilter::Run(REFERENCE_TIME tStart)
 #ifdef DEBUG_DSHOW
     msg_Dbg( p_input, "CaptureFilter::Run" );
 #endif
+
+    state = State_Running;
     return S_OK;
 };
 
@@ -769,18 +782,20 @@ STDMETHODIMP CaptureEnumPins::Next( ULONG cPins, IPin ** ppPins,
     msg_Dbg( p_input, "CaptureEnumPins::Next" );
 #endif
 
-    *pcFetched = 0;
+    unsigned int i_fetched = 0;
 
     if( i_position < 1 && cPins > 0 )
     {
         IPin *pPin = p_filter->CustomGetPin();
         *ppPins = pPin;
         pPin->AddRef();
-        *pcFetched = 1;
+        i_fetched = 1;
         i_position++;
     }
 
-    return *pcFetched == cPins ? NOERROR : S_FALSE;
+    if( pcFetched ) *pcFetched = i_fetched;
+
+    return (i_fetched == cPins) ? S_OK : S_FALSE;
 };
 STDMETHODIMP CaptureEnumPins::Skip( ULONG cPins )
 {
@@ -788,13 +803,14 @@ STDMETHODIMP CaptureEnumPins::Skip( ULONG cPins )
     msg_Dbg( p_input, "CaptureEnumPins::Skip" );
 #endif
 
-    if( cPins > 1 )
+    i_position += cPins;
+
+    if( i_position > 1 )
     {
         return S_FALSE;
     }
 
-    i_position += cPins;
-    return NOERROR;
+    return S_OK;
 };
 STDMETHODIMP CaptureEnumPins::Reset()
 {
