@@ -4,8 +4,8 @@
  * Copyright (C) 1998-2003 VideoLAN
  *
  * Authors: Johan Bilien <jobi@via.ecp.fr>
- *          Jean-Paul Saman <saman@natlab.research.philips.com>
- *          Christopher Ross <ross@natlab.research.philips.com>
+ *          Jean-Paul Saman <jpsaman@saman>
+ *          Christopher Ross <chris@tebibyte.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,11 +33,15 @@
 /*****************************************************************************
  * Prototypes
  *****************************************************************************/
-int ioctl_SetFrontend (input_thread_t * p_input, struct dvb_frontend_parameters fep, int b_polarisation,
+int ioctl_SetQPSKFrontend (input_thread_t * p_input, struct dvb_frontend_parameters fep, int b_polarisation,
                        unsigned int u_lnb_lof1, unsigned int u_lnb_lof2, unsigned int u_lnb_slof,
                        unsigned int u_adapter, unsigned int u_device );
+int ioctl_SetOFDMFrontend (input_thread_t * p_input, struct dvb_frontend_parameters fep,
+                       unsigned int u_adapter, unsigned int u_device );
+int ioctl_SetQAMFrontend (input_thread_t * p_input, struct dvb_frontend_parameters fep,
+                       unsigned int u_adapter, unsigned int u_device );
 int ioctl_SetDMXFilter(input_thread_t * p_input, int i_pid, int *pi_fd, int i_type, unsigned int u_adapter, unsigned int u_device );
-int ioctl_UnsetDMXFilter(input_thread_t * p_input, int );
+int ioctl_UnsetDMXFilter(input_thread_t * p_input, int pi_fd);
 int ioctl_InfoFrontend(input_thread_t * p_input, struct dvb_frontend_info *info, unsigned int u_adapter, unsigned int u_device );
 
 /*****************************************************************************
@@ -50,3 +54,4 @@ fe_transmit_mode_t dvb_DecodeTransmission(input_thread_t * p_input, int transmis
 fe_guard_interval_t dvb_DecodeGuardInterval(input_thread_t * p_input, int guard);
 fe_hierarchy_t dvb_DecodeHierarchy(input_thread_t * p_input, int hierarchy);
 fe_spectral_inversion_t dvb_DecodeInversion(input_thread_t * p_input, int inversion);
+
