@@ -2,7 +2,7 @@
  * http.c
  *****************************************************************************
  * Copyright (C) 2001-2003 VideoLAN
- * $Id: http.c,v 1.5 2003/08/03 16:36:20 bigben Exp $
+ * $Id: http.c,v 1.11 2004/02/03 20:12:53 fenrir Exp $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -54,6 +54,7 @@ vlc_module_begin();
     set_description( _("HTTP stream ouput") );
     set_capability( "sout access", 0 );
     add_shortcut( "http" );
+    add_shortcut( "mmsh" );
     set_callbacks( Open, Close );
 vlc_module_end();
 
@@ -96,6 +97,8 @@ static struct
     { ".moov",  "video/quicktime" },
     { ".ogg",   "application/ogg" },
     { ".ogm",   "application/ogg" },
+    { ".wma",   "audio/x-ms-wma" },
+    { ".wmv",   "video/x-ms-wmv" },
     { ".wav",   "audio/wav" },
     { NULL,     NULL }
 };
@@ -133,6 +136,8 @@ static int Open( vlc_object_t *p_this )
     int                 i_bind_port;
     char                *psz_file_name;
 
+    char                *psz_mime;
+
     if( !( p_sys = p_access->p_sys =
                 malloc( sizeof( sout_access_out_sys_t ) ) ) )
     {
@@ -140,7 +145,6 @@ static int Open( vlc_object_t *p_this )
         return( VLC_EGENERIC );
     }
 
-    /* *** parse p_access->psz_name to extract bind address, port and file name *** */
     /* p_access->psz_name host.name:port/filename */
     psz_name = psz_parser = strdup( p_access->psz_name );
 
@@ -187,6 +191,10 @@ static int Open( vlc_object_t *p_this )
         strcpy( psz_file_name, "/" );
         strcat( psz_file_name, p );
     }
+    else
+    {
+        psz_file_name = strdup( psz_file_name );
+    }
 
     p_sys->p_httpd = httpd_Find( VLC_OBJECT(p_access), VLC_TRUE );
     if( !p_sys->p_httpd )
@@ -214,10 +222,18 @@ static int Open( vlc_object_t *p_this )
         return( VLC_EGENERIC );
     }
 
+    if( p_access->psz_access && !strcmp( p_access->psz_access, "mmsh" ) )
+    {
+        psz_mime = "video/x-ms-asf-stream";
+    }
+    else
+    {
+        psz_mime = GetMime( psz_file_name );
+    }
     p_sys->p_httpd_stream =
         p_sys->p_httpd->pf_register_stream( p_sys->p_httpd,
                                             psz_file_name,
-                                            GetMime( psz_file_name ),
+                                            psz_mime,
                                             sout_cfg_find_value( p_access->p_cfg, "user" ),
                                             sout_cfg_find_value( p_access->p_cfg, "pwd" ) );
 
@@ -234,6 +250,9 @@ static int Open( vlc_object_t *p_this )
         return( VLC_EGENERIC );
     }
 
+    free( psz_file_name );
+    free( psz_name );
+
     p_sys->i_header_allocated = 1024;
     p_sys->i_header_size      = 0;
     p_sys->p_header           = malloc( p_sys->i_header_allocated );
@@ -286,7 +305,7 @@ static int Write( sout_access_out_t *p_access, sout_buffer_t *p_buffer )
                 p_sys->i_header_size = 0;
                 p_sys->b_header_complete = VLC_FALSE;
             }
-            if( p_buffer->i_size + p_sys->i_header_size > p_sys->i_header_allocated )
+            if( (int)(p_buffer->i_size + p_sys->i_header_size) > p_sys->i_header_allocated )
             {
                 p_sys->i_header_allocated = p_buffer->i_size + p_sys->i_header_size + 1024;
                 p_sys->p_header = realloc( p_sys->p_header, p_sys->i_header_allocated );
@@ -337,7 +356,7 @@ static int Write( sout_access_out_t *p_access, sout_buffer_t *p_buffer )
  *****************************************************************************/
 static int Seek( sout_access_out_t *p_access, off_t i_pos )
 {
-    msg_Err( p_access, "http sout access cannot seek" );
-    return( VLC_EGENERIC );
+    msg_Warn( p_access, "HTTP sout access cannot seek" );
+    return VLC_EGENERIC;
 }
 
