@@ -2,7 +2,7 @@
  * alsa.c : alsa plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000-2001 VideoLAN
- * $Id: alsa.c,v 1.33 2003/07/27 16:20:53 gbazin Exp $
+ * $Id: alsa.c,v 1.40 2004/02/20 19:21:23 gbazin Exp $
  *
  * Authors: Henri Fallon <henri@videolan.org> - Original Author
  *          Jeffrey Baker <jwbaker@acm.org> - Port to ALSA 1.0 API
@@ -63,6 +63,8 @@ struct aout_sys_t
 
     vlc_mutex_t lock;
     vlc_cond_t  wait ;
+
+    snd_pcm_status_t *p_status;
 };
 
 #define A52_FRAME_NB 1536
@@ -70,16 +72,16 @@ struct aout_sys_t
 /* These values are in frames.
    To convert them to a number of bytes you have to multiply them by the
    number of channel(s) (eg. 2 for stereo) and the size of a sample (eg.
-   2 for s16). */
+   2 for int16_t). */
 #define ALSA_DEFAULT_PERIOD_SIZE        1024
-#define ALSA_DEFAULT_BUFFER_SIZE        ( ALSA_DEFAULT_PERIOD_SIZE << 4 )
+#define ALSA_DEFAULT_BUFFER_SIZE        ( ALSA_DEFAULT_PERIOD_SIZE << 8 )
 #define ALSA_SPDIF_PERIOD_SIZE          A52_FRAME_NB
 #define ALSA_SPDIF_BUFFER_SIZE          ( ALSA_SPDIF_PERIOD_SIZE << 4 )
 /* Why << 4 ? --Meuuh */
 /* Why not ? --Bozo */
 /* Right. --Meuuh */
 
-#define DEFAULT_ALSA_DEVICE "default"
+#define DEFAULT_ALSA_DEVICE N_("default")
 
 /*****************************************************************************
  * Local prototypes
@@ -94,10 +96,9 @@ static void ALSAFill     ( aout_instance_t * );
  * Module descriptor
  *****************************************************************************/
 vlc_module_begin();
-    add_category_hint( N_("ALSA"), NULL, VLC_FALSE );
-    add_string( "alsadev", DEFAULT_ALSA_DEVICE, aout_FindAndRestart,
-                N_("ALSA device name"), NULL, VLC_FALSE );
     set_description( _("ALSA audio output") );
+    add_string( "alsadev", DEFAULT_ALSA_DEVICE, aout_FindAndRestart,
+                N_("ALSA Device Name"), NULL, VLC_FALSE );
     set_capability( "audio output", 150 );
     set_callbacks( Open, Close );
 vlc_module_end();
@@ -107,14 +108,14 @@ vlc_module_end();
  *****************************************************************************/
 static void Probe( aout_instance_t * p_aout,
                    const char * psz_device, const char * psz_iec_device,
-                   int i_snd_pcm_format )
+                   int *pi_snd_pcm_format )
 {
     struct aout_sys_t * p_sys = p_aout->output.p_sys;
     vlc_value_t val, text;
     int i_ret;
 
     var_Create ( p_aout, "audio-device", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE );
-    text.psz_string = _("Audio device");
+    text.psz_string = _("Audio Device");
     var_Change( p_aout, "audio-device", VLC_VAR_SETTEXT, &text, NULL );
 
     /* We'll open the audio device in non blocking mode so we can just exit
@@ -140,29 +141,27 @@ static void Probe( aout_instance_t * p_aout,
         }
 
         if ( snd_pcm_hw_params_set_format( p_sys->p_snd_pcm, p_hw,
-                                           i_snd_pcm_format ) < 0 )
+                                           *pi_snd_pcm_format ) < 0 )
         {
-            /* Assume a FPU enabled computer can handle float32 format.
-               If somebody tells us it's not always true then we'll have
-               to change this */
-            msg_Warn( p_aout, "unable to set stream sample size and word order"
-                              ", disabling linear PCM audio" );
-            snd_pcm_close( p_sys->p_snd_pcm );
-            var_Destroy( p_aout, "audio-device" );
-            return;
+            if( *pi_snd_pcm_format != SND_PCM_FORMAT_S16 )
+            {
+                *pi_snd_pcm_format = SND_PCM_FORMAT_S16;
+                if ( snd_pcm_hw_params_set_format( p_sys->p_snd_pcm, p_hw,
+                                                   *pi_snd_pcm_format ) < 0 )
+                {
+                    msg_Warn( p_aout, "unable to set stream sample size and "
+                              "word order, disabling linear PCM audio" );
+                    snd_pcm_close( p_sys->p_snd_pcm );
+                    var_Destroy( p_aout, "audio-device" );
+                    return;
+                }
+            }
         }
 
         i_channels = aout_FormatNbChannels( &p_aout->output.output );
 
         while ( i_channels > 0 )
         {
-            /* Here we have to probe multi-channel capabilities but I have
-               no idea (at the moment) of how its managed by the ALSA
-               library.
-               It seems that '6' channels aren't well handled on a stereo
-               sound card like my i810 but it requires some more
-               investigations. That's why '4' and '6' cases are disabled.
-               -- Bozo */
             if ( !snd_pcm_hw_params_test_channels( p_sys->p_snd_pcm, p_hw,
                                                    i_channels ) )
             {
@@ -285,6 +284,7 @@ static int Open( vlc_object_t *p_this )
     }
     p_sys->b_playing = VLC_FALSE;
     p_sys->start_date = 0;
+    p_sys->p_status = (snd_pcm_status_t *)malloc(snd_pcm_status_sizeof());
     vlc_cond_init( p_aout, &p_sys->wait );
     vlc_mutex_init( p_aout, &p_sys->lock );
 
@@ -339,7 +339,16 @@ static int Open( vlc_object_t *p_this )
        and we have to probe the available audio formats and channels */
     if ( var_Type( p_aout, "audio-device" ) == 0 )
     {
-        Probe( p_aout, psz_device, psz_iec_device, i_snd_pcm_format );
+        Probe( p_aout, psz_device, psz_iec_device, &i_snd_pcm_format );
+        switch( i_snd_pcm_format )
+        {
+        case SND_PCM_FORMAT_FLOAT:
+            i_vlc_pcm_format = VLC_FOURCC('f','l','3','2');
+            break;
+        case SND_PCM_FORMAT_S16:
+            i_vlc_pcm_format = AOUT_FMT_S16_NE;
+            break;
+        }
     }
 
     if ( var_Get( p_aout, "audio-device", &val ) < 0 )
@@ -360,6 +369,8 @@ static int Open( vlc_object_t *p_this )
             = AOUT_CHAN_LEFT | AOUT_CHAN_RIGHT | AOUT_CHAN_CENTER
                | AOUT_CHAN_REARLEFT | AOUT_CHAN_REARRIGHT
                | AOUT_CHAN_LFE;
+        free( psz_device );
+        psz_device = strdup( "surround51" );
     }
     else if ( val.i_int == AOUT_VAR_2F2R )
     {
@@ -367,6 +378,8 @@ static int Open( vlc_object_t *p_this )
         p_aout->output.output.i_physical_channels
             = AOUT_CHAN_LEFT | AOUT_CHAN_RIGHT
                | AOUT_CHAN_REARLEFT | AOUT_CHAN_REARRIGHT;
+        free( psz_device );
+        psz_device = strdup( "surround40" );
     }
     else if ( val.i_int == AOUT_VAR_STEREO )
     {
@@ -416,6 +429,8 @@ static int Open( vlc_object_t *p_this )
     }
     else
     {
+        msg_Dbg( p_aout, "opening ALSA device `%s'", psz_device );
+
         if ( ( i_snd_rc = snd_pcm_open( &p_sys->p_snd_pcm, psz_device,
                             SND_PCM_STREAM_PLAYBACK, 0 ) ) < 0 )
         {
@@ -632,6 +647,7 @@ static void Close( vlc_object_t *p_this )
     snd_output_close( p_sys->p_snd_stderr );
 #endif
 
+    free( p_sys->p_status );
     free( p_sys );
 }
 
@@ -665,13 +681,11 @@ static void ALSAFill( aout_instance_t * p_aout )
     struct aout_sys_t * p_sys = p_aout->output.p_sys;
 
     aout_buffer_t * p_buffer;
-    snd_pcm_status_t * p_status;
+    snd_pcm_status_t * p_status = p_sys->p_status;
     snd_timestamp_t ts_next;
     int i_snd_rc;
     mtime_t next_date;
 
-    snd_pcm_status_alloca( &p_status );
-
     /* Fill in the buffer until space or audio output buffer shortage */
     {
         /* Get the status */
@@ -713,15 +727,20 @@ static void ALSAFill( aout_instance_t * p_aout )
                 msleep( p_sys->i_period_time >> 1 );
                 return;
             }
+
+            /* Underrun, try to recover as quickly as possible */
+            next_date = mdate();
         }
+        else
+        {
+            /* Here the device should be either in the RUNNING state.
+             * p_status is valid. */
 
-        /* Here the device should be either in the RUNNING state either in
-           the PREPARE state. p_status is valid. */
-
-        snd_pcm_status_get_tstamp( p_status, &ts_next );
-        next_date = (mtime_t)ts_next.tv_sec * 1000000 + ts_next.tv_usec;
-        next_date += (mtime_t)snd_pcm_status_get_delay(p_status)
-                     * 1000000 / p_aout->output.output.i_rate;
+            snd_pcm_status_get_tstamp( p_status, &ts_next );
+            next_date = (mtime_t)ts_next.tv_sec * 1000000 + ts_next.tv_usec;
+            next_date += (mtime_t)snd_pcm_status_get_delay(p_status)
+                * 1000000 / p_aout->output.output.i_rate;
+        }
 
         p_buffer = aout_OutputNextBuffer( p_aout, next_date,
                         (p_aout->output.output.i_format ==
