@@ -1,8 +1,8 @@
 /*****************************************************************************
  * http.c :  http mini-server ;)
  *****************************************************************************
- * Copyright (C) 2001 VideoLAN
- * $Id: http.c,v 1.41 2003/12/21 23:32:58 sam Exp $
+ * Copyright (C) 2001-2004 VideoLAN
+ * $Id: http.c,v 1.53 2004/02/09 23:30:37 garf Exp $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *          Laurent Aimar <fenrir@via.ecp.fr>
@@ -71,13 +71,12 @@ static void Close        ( vlc_object_t * );
 
 #define HOST_TEXT N_( "Host address" )
 #define HOST_LONGTEXT N_( \
-    "You can set the address and port on which the http interface will bind" )
+    "You can set the address and port the http interface will bind to." )
 #define SRC_TEXT N_( "Source directory" )
 #define SRC_LONGTEXT N_( "Source directory" )
 
 vlc_module_begin();
     set_description( _("HTTP remote control interface") );
-    add_category_hint( N_("HTTP remote control"), NULL, VLC_TRUE );
         add_string ( "http-host", NULL, NULL, HOST_TEXT, HOST_LONGTEXT, VLC_TRUE );
         add_string ( "http-src",  NULL, NULL, SRC_TEXT,  SRC_LONGTEXT,  VLC_TRUE );
     set_capability( "interface", 0 );
@@ -126,7 +125,7 @@ static void uri_decode_url_encoded( char *psz );
 
 static char *Find_end_MRL( char *psz );
 
-static playlist_item_t * parse_MRL( char *psz );
+static playlist_item_t * parse_MRL( intf_thread_t * , char *psz );
 
 /*****************************************************************************
  *
@@ -297,10 +296,12 @@ static int Activate( vlc_object_t *p_this )
         goto failed;
     }
     p_intf->pf_run = Run;
+    free( psz_src );
 
     return VLC_SUCCESS;
 
 failed:
+    if( psz_src ) free( psz_src );
     free( p_sys->pp_files );
     p_sys->p_httpd->pf_unregister_host( p_sys->p_httpd,
                                         p_sys->p_httpd_host );
@@ -584,7 +585,8 @@ static int ParseDirectory( intf_thread_t *p_intf, char *psz_root,
             f = malloc( sizeof( httpd_file_callback_args_t ) );
             if( !f )
             {
-                msg_Err( p_intf, "Out of memory" );
+                msg_Err( p_intf, "out of memory" );
+                closedir( p_dir );
                 return( VLC_ENOMEM );
             }
             f->p_intf  = p_intf;
@@ -594,7 +596,9 @@ static int ParseDirectory( intf_thread_t *p_intf, char *psz_root,
 
             if( !f->name || !f->mime )
             {
-                msg_Err( p_intf , "Unable to parse directory" );
+                msg_Err( p_intf , "unable to parse directory" );
+                closedir( p_dir );
+                free( f );
                 return( VLC_ENOMEM );
             }
             msg_Dbg( p_intf, "file=%s (url=%s mime=%s)",
@@ -621,7 +625,8 @@ static int ParseDirectory( intf_thread_t *p_intf, char *psz_root,
                 f = malloc( sizeof( httpd_file_callback_args_t ) );
                 if( !f )
                 {
-                    msg_Err( p_intf, "Out of memory" );
+                    msg_Err( p_intf, "out of memory" );
+                    closedir( p_dir );
                     return( VLC_ENOMEM );
                 }
                 f->p_intf  = p_intf;
@@ -659,6 +664,9 @@ static int ParseDirectory( intf_thread_t *p_intf, char *psz_root,
     {
         free( password );
     }
+
+    closedir( p_dir );
+
     return VLC_SUCCESS;
 }
 
@@ -1017,6 +1025,10 @@ static mvar_t *mvar_HttpdInfoSetNew( char *name, httpd_t *p_httpd, int i_type )
         free( info.info[i].psz_name );
         free( info.info[i].psz_value );
     }
+    if( info.i_count > 0 )
+    {
+        free( info.info );
+    }
 
     return s;
 }
@@ -1752,7 +1764,7 @@ static void MacroDo( httpd_file_callback_args_t *p_args,
 
                     uri_extract_value( p_request, "mrl", mrl, 512 );
                     uri_decode_url_encoded( mrl );
-                    p_item = parse_MRL( mrl );
+                    p_item = parse_MRL( p_intf, mrl );
 
                     if( !p_item || !p_item->psz_uri || !*p_item->psz_uri )
                     {
@@ -1864,8 +1876,8 @@ static void MacroDo( httpd_file_callback_args_t *p_args,
                     uri_extract_value( p_request, "type", type, 12 );
                     uri_extract_value( p_request, "order", order, 2 );
 
-                    if( order[0] == '0' ) i_order = SORT_NORMAL;
-                    else i_order = SORT_REVERSE;
+                    if( order[0] == '0' ) i_order = ORDER_NORMAL;
+                    else i_order = ORDER_REVERSE;
 
                     if( !strcmp( type , "title" ) )
                     {
@@ -1966,6 +1978,7 @@ static void MacroDo( httpd_file_callback_args_t *p_args,
                 case MVLC_STRING:
                     psz = config_GetPsz( p_intf, m->param1 );
                     sprintf( value, "%s", psz ? psz : "" );
+                    if( psz ) free( psz );
                     break;
                 default:
                     sprintf( value, "invalid type(%s) in set", m->param2 );
@@ -2037,6 +2050,8 @@ static uint8_t *MacroSearch( uint8_t *src, uint8_t *end, int i_mvlc, vlc_bool_t
                     break;
             }
 
+            MacroClean( &m );
+
             if( ( i_mvlc == MVLC_END && i_level == -1 ) ||
                 ( i_mvlc != MVLC_END && i_level == 0 && i_mvlc == i_id ) )
             {
@@ -2274,7 +2289,7 @@ static int  http_get( httpd_file_callback_args_t *p_args,
         p += sprintf( p, "</body>\n" );
         p += sprintf( p, "</html>\n" );
 
-        *pi_data = strlen( *pp_data ) + 1;
+        *pi_data = strlen( *pp_data );
 
         return VLC_SUCCESS;
     }
@@ -2352,11 +2367,12 @@ static int  http_get( httpd_file_callback_args_t *p_args,
         /* we parse executing all  <vlc /> macros */
         Execute( p_args, p_request, i_request, pp_data, pi_data, &dst, &p_buffer[0], &p_buffer[i_buffer] );
 
-        *dst++ = '\0';
+        *dst     = '\0';
         *pi_data = dst - *pp_data;
 
         SSClean( &p_args->stack );
         mvar_Delete( p_args->vars );
+        free( p_buffer );
     }
 
     fclose( f );
@@ -2370,9 +2386,16 @@ static int  http_get( httpd_file_callback_args_t *p_args,
 static char *uri_extract_value( char *psz_uri, char *psz_name,
                                 char *psz_value, int i_value_max )
 {
-    char *p;
+    char *p = psz_uri;
+
+    while( (p = strstr( p, psz_name )) )
+    {
+        /* Verify that we are dealing with a post/get argument */
+        if( p == psz_uri || *(p - 1) == '&' || *(p - 1) == '\n' )
+            break;
+        p++;
+    }
 
-    p = strstr( psz_uri, psz_name );
     if( p )
     {
         int i_len;
@@ -2682,7 +2705,33 @@ static void  EvaluateRPN( mvar_t  *vars, rpn_stack_t *st, char *exp )
             free( s1 );
             free( s2 );
         }
-        else if( !strcmp( s, "strlen" ) )
+        else if( !strcmp( s, "strsub" ) )
+        {
+            int n = SSPopN( st, vars );
+            int m = SSPopN( st, vars );
+            int i_len;
+            char *s = SSPop( st );
+            char *str;
+
+            if( n >= m )
+            {
+                i_len = n - m + 1;
+            }
+            else
+            {
+                i_len = 0;
+            }
+
+            str = malloc( i_len + 1 );
+
+            memcpy( str, s + m - 1, i_len );
+            str[ i_len ] = '\0';
+
+            SSPush( st, str );
+            free( s );
+            free( str );
+        }
+       else if( !strcmp( s, "strlen" ) )
         {
             char *str = SSPop( st );
 
@@ -2847,7 +2896,7 @@ static char *Find_end_MRL( char *psz )
  * create an item with all informations in it, and return the item.
  * return NULL if there is an error.
  **********************************************************************/
-playlist_item_t * parse_MRL( char *psz )
+playlist_item_t * parse_MRL( intf_thread_t *p_intf, char *psz )
 {
     char **ppsz_options = NULL;
     char *mrl;
@@ -2856,7 +2905,7 @@ playlist_item_t * parse_MRL( char *psz )
     char *s_temp;
     int i = 0;
     int i_options = 0;
-    playlist_item_t * p_item;
+    playlist_item_t * p_item = NULL;
 
     /* In case there is spaces before the mrl */
     while( ( *s_mrl == ' ' ) && ( *s_mrl != '\0' ) )
@@ -2910,13 +2959,18 @@ playlist_item_t * parse_MRL( char *psz )
                 if( s_temp == NULL )
                 {
                     i_error = 1;
-                } else
+                }
+                else
                 {
                     i_options++;
-                    ppsz_options = (char **)realloc( ppsz_options , i_options * sizeof(char *) );
-                    ppsz_options[ i_options - 1 ] = (char *)malloc( (s_temp - s_mrl + 1) * sizeof( char ) );
+                    ppsz_options = realloc( ppsz_options , i_options *
+                                            sizeof(char *) );
+                    ppsz_options[ i_options - 1 ] =
+                        malloc( (s_temp - s_mrl + 1) * sizeof(char) );
+
+                    strncpy( ppsz_options[ i_options - 1 ] , s_mrl ,
+                             s_temp - s_mrl );
 
-                    strncpy( ppsz_options[ i_options - 1 ] , s_mrl , s_temp - s_mrl );
                     /* don't forget to finish the string with a '\0' */
                     (ppsz_options[ i_options - 1 ])[ s_temp - s_mrl ] = '\0';
 
@@ -2935,35 +2989,22 @@ playlist_item_t * parse_MRL( char *psz )
     if( i_error != 0 )
     {
         free( mrl );
-        for( i = 0 ; i < i_options ; i++ )
-        {
-            free( ppsz_options[i] );
-        }
-        free( ppsz_options );
-        return NULL;
-    } else
+    }
+    else
     {
         /* now create an item */
-        p_item = malloc( sizeof( playlist_item_t ) );
-
-        p_item->psz_name   = mrl;
-        p_item->psz_uri    = strdup( mrl );
-        p_item->psz_author = strdup( "" );
-        p_item->i_duration = -1;
-        p_item->i_type = 0;
-        p_item->i_status = 0;
-        p_item->b_autodeletion = VLC_FALSE;
-        p_item->b_enabled = VLC_TRUE;
-        p_item->i_group = PLAYLIST_TYPE_MANUAL;
-
-        p_item->ppsz_options = NULL;
-        p_item->i_options = i_options;
-
-        if( i_options )
+        p_item = playlist_ItemNew( p_intf, mrl, mrl);
+        for( i = 0 ; i< i_options ; i++ )
         {
-            p_item->ppsz_options = ppsz_options;
+            playlist_ItemAddOption( p_item, ppsz_options[i] );
         }
-
-        return p_item;
     }
+
+    for( i = 0 ; i < i_options ; i++ )
+    {
+        free( ppsz_options[i] );
+    }
+    free( ppsz_options );
+
+    return p_item;
 }
