@@ -1,8 +1,8 @@
 /*****************************************************************************
  * ps.c : Program Stream input module for vlc
  *****************************************************************************
- * Copyright (C) 2000-2001 VideoLAN
- * $Id: ps.c,v 1.13 2003/12/22 14:32:56 sam Exp $
+ * Copyright (C) 2000-2004, 2004 VideoLAN
+ * $Id: ps.c,v 1.17 2004/01/25 20:05:28 hartman Exp $
  *
  * Author: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -263,6 +263,14 @@ static int Activate( vlc_object_t * p_this )
                         }
                         break;
 
+                    case VLC_FOURCC('o','g','t',' '):
+                        if( config_GetInt( p_input, "spu-channel" )
+			    == (p_es->i_id & 0x0003) )
+                        {
+                            input_SelectES( p_input, p_es );
+                        }
+                        break;
+
                     case VLC_FOURCC('l','p','c','m'):
                     case VLC_FOURCC('l','p','c','b'):
                         if( config_GetInt( p_input, "audio-channel" )
