@@ -1,8 +1,8 @@
 /*****************************************************************************
  * id3tag.c: id3 tag parser/skipper based on libid3tag
  *****************************************************************************
- * Copyright (C) 2001 VideoLAN
- * $Id: id3tag.c,v 1.17 2003/12/22 22:37:01 hartman Exp $
+ * Copyright (C) 2002-2004 VideoLAN
+ * $Id: id3tag.c,v 1.21 2004/02/14 01:53:17 gbazin Exp $
  *
  * Authors: Sigmund Augdal <sigmunau@idi.ntnu.no>
  *
@@ -47,7 +47,7 @@ static int  ParseID3Tags ( vlc_object_t * );
  * Module descriptor
  *****************************************************************************/
 vlc_module_begin();
-set_description( _("id3 tag parser using libid3tag" ) );
+set_description( _("ID3 tag parser using libid3tag" ) );
 set_capability( "id3", 70 );
 set_callbacks( ParseID3Tags, NULL );
 vlc_module_end();
@@ -72,7 +72,7 @@ static void ParseID3Tag( input_thread_t *p_input, uint8_t *p_data, int i_size )
     var_Get( p_input, "demuxed-id3", &val );
     if( val.b_bool )
     {
-        msg_Dbg( p_input, "The ID3 tag was already parsed" );
+        msg_Dbg( p_input, "the ID3 tag was already parsed" );
         return;
     }
 
@@ -85,8 +85,17 @@ static void ParseID3Tag( input_thread_t *p_input, uint8_t *p_data, int i_size )
 
     while ( ( p_frame = id3_tag_findframe( p_id3_tag , "T", i ) ) )
     {
-        playlist_item_t *p_item = p_playlist ? p_playlist->pp_items[p_playlist->i_index] : NULL;
-        int i_strings = id3_field_getnstrings( &p_frame->fields[1] );
+        int i_strings;
+        playlist_item_t *p_item = playlist_ItemGetByPos( p_playlist, -1 );
+        if( !p_item )
+        {
+            msg_Err( p_input, "Unable to get item" );
+            return;
+        }
+
+        vlc_mutex_lock( &p_item->lock );
+
+        i_strings = id3_field_getnstrings( &p_frame->fields[1] );
 
         while ( i_strings > 0 )
         {
@@ -100,11 +109,17 @@ static void ParseID3Tag( input_thread_t *p_input, uint8_t *p_data, int i_size )
                 {
                     input_AddInfo( p_category, (char *)p_frame->description,
                                    ppsz_genres[atoi(psz_temp)]);
+                    playlist_ItemAddInfo( p_item, "ID3",
+                                    (char *)p_frame->description,
+                                    ppsz_genres[atoi(psz_temp)]);
                 }
                 else
                 {
                     input_AddInfo( p_category, (char *)p_frame->description,
                                                 psz_temp );
+                    playlist_ItemAddInfo( p_item, "ID3",
+                                    (char *)p_frame->description,
+                                    psz_temp);
                 }
             }
             else if ( !strcmp(p_frame->id, ID3_FRAME_TITLE ) )
@@ -121,38 +136,44 @@ static void ParseID3Tag( input_thread_t *p_input, uint8_t *p_data, int i_size )
                 }
                 input_AddInfo( p_category, (char *)p_frame->description,
                                             psz_temp );
+                playlist_ItemAddInfo( p_item, "ID3",
+                                        (char *)p_frame->description,
+                                    psz_temp);
             }
             else if ( !strcmp(p_frame->id, ID3_FRAME_ARTIST ) )
             {
                 if( p_item )
                 {
-                    if( p_item->psz_author )
-                    {
-                        free( p_item->psz_author );
-                    }
-                    p_item->psz_author = strdup( psz_temp );
-
+                    playlist_ItemAddInfo( p_item,
+                                          _("General"), _("Author"), psz_temp);
                     val.b_bool = VLC_TRUE;
                 }
                 input_AddInfo( p_category, (char *)p_frame->description,
                                             psz_temp );
+                playlist_ItemAddInfo( p_item, "ID3",
+                                           (char *)p_frame->description,
+                                           psz_temp);
             }
             else
             {
                 input_AddInfo( p_category, (char *)p_frame->description,
                                             psz_temp );
+                playlist_ItemAddInfo( p_item, "ID3",
+                                           (char *)p_frame->description,
+                                           psz_temp);
             }
             free( psz_temp );
         }
         i++;
+        vlc_mutex_unlock( &p_item->lock );
     }
     id3_tag_delete( p_id3_tag );
     if(val.b_bool == VLC_TRUE )
     {
         if( p_playlist )
         {
-            val.b_bool = VLC_TRUE;
-            var_Set( p_playlist, "intf-change", val );
+            val.b_bool = p_playlist->i_index;
+            var_Set( p_playlist, "item-change", val );
         }
     }
     val.b_bool = VLC_TRUE;
@@ -181,7 +202,7 @@ static int ParseID3Tags( vlc_object_t *p_this )
     }
     p_input = (input_thread_t *)p_this;
 
-    msg_Dbg( p_input, "Checking for ID3 tag" );
+    msg_Dbg( p_input, "checking for ID3 tag" );
 
     if ( p_input->stream.b_seekable &&
          p_input->stream.i_method != INPUT_METHOD_NETWORK )
@@ -212,7 +233,7 @@ static int ParseID3Tags( vlc_object_t *p_this )
                     msg_Err( p_input, "cannot peek()" );
                     return( VLC_EGENERIC );
                 }
-                msg_Dbg( p_input, "Found ID3v1 tag" );
+                msg_Dbg( p_input, "found ID3v1 tag" );
                 ParseID3Tag( p_input, p_peek, i_size2 );
             }
 
@@ -234,7 +255,7 @@ static int ParseID3Tags( vlc_object_t *p_this )
                     msg_Err( p_input, "cannot peek()" );
                     return( VLC_EGENERIC );
                 }
-                msg_Dbg( p_input, "Found ID3v2 tag at end of file" );
+                msg_Dbg( p_input, "found ID3v2 tag at end of file" );
                 ParseID3Tag( p_input, p_peek, i_size2 );
             }
         }
@@ -258,13 +279,13 @@ static int ParseID3Tags( vlc_object_t *p_this )
     p_peek = malloc( i_size );
     if( !p_peek || stream_Read( p_input->s, p_peek, i_size ) < i_size )
     {
-        msg_Err( p_input, "Cannot read ID3 tag" );
+        msg_Err( p_input, "cannot read ID3 tag" );
         if( p_peek ) free( p_peek );
         return( VLC_EGENERIC );
     }
 
     ParseID3Tag( p_input, p_peek, i_size );
-    msg_Dbg( p_input, "Found ID3v2 tag" );
+    msg_Dbg( p_input, "found ID3v2 tag" );
 
     free( p_peek );
     return( VLC_SUCCESS );
