@@ -1,10 +1,11 @@
 /*****************************************************************************
- * sub.c
+ * sub.c: subtitle demux for external subtitle files
  *****************************************************************************
- * Copyright (C) 1999-2003 VideoLAN
- * $Id: sub.c,v 1.39 2003/11/27 04:11:40 fenrir Exp $
+ * Copyright (C) 1999-2004 VideoLAN
+ * $Id: sub.c,v 1.52 2004/02/22 15:59:53 fenrir Exp $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
+ *          Derk-Jan Hartman <hartman at videolan dot org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,13 +41,14 @@
 #    include <dirent.h>
 #endif
 
+#define DVD_VIDEO_LB_LEN 2048
+
 static int  Open ( vlc_object_t *p_this );
 
 static int  sub_open ( subtitle_demux_t *p_sub,
                        input_thread_t  *p_input,
                        char  *psz_name,
-                       mtime_t i_microsecperframe,
-                       int i_track_id );
+                       mtime_t i_microsecperframe );
 static int  sub_demux( subtitle_demux_t *p_sub, mtime_t i_maxdate );
 static int  sub_seek ( subtitle_demux_t *p_sub, mtime_t i_date );
 static void sub_close( subtitle_demux_t *p_sub );
@@ -71,16 +73,15 @@ static char *ppsz_sub_type[] = { "auto", "microdvd", "subrip", "ssa1",
 vlc_module_begin();
     set_description( _("Text subtitles demux") );
     set_capability( "subtitle demux", 12 );
-    add_category_hint( "Subtitles", NULL, VLC_TRUE );
-        add_float( "sub-fps", 0.0, NULL,
-                   "Frames per second",
-                   SUB_FPS_LONGTEXT, VLC_TRUE );
-        add_integer( "sub-delay", 0, NULL,
-                     "Delay subtitles (in 1/10s)",
-                     SUB_DELAY_LONGTEXT, VLC_TRUE );
-        add_string( "sub-type", "auto", NULL, "subtitles type",
-                    SUB_TYPE_LONGTEXT, VLC_TRUE );
-            change_string_list( ppsz_sub_type, 0, 0 );
+    add_float( "sub-fps", 25.0, NULL,
+               N_("Frames per second"),
+               SUB_FPS_LONGTEXT, VLC_TRUE );
+    add_integer( "sub-delay", 0, NULL,
+                 N_("Delay subtitles (in 1/10s)"),
+                 SUB_DELAY_LONGTEXT, VLC_TRUE );
+    add_string( "sub-type", "auto", NULL, "Subtitles fileformat",
+                SUB_TYPE_LONGTEXT, VLC_TRUE );
+        change_string_list( ppsz_sub_type, 0, 0 );
     set_callbacks( Open, NULL );
 vlc_module_end();
 
@@ -95,7 +96,7 @@ static int Open ( vlc_object_t *p_this )
     p_sub->pf_demux = sub_demux;
     p_sub->pf_seek  = sub_seek;
     p_sub->pf_close = sub_close;
-
+    
     /* Initialize the variables */
     var_Create( p_this, "sub-fps", VLC_VAR_FLOAT | VLC_VAR_DOINHERIT );
     var_Create( p_this, "sub-delay", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
@@ -209,7 +210,9 @@ static int  sub_SubRipRead  ( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *
 static int  sub_SSARead     ( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
 static int  sub_Vplayer     ( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
 static int  sub_Sami        ( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
-static int  sub_VobSub      ( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
+static int  sub_VobSubIDX   ( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe );
+
+static int  DemuxVobSub     ( subtitle_demux_t *, block_t * );
 
 static struct
 {
@@ -225,7 +228,7 @@ static struct
     { "ssa2-4",     SUB_TYPE_SSA2_4,    "SSA-2/3/4",sub_SSARead },
     { "vplayer",    SUB_TYPE_VPLAYER,   "VPlayer",  sub_Vplayer },
     { "sami",       SUB_TYPE_SAMI,      "SAMI",     sub_Sami },
-    { "vobsub",     SUB_TYPE_VOBSUB,    "VobSub",   sub_VobSub },
+    { "vobsub",     SUB_TYPE_VOBSUB,    "VobSub",   sub_VobSubIDX },
     { NULL,         SUB_TYPE_UNKNOWN,   "Unknown",  NULL }
 };
 
@@ -256,31 +259,26 @@ static char * local_stristr( char *psz_big, char *psz_little)
 /*****************************************************************************
  * sub_open: Open a subtitle file and add subtitle ES
  *****************************************************************************/
-static int  sub_open ( subtitle_demux_t *p_sub,
-                       input_thread_t  *p_input,
-                       char     *psz_name,
-                       mtime_t i_microsecperframe,
-                       int i_track_id )
+static int sub_open( subtitle_demux_t *p_sub, input_thread_t  *p_input,
+                     char *psz_name, mtime_t i_microsecperframe )
 {
     text_t  txt;
     vlc_value_t val;
-    es_format_t  fmt;
-
-    int     i;
-    int     i_sub_type;
-    int     i_max;
-    int (*pf_read_subtitle)( subtitle_demux_t *, text_t *, subtitle_t *, mtime_t ) = NULL;
+    es_format_t fmt;
+    int i, i_sub_type, i_max;
+    int (*pf_read_subtitle)( subtitle_demux_t *, text_t *, subtitle_t *,
+                             mtime_t ) = NULL;
 
     p_sub->i_sub_type = SUB_TYPE_UNKNOWN;
     p_sub->p_es = NULL;
     p_sub->i_subtitles = 0;
     p_sub->subtitle = NULL;
+    p_sub->p_vobsub_file = 0;
     p_sub->p_input = p_input;
 
     if( !psz_name || !*psz_name )
     {
         msg_Err( p_sub, "no subtitle file specified" );
-        if( psz_name ) free( psz_name );
         return VLC_EGENERIC;
     }
 
@@ -288,19 +286,17 @@ static int  sub_open ( subtitle_demux_t *p_sub,
     if( text_load( &txt, psz_name ) )
     {
         msg_Err( p_sub, "cannot open `%s' subtitle file", psz_name );
-        free( psz_name );
         return VLC_EGENERIC;
     }
+
     msg_Dbg( p_sub, "opened `%s'", psz_name );
-    free( psz_name );
 
     var_Get( p_sub, "sub-fps", &val );
     if( val.i_int >= 1.0 )
     {
-        var_Get( p_sub, "sub-fps", &val );
         i_microsecperframe = (mtime_t)( (float)1000000 / val.f_float );
     }
-    else if( i_microsecperframe <= 0 )
+    else if( val.f_float <= 0 )
     {
         i_microsecperframe = 40000; /* default: 25fps */
     }
@@ -431,22 +427,11 @@ static int  sub_open ( subtitle_demux_t *p_sub,
         if( p_sub->i_subtitles >= i_max )
         {
             i_max += 128;
-            if( p_sub->subtitle )
+            if( !( p_sub->subtitle = realloc( p_sub->subtitle,
+                                              sizeof(subtitle_t) * i_max ) ) )
             {
-                if( !( p_sub->subtitle = realloc( p_sub->subtitle,
-                                           sizeof( subtitle_t ) * i_max ) ) )
-                {
-                    msg_Err( p_sub, "out of memory");
-                    return VLC_ENOMEM;
-                }
-            }
-            else
-            {
-                if( !(  p_sub->subtitle = malloc( sizeof( subtitle_t ) * i_max ) ) )
-                {
-                    msg_Err( p_sub, "out of memory");
-                    return VLC_ENOMEM;
-                }
+                msg_Err( p_sub, "out of memory");
+                return VLC_ENOMEM;
             }
         }
         if( pf_read_subtitle( p_sub, &txt,
@@ -464,6 +449,7 @@ static int  sub_open ( subtitle_demux_t *p_sub,
 
     /* *** fix subtitle (order and time) *** */
     p_sub->i_subtitle = 0;  /* will be modified by sub_fix */
+    
     if( p_sub->i_sub_type != SUB_TYPE_VOBSUB )
     {
         sub_fix( p_sub );
@@ -472,6 +458,18 @@ static int  sub_open ( subtitle_demux_t *p_sub,
     /* *** add subtitle ES *** */
     if( p_sub->i_sub_type == SUB_TYPE_VOBSUB )
     {
+        int i_len = strlen( psz_name );
+        char *psz_vobname = strdup(psz_name);
+
+        strcpy( psz_vobname + i_len - 4, ".sub" );
+
+        /* open file */
+        if( !( p_sub->p_vobsub_file = fopen( psz_vobname, "rb" ) ) )
+        {
+            msg_Err( p_sub, "couldn't open .sub Vobsub file: %s", psz_vobname );
+        }
+        free( psz_vobname );
+
         es_format_Init( &fmt, SPU_ES, VLC_FOURCC( 's','p','u',' ' ) );
     }
     else if( p_sub->i_sub_type == SUB_TYPE_SSA1 ||
@@ -489,8 +487,8 @@ static int  sub_open ( subtitle_demux_t *p_sub,
         fmt.p_extra = strdup( p_sub->psz_header );
     }
     p_sub->p_es = es_out_Add( p_input->p_es_out, &fmt );
-
     p_sub->i_previously_selected = 0;
+
     return VLC_SUCCESS;
 }
 
@@ -515,59 +513,107 @@ static int  sub_demux( subtitle_demux_t *p_sub, mtime_t i_maxdate )
         return VLC_SUCCESS;
     }
 
-    while( p_sub->i_subtitle < p_sub->i_subtitles &&
-           p_sub->subtitle[p_sub->i_subtitle].i_start < i_maxdate )
+    if( p_sub->i_sub_type != SUB_TYPE_VOBSUB )
     {
-        block_t *p_block;
-        int i_len;
-
-        i_len = strlen( p_sub->subtitle[p_sub->i_subtitle].psz_text ) + 1;
-
-        if( i_len <= 1 )
+        while( p_sub->i_subtitle < p_sub->i_subtitles &&
+               p_sub->subtitle[p_sub->i_subtitle].i_start < i_maxdate )
         {
-            /* empty subtitle */
-            p_sub->i_subtitle++;
-            continue;
-        }
+            block_t *p_block;
+            int i_len = strlen( p_sub->subtitle[p_sub->i_subtitle].psz_text ) + 1;
 
-        if( !( p_block = block_New( p_sub->p_input, i_len ) ) )
-        {
-            p_sub->i_subtitle++;
-            continue;
-        }
+            if( i_len <= 1 )
+            {
+                /* empty subtitle */
+                p_sub->i_subtitle++;
+                continue;
+            }
 
-        p_block->i_pts =
-            input_ClockGetTS( p_sub->p_input,
-                              p_sub->p_input->stream.p_selected_program,
-                              p_sub->subtitle[p_sub->i_subtitle].i_start*9/100);
-        if( p_sub->subtitle[p_sub->i_subtitle].i_stop > 0 )
-        {
-            /* FIXME kludge ...
-             * i_dts means end of display...
-             */
-            p_block->i_dts =
+            if( ( p_block = block_New( p_sub->p_input, i_len ) ) == NULL )
+            {
+                p_sub->i_subtitle++;
+                continue;
+            }
+
+            /* XXX we should convert all demuxers to use es_out_Control to set pcr and
+             * then remove that */
+            p_block->i_pts =
                 input_ClockGetTS( p_sub->p_input,
-                              p_sub->p_input->stream.p_selected_program,
-                              p_sub->subtitle[p_sub->i_subtitle].i_stop *9/100);
-        }
-        else
-        {
+                                  p_sub->p_input->stream.p_selected_program,
+                                  p_sub->subtitle[p_sub->i_subtitle].i_start*9/100);
             p_block->i_dts = 0;
+            if( p_sub->subtitle[p_sub->i_subtitle].i_stop > 0 )
+            {
+                /* FIXME kludge i_dts means end of display... */
+                p_block->i_dts =
+                    input_ClockGetTS( p_sub->p_input,
+                                  p_sub->p_input->stream.p_selected_program,
+                                  p_sub->subtitle[p_sub->i_subtitle].i_stop *9/100);
+            }
+
+            memcpy( p_block->p_buffer, p_sub->subtitle[p_sub->i_subtitle].psz_text, i_len );
+
+            if( p_block->i_pts > 0 )
+            {
+                es_out_Send( p_input->p_es_out, p_sub->p_es, p_block );
+            }
+            else
+            {
+                block_Release( p_block );
+            }
+            p_sub->i_subtitle++;
         }
-
-        memcpy( p_block->p_buffer,
-                p_sub->subtitle[p_sub->i_subtitle].psz_text, i_len );
-
-        if( p_block->i_pts > 0 )
+    }
+    else
+    {
+        while( p_sub->i_subtitle < p_sub->i_subtitles &&
+               p_sub->subtitle[p_sub->i_subtitle].i_start < i_maxdate )
         {
-            es_out_Send( p_input->p_es_out, p_sub->p_es, p_block );
-        }
-        else
-        {
-            block_Release( p_block );
-        }
+            int i_pos = p_sub->subtitle[p_sub->i_subtitle].i_vobsub_location;
+            block_t *p_block;
+            int i_size = 0;
 
-        p_sub->i_subtitle++;
+            /* first compute SPU size */
+            if( p_sub->i_subtitle + 1 < p_sub->i_subtitles )
+            {
+                i_size = p_sub->subtitle[p_sub->i_subtitle+1].i_vobsub_location - i_pos;
+            }
+            if( i_size <= 0 ) i_size = 65535;   /* Invalid or EOF */
+
+            /* Seek at the right place (could be avoid if sub_seek is fixed to do his job) */
+            if( fseek( p_sub->p_vobsub_file, i_pos, SEEK_SET ) )
+            {
+                msg_Warn( p_sub, "cannot seek at right vobsub location %d", i_pos );
+                p_sub->i_subtitle++;
+                continue;
+            }
+
+            /* allocate a packet */
+            if( ( p_block = block_New( p_sub, i_size ) ) == NULL )
+            {
+                p_sub->i_subtitle++;
+                continue;
+            }
+
+            /* read data */
+            p_block->i_buffer = fread( p_block->p_buffer, 1, i_size, p_sub->p_vobsub_file );
+            if( p_block->i_buffer <= 6 )
+            {
+                block_Release( p_block );
+                p_sub->i_subtitle++;
+                continue;
+            }
+
+            /* pts */
+            p_block->i_pts =
+                input_ClockGetTS( p_sub->p_input,
+                                  p_sub->p_input->stream.p_selected_program,
+                                  p_sub->subtitle[p_sub->i_subtitle].i_start*9/100);
+
+            /* demux this block */
+            DemuxVobSub( p_sub, p_block );
+
+            p_sub->i_subtitle++;
+        }
     }
     return VLC_SUCCESS;
 }
@@ -604,6 +650,10 @@ static void sub_close( subtitle_demux_t *p_sub )
         }
         free( p_sub->subtitle );
     }
+    if( p_sub->p_vobsub_file )
+    {
+        fclose( p_sub->p_vobsub_file );
+    }
 }
 
 /*****************************************************************************
@@ -677,9 +727,14 @@ static int  sub_MicroDvdRead( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *
     char *s;
 
     char buffer_text[MAX_LINE + 1];
-    uint32_t    i_start;
-    uint32_t    i_stop;
+    unsigned int    i_start;
+    unsigned int    i_stop;
     unsigned int i;
+    
+    p_subtitle->i_start = 0;
+    p_subtitle->i_stop  = 0;
+    p_subtitle->i_vobsub_location  = 0;
+    p_subtitle->psz_text = NULL;
 
     for( ;; )
     {
@@ -728,6 +783,11 @@ static int  sub_SubRipRead( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_
     mtime_t     i_start;
     mtime_t     i_stop;
 
+    p_subtitle->i_start = 0;
+    p_subtitle->i_stop  = 0;
+    p_subtitle->i_vobsub_location  = 0;
+    p_subtitle->psz_text = NULL;
+
     for( ;; )
     {
         int h1, m1, s1, d1, h2, m2, s2, d2;
@@ -795,6 +855,11 @@ static int  sub_SSARead( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_sub
     mtime_t     i_start;
     mtime_t     i_stop;
 
+    p_subtitle->i_start = 0;
+    p_subtitle->i_stop  = 0;
+    p_subtitle->i_vobsub_location  = 0;
+    p_subtitle->psz_text = NULL;
+
     for( ;; )
     {
         int h1, m1, s1, c1, h2, m2, s2, c2;
@@ -878,6 +943,11 @@ static int  sub_Vplayer( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_sub
     char buffer_text[MAX_LINE + 1];
     mtime_t    i_start;
     unsigned int i;
+    
+    p_subtitle->i_start = 0;
+    p_subtitle->i_stop  = 0;
+    p_subtitle->i_vobsub_location  = 0;
+    p_subtitle->psz_text = NULL;
 
     for( ;; )
     {
@@ -954,6 +1024,12 @@ static int  sub_Sami( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_subtit
 
     int  i_text;
     char buffer_text[10*MAX_LINE + 1];
+
+    p_subtitle->i_start = 0;
+    p_subtitle->i_stop  = 0;
+    p_subtitle->i_vobsub_location  = 0;
+    p_subtitle->psz_text = NULL;
+
 #define ADDC( c ) \
     if( i_text < 10*MAX_LINE )      \
     {                               \
@@ -1036,7 +1112,7 @@ static int  sub_Sami( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_subtit
 #undef ADDC
 }
 
-static int  sub_VobSub( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe)
+static int  sub_VobSubIDX( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_subtitle, mtime_t i_microsecperframe)
 {
     /*
      * Parse the idx file. Each line:
@@ -1045,9 +1121,13 @@ static int  sub_VobSub( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_subt
      *
      */
     char *p;
-
     char buffer_text[MAX_LINE + 1];
-    uint32_t    i_start, i_location;
+    unsigned int    i_start, i_location;
+
+    p_subtitle->i_start = 0;
+    p_subtitle->i_stop  = 0;
+    p_subtitle->i_vobsub_location  = 0;
+    p_subtitle->psz_text = NULL;
 
     for( ;; )
     {
@@ -1073,9 +1153,71 @@ static int  sub_VobSub( subtitle_demux_t *p_sub, text_t *txt, subtitle_t *p_subt
     }
     p_subtitle->i_start = (mtime_t)i_start;
     p_subtitle->i_stop  = 0;
+    p_subtitle->psz_text = NULL;
     p_subtitle->i_vobsub_location = i_location;
     fprintf( stderr, "time: %x, location: %x\n", i_start, i_location );
     return( 0 );
 }
 
+static int  DemuxVobSub( subtitle_demux_t *p_demux, block_t *p_bk )
+{
+    uint8_t     *p = p_bk->p_buffer;
+    uint8_t     *p_end = &p_bk->p_buffer[p_bk->i_buffer];
 
+    while( p < p_end )
+    {
+        int i_size = ps_pkt_size( p, p_end - p );
+        block_t *p_pkt;
+        int      i_id;
+        int      i_spu;
+
+        if( i_size <= 0 )
+        {
+            break;
+        }
+        if( p[0] != 0 || p[1] != 0 || p[2] != 0x01 )
+        {
+            msg_Warn( p_demux, "invalid PES" );
+            break;
+        }
+
+        if( p[3] != 0xbd )
+        {
+            msg_Dbg( p_demux, "we don't need these ps packets (id=0x1%2.2x)", p[3] );
+            p += i_size;
+            continue;
+        }
+
+        /* Create a block */
+        p_pkt = block_New( p_demux, i_size );
+        memcpy( p_pkt->p_buffer, p, i_size);
+        p += i_size;
+
+        i_id = ps_pkt_id( p_pkt );
+        if( (i_id&0xffe0) != 0xbd20 ||
+            ps_pkt_parse_pes( p_pkt, 1 ) )
+        {
+            block_Release( p_pkt );
+            continue;
+        }
+        i_spu = i_id&0x1f;
+        msg_Dbg( p_demux, "SPU track %d size %d", i_spu, i_size );
+
+        /* FIXME i_spu == determines which of the spu tracks we will show. */
+        if( p_demux->p_es && i_spu == 0 )
+        {
+            p_pkt->i_pts = p_bk->i_pts;
+            p_pkt->i_dts = 0;
+            es_out_Send( p_demux->p_input->p_es_out, p_demux->p_es, p_pkt );
+
+            p_bk->i_pts = 0;    /* only first packet has a pts */
+        }
+        else
+        {
+            block_Release( p_pkt );
+            continue;
+        }
+    }
+
+    return VLC_SUCCESS;
+}
