@@ -2,7 +2,7 @@
  * MsgVals.h
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: MsgVals.h,v 1.9 2003/05/30 17:30:54 titer Exp $
+ * $Id: MsgVals.h,v 1.11 2004/01/26 16:52:31 zorglub Exp $
  *
  * Authors: Tony Castley <tcastley@mail.powerup.com.au>
  *          Stephan Aßmus <stippi@yellowbites.com>
@@ -11,7 +11,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -76,5 +76,6 @@
 #define MSG_SOUNDPLAY      'move'  // drag'n'drop from soundplay playlist
 #define INTERFACE_CREATED  'ifcr'  /* see VlcApplication::MessageReceived()
                                             * in src/misc/beos_specific.cpp */
+#define SHORTCUT           'shcu'
 
 #endif    // BEOS_MESSAGE_VALUES_H
