@@ -2,7 +2,7 @@
  * gnome.c : Gnome plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000 VideoLAN
- * $Id: gnome.c,v 1.13 2003/03/10 18:17:19 gbazin Exp $
+ * $Id: gnome.c,v 1.17 2004/01/25 18:53:07 gbazin Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -52,13 +52,13 @@ static void Manage       ( intf_thread_t * );
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-#define TOOLTIPS_TEXT N_("show tooltips")
+#define TOOLTIPS_TEXT N_("Show tooltips")
 #define TOOLTIPS_LONGTEXT N_("Show tooltips for configuration options.")
 
-#define TOOLBAR_TEXT N_("show text on toolbar buttons")
+#define TOOLBAR_TEXT N_("Show text on toolbar buttons")
 #define TOOLBAR_LONGTEXT N_("Show the text below icons on the toolbar.")
 
-#define PREFS_MAXH_TEXT N_("maximum height for the configuration windows")
+#define PREFS_MAXH_TEXT N_("Maximum height for the configuration windows")
 #define PREFS_MAXH_LONGTEXT N_( \
     "You can set the maximum height that the configuration windows in the " \
     "preferences menu will occupy.")
@@ -69,7 +69,8 @@ vlc_module_begin();
 #else
     int i = getenv( "DISPLAY" ) == NULL ? 15 : 100;
 #endif
-    add_category_hint( N_("GNOME"), NULL, VLC_FALSE );
+    set_description( _("GNOME interface") );
+
     add_bool( "gnome-tooltips", 1, E_(GtkHideTooltips),
               TOOLTIPS_TEXT, TOOLTIPS_LONGTEXT, VLC_FALSE );
     add_bool( "gnome-toolbartext", 1, GtkHideToolbarText, TOOLBAR_TEXT,
@@ -77,7 +78,6 @@ vlc_module_begin();
     add_integer( "gnome-prefs-maxh", 480, NULL,
                  PREFS_MAXH_TEXT, PREFS_MAXH_LONGTEXT, VLC_TRUE );
 
-    set_description( _("GNOME interface module") );
     set_capability( "interface", i );
     set_callbacks( Open, Close );
     set_program( "gnome-vlc" );
@@ -98,7 +98,7 @@ static int Open( vlc_object_t *p_this )
         return VLC_ENOMEM;
     }
 
-    p_intf->p_sys->p_gtk_main = module_Need( p_this, "gtk_main", "gnome" );
+    p_intf->p_sys->p_gtk_main = module_Need( p_this, "gui-helper", "gnome" );
     if( p_intf->p_sys->p_gtk_main == NULL )
     {
         free( p_intf->p_sys );
