@@ -2,7 +2,7 @@
  * kde.cpp : KDE plugin for vlc
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: kde.cpp,v 1.13 2003/12/04 16:02:54 sam Exp $
+ * $Id: kde.cpp,v 1.15 2004/01/25 18:53:07 gbazin Exp $
  *
  * Authors: Andres Krapf <dae@chez.com> Sun Mar 25 2001
  *
@@ -51,9 +51,8 @@ static void run(intf_thread_t *p_intf);
  *****************************************************************************/
 vlc_module_begin();
     /* int i = getenv( "DISPLAY" ) == NULL ? 8 : 85; */
-    add_category_hint( "kde", NULL, VLC_TRUE );
-    add_file( "kde-uirc", DATA_PATH "/ui.rc", NULL, N_( "path to ui.rc file" ), NULL, VLC_TRUE );
     set_description( _("KDE interface") );
+    add_file( "kde-uirc", DATA_PATH "/ui.rc", NULL, N_( "path to ui.rc file" ), NULL, VLC_TRUE );
     set_capability( "interface", 0 ); /* 0 used to be i, disabled because kvlc not maintained */
     set_program( "kvlc" );
     set_callbacks( open, close );
@@ -105,10 +104,10 @@ void run(intf_thread_t *p_intf)
 {
     p_intf->p_sys->p_about =
       new KAboutData( "kvlc", I18N_NOOP("Kvlc"), VERSION,
-         _("This is the VideoLAN Client, a DVD, MPEG and DivX player. It can "
+         _("This is the VLC media player, a DVD, MPEG and DivX player. It can "
            "play MPEG and MPEG2 files from a file or from a network source."),
          KAboutData::License_GPL,
-         _("(c) 1996-2003 the VideoLAN team"),
+         _("(c) 1996-2004 the VideoLAN team"),
          0, 0, "");
 
     p_intf->p_sys->p_about->addAuthor( "the VideoLAN team", 0,
