@@ -2,7 +2,7 @@
  * preferences.cpp: preferences window for the kde gui
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: preferences.cpp,v 1.17 2003/07/04 16:35:20 sam Exp $
+ * $Id: preferences.cpp,v 1.18 2004/01/25 15:42:57 kuehne Exp $
  *
  * Authors: Sigmund Augdal <sigmunau@idi.ntnu.no> Mon Aug 12 2002
  *
@@ -53,7 +53,7 @@
 KPreferences::KPreferences(intf_thread_t *p_intf, const char *psz_module_name,
                            QWidget *parent, const QString &caption) :
     KDialogBase ( TreeList, caption, Ok| Apply|Cancel|User1, Ok, parent,
-                  "vlc preferences", true, false, i18n("&Save") )
+                  _("vlc preferences"), true, false, i18n(_("&Save")) )
 {
     module_t *p_parser = NULL;
     vlc_list_t *p_list;
