@@ -1,8 +1,8 @@
 /*****************************************************************************
- * macosx.m: MacOS X plugin for vlc
+ * macosx.m: MacOS X module for vlc
  *****************************************************************************
  * Copyright (C) 2001-2003 VideoLAN
- * $Id: macosx.m,v 1.16 2003/12/08 19:50:22 gbazin Exp $
+ * $Id: macosx.m,v 1.22 2004/02/26 13:04:55 hartman Exp $
  *
  * Authors: Colin Delacroix <colin@zoy.org>
  *          Eugenio Jarosiewicz <ej0@cise.ufl.edu>
@@ -54,6 +54,25 @@ void E_(CloseVideo)   ( vlc_object_t * );
     "Set the transparency of the video output. 1 is non-transparent (default) " \
     "0 is fully transparent.")
     
+#define STRETCH_TEXT N_("Stretch Aspect Ratio")
+#define STRETCH_LONGTEXT N_("Instead of keeping the aspect ratio " \
+        "of the movie when resizing the video, stretch the video " \
+        "to fill the entire window." )
+
+#define OPENGL_TEXT N_("Use OpenGL")
+#define OPENGL_LONGTEXT N_("Use OpenGL instead of QuickTime to " \
+        "render the video on the screen.")
+
+#define OPENGL_EFFECT_TEXT N_("OpenGL effect")
+#define OPENGL_EFFECT_LONGTEXT N_("Use 'None' to display the video " \
+        "without any fantasy, 'Cube' to let the video play on " \
+        "the faces of a rotating cube, 'Transparent cube' do make this " \
+        "cube transparent" )
+
+static char * effect_list[] = { "none", "cube", "transparent-cube" };
+static char * effect_list_text[] = { N_("None"), N_("Cube"),
+                                     N_("Transparent cube") };
+    
 vlc_module_begin();
     set_description( _("MacOS X interface, sound and video") );
     set_capability( "interface", 100 );
@@ -61,9 +80,17 @@ vlc_module_begin();
     add_submodule();
         set_capability( "video output", 200 );
         set_callbacks( E_(OpenVideo), E_(CloseVideo) );
-        add_category_hint( N_("Video"), NULL, VLC_FALSE );
-        add_integer( "macosx-vdev", 0, NULL, VDEV_TEXT, VDEV_LONGTEXT, VLC_FALSE );
-        add_float_with_range( "macosx-opaqueness", 1, 0, 1, NULL, OPAQUENESS_TEXT,
-            OPAQUENESS_LONGTEXT, VLC_TRUE );
+        add_integer( "macosx-vdev", 0, NULL, VDEV_TEXT, VDEV_LONGTEXT,
+                     VLC_FALSE );
+        add_bool( "macosx-stretch", 0, NULL, STRETCH_TEXT, STRETCH_LONGTEXT,
+                     VLC_FALSE );
+        add_float_with_range( "macosx-opaqueness", 1, 0, 1, NULL,
+                OPAQUENESS_TEXT, OPAQUENESS_LONGTEXT, VLC_TRUE );
+        add_bool( "macosx-opengl", 0, NULL, OPENGL_TEXT,
+                  OPENGL_LONGTEXT, VLC_TRUE );
+        add_string( "macosx-opengl-effect", "none", NULL,
+                    OPENGL_EFFECT_TEXT, OPENGL_EFFECT_LONGTEXT,
+                    VLC_TRUE );
+        change_string_list( effect_list, effect_list_text, 0 );
 vlc_module_end();
 
