@@ -1,8 +1,8 @@
 /*****************************************************************************
- * open.h: MacOS X plugin for vlc
+ * open.h: MacOS X module for vlc
  *****************************************************************************
  * Copyright (C) 2002-2003 VideoLAN
- * $Id: open.h,v 1.15 2003/07/27 23:05:41 hartman Exp $
+ * $Id: open.h,v 1.17 2004/01/25 17:01:57 murray Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -40,7 +40,6 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
 
     IBOutlet id o_btn_ok;
     IBOutlet id o_btn_cancel;
-    IBOutlet id o_ckbox_enqueue;
 
     IBOutlet id o_file_path;
     IBOutlet id o_file_btn_browse;
