@@ -2,7 +2,7 @@
  * output.m: MacOS X Output Dialog
  *****************************************************************************
  * Copyright (C) 2002-2003 VideoLAN
- * $Id: output.m,v 1.12 2003/07/28 21:41:20 massiot Exp $
+ * $Id: output.m,v 1.18 2004/01/25 17:01:57 murray Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net> 
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -104,7 +104,7 @@
     [[NSNotificationCenter defaultCenter] addObserver: self
         selector: @selector(transcodeInfoChanged:)
         name: NSControlTextDidChangeNotification
-        object: o_sap_name];
+        object: o_channel_name];
 
     [o_mux_selector setAutoenablesItems: NO];
     [self transcodeChanged:nil];
@@ -112,14 +112,16 @@
 
 - (void)initStrings
 {
+    NSArray *o_muxers = [NSArray arrayWithObjects: @"MPEG TS", @"MPEG PS", @"MPEG 1",
+        @"Ogg", @"AVI", @"ASF", @"MPEG 4", @"Quicktime", nil];
     NSArray *o_a_channels = [NSArray arrayWithObjects: @"1", @"2", @"4", @"6", nil];
-    NSArray *o_a_bitrates = [NSArray arrayWithObjects: @"96", @"128", @"192", @"256", @"512", nil];
-    NSArray *o_v_bitrates = [NSArray arrayWithObjects:
-        @"100", @"150", @"200", @"400", @"500", @"750", @"1000", @"2000", @"3000", nil];
-    NSArray *o_a_codecs = [NSArray arrayWithObjects:
-        @"mpga", @"mp3 ", @"a52 ", @"vorb", nil];
-    NSArray *o_v_codecs = [NSArray arrayWithObjects:
-        @"mpgv", @"mp4v", @"DIV1", @"DIV2", @"DIV3", @"H263", @"I263", @"WMV1", @"WMV2", @"MJPG", nil];
+    NSArray *o_a_bitrates = [NSArray arrayWithObjects: @"16", @"32", @"64", @"96",
+        @"128", @"192", @"256", @"512", nil];
+    NSArray *o_v_bitrates = [NSArray arrayWithObjects: @"16", @"32", @"64", @"96",
+        @"128", @"192", @"256", @"384", @"512", @"768", @"1024", @"2048", @"3072", nil];
+    NSArray *o_a_codecs = [NSArray arrayWithObjects: @"mpga", @"mp3 ", @"a52 ", @"vorb", @"flac", @"spx ", nil];
+    NSArray *o_v_codecs = [NSArray arrayWithObjects: @"mp1v", @"mp2v", @"mp4v", @"DIV1",
+        @"DIV2", @"DIV3", @"H263", @"I263", @"WMV1", @"WMV2", @"MJPG", @"theo", nil];
     
     [o_output_ckbox setTitle: _NS("Advanced output:")];
     [o_output_settings setTitle: _NS("Settings...")];
@@ -129,22 +131,20 @@
     [o_display setTitle: _NS("Screen")];
     [[o_method cellAtRow:0 column:0] setTitle: _NS("File")];
     [[o_method cellAtRow:1 column:0] setTitle: _NS("Stream")];
+    [o_dump_chkbox setTitle: _NS("Dump raw input")];
     [o_btn_browse setTitle: _NS("Browse...")]; 
     [o_stream_address_lbl setStringValue: _NS("Address")];
     [o_stream_port_lbl setStringValue: _NS("Port")];
-    [o_stream_ttl_lbl setStringValue: _NS("TTL")];
-    [[o_stream_type itemAtIndex: 0] setTitle: _NS("HTTP")];
-    [[o_stream_type itemAtIndex: 1] setTitle: _NS("UDP")];
-    [[o_stream_type itemAtIndex: 2] setTitle: _NS("RTP")];
+    [o_stream_ttl_lbl setStringValue: @"TTL"];
+    [[o_stream_type itemAtIndex: 0] setTitle: @"HTTP"];
+    [[o_stream_type itemAtIndex: 1] setTitle: @"MMSH"];
+    [[o_stream_type itemAtIndex: 2] setTitle: @"UDP"];
+    [[o_stream_type itemAtIndex: 3] setTitle: @"RTP"];
     [o_stream_type_lbl setStringValue: _NS("Type")];
     
     [o_mux_lbl setStringValue: _NS("Encapsulation Method")];
-    [[o_mux_selector itemAtIndex: 0] setTitle: _NS("MPEG TS")];
-    [[o_mux_selector itemAtIndex: 1] setTitle: _NS("MPEG PS")];
-    [[o_mux_selector itemAtIndex: 1] setTitle: _NS("MPEG1")];
-    [[o_mux_selector itemAtIndex: 3] setTitle: _NS("AVI")];
-    [[o_mux_selector itemAtIndex: 4] setTitle: _NS("Ogg")];
-    //[[o_mux_selector itemAtIndex: 5] setTitle: _NS("mp4")];
+    [o_mux_selector removeAllItems];
+    [o_mux_selector addItemsWithTitles: o_muxers];
     
     [o_transcode_lbl setTitle: _NS("Transcode options")];
     [o_transcode_video_chkbox setTitle: _NS("Video")];
@@ -163,8 +163,10 @@
     [o_transcode_audio_channels removeAllItems];
     [o_transcode_audio_channels addItemsWithObjectValues: o_a_channels];
     
-    [o_misc_lbl setTitle: _NS("Miscellaneous Options")];
-    [o_sap_chkbox setTitle: _NS("Announce streams via SAP Channel:")];
+    [o_misc_lbl setTitle: _NS("Stream Announcing")];
+    [o_sap_chkbox setTitle: _NS("SAP announce")];
+    [o_slp_chkbox setTitle: _NS("SLP announce")];
+    [o_channel_name_lbl setStringValue: _NS("Channel Name")];
 }
 
 - (IBAction)outputChanged:(id)sender;
@@ -190,9 +192,6 @@
 
 - (IBAction)outputCloseSheet:(id)sender
 {
-    intf_thread_t * p_intf = [NSApp getIntf];
-    //export sout[o_mrl UTF8String]
-    
     [o_output_sheet orderOut:sender];
     [NSApp endSheet: o_output_sheet];
 }
@@ -203,12 +202,15 @@
     o_mode = [[o_method selectedCell] title];
     
     [o_sap_chkbox setEnabled: NO];
-    [o_sap_name setEnabled: NO];
+    [o_slp_chkbox setEnabled: NO];
+    [o_channel_name setEnabled: NO];
+    [[o_mux_selector itemAtIndex: 0] setEnabled: YES];
 
     if( [o_mode isEqualToString: _NS("File")] )
     {
         [o_file_field setEnabled: YES];
         [o_btn_browse setEnabled: YES];
+        [o_dump_chkbox setEnabled: YES];
         [o_stream_address setEnabled: NO];
         [o_stream_port setEnabled: NO];
         [o_stream_ttl setEnabled: NO];
@@ -216,15 +218,18 @@
         [o_stream_ttl_stp setEnabled: NO];
         [o_stream_type setEnabled: NO];
         [o_mux_selector setEnabled: YES];
-        [[o_mux_selector itemAtIndex: 1] setEnabled: YES];
-        [[o_mux_selector itemAtIndex: 2] setEnabled: YES];
-        [[o_mux_selector itemAtIndex: 3] setEnabled: YES];
-        [[o_mux_selector itemAtIndex: 4] setEnabled: YES];
-        //[[o_mux_selector itemAtIndex: 5] setEnabled: YES];
+        [[o_mux_selector itemAtIndex: 1] setEnabled: YES]; // MPEG PS
+        [[o_mux_selector itemAtIndex: 2] setEnabled: YES]; // MPEG 1
+        [[o_mux_selector itemAtIndex: 3] setEnabled: YES]; // Ogg
+        [[o_mux_selector itemAtIndex: 4] setEnabled: YES]; // AVI
+        [[o_mux_selector itemAtIndex: 5] setEnabled: YES]; // ASF
+        [[o_mux_selector itemAtIndex: 6] setEnabled: YES]; // MPEG 4
+        [[o_mux_selector itemAtIndex: 7] setEnabled: YES]; // QuickTime
     }
     else if( [o_mode isEqualToString: _NS("Stream")] )
     {
         [o_file_field setEnabled: NO];
+        [o_dump_chkbox setEnabled: NO];
         [o_btn_browse setEnabled: NO];
         [o_stream_port setEnabled: YES];
         [o_stream_port_stp setEnabled: YES];
@@ -233,18 +238,34 @@
         
         o_mode = [o_stream_type titleOfSelectedItem];
         
-        if( [o_mode isEqualToString: _NS("HTTP")] )
+        if( [o_mode isEqualToString: @"HTTP"] )
         {
             [o_stream_address setEnabled: YES];
             [o_stream_ttl setEnabled: NO];
             [o_stream_ttl_stp setEnabled: NO];
             [[o_mux_selector itemAtIndex: 1] setEnabled: YES];
             [[o_mux_selector itemAtIndex: 2] setEnabled: YES];
-            [[o_mux_selector itemAtIndex: 3] setEnabled: NO];
-            [[o_mux_selector itemAtIndex: 4] setEnabled: YES];
-            //[[o_mux_selector itemAtIndex: 5] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 3] setEnabled: YES];
+            [[o_mux_selector itemAtIndex: 4] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 5] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 6] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 7] setEnabled: NO];
         }
-        else if( [o_mode isEqualToString: _NS("UDP")] )
+        else if( [o_mode isEqualToString: @"MMSH"] )
+        {
+            [o_stream_address setEnabled: YES];
+            [o_stream_ttl setEnabled: NO];
+            [o_stream_ttl_stp setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 0] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 1] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 2] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 3] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 4] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 5] setEnabled: YES];
+            [[o_mux_selector itemAtIndex: 6] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 7] setEnabled: NO];
+        }
+        else if( [o_mode isEqualToString: @"UDP"] )
         {
             [o_stream_address setEnabled: YES];
             [o_stream_ttl setEnabled: YES];
@@ -253,11 +274,14 @@
             [[o_mux_selector itemAtIndex: 2] setEnabled: NO];
             [[o_mux_selector itemAtIndex: 3] setEnabled: NO];
             [[o_mux_selector itemAtIndex: 4] setEnabled: NO];
-            //[[o_mux_selector itemAtIndex: 5] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 5] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 6] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 7] setEnabled: NO];
             [o_sap_chkbox setEnabled: YES];
-            [o_sap_name setEnabled: YES];
+            [o_slp_chkbox setEnabled: YES];
+            [o_channel_name setEnabled: YES];
         }
-        else if( [o_mode isEqualToString: _NS("RTP")] )
+        else if( [o_mode isEqualToString: @"RTP"] )
         {
             [o_stream_address setEnabled: YES];
             [o_stream_ttl setEnabled: YES];
@@ -266,7 +290,9 @@
             [[o_mux_selector itemAtIndex: 2] setEnabled: NO];
             [[o_mux_selector itemAtIndex: 3] setEnabled: NO];
             [[o_mux_selector itemAtIndex: 4] setEnabled: NO];
-            //[[o_mux_selector itemAtIndex: 5] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 5] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 6] setEnabled: NO];
+            [[o_mux_selector itemAtIndex: 7] setEnabled: NO];
         }
     }
     if( ![[o_mux_selector selectedItem] isEnabled] )
@@ -278,8 +304,8 @@
 
 - (void)outputInfoChanged:(NSNotification *)o_notification
 {
-    NSString *o_mode, *o_mux, *o_mux_string, *o_sap;
-    NSMutableString *o_mrl_string = [NSMutableString stringWithString:@"#"];
+    NSString *o_mode, *o_mux, *o_mux_string, *o_announce;
+    NSMutableString *o_mrl_string = [NSMutableString stringWithString:@":sout=#"];
 
     [o_mrl_string appendString: o_transcode];
     if( [o_display state] == NSOnState )
@@ -290,43 +316,70 @@
     o_mode = [[o_method selectedCell] title];
     o_mux = [o_mux_selector titleOfSelectedItem];
 
-    if ( [o_mux isEqualToString: _NS("AVI")] ) o_mux_string = @"avi";
-    else if ( [o_mux isEqualToString: _NS("Ogg")] ) o_mux_string = @"ogg";
-    else if ( [o_mux isEqualToString: _NS("MPEG PS")] ) o_mux_string = @"ps";
-    else if ( [o_mux isEqualToString: _NS("mp4")] ) o_mux_string = @"mp4";
-    else if ( [o_mux isEqualToString: _NS("MPEG1")] ) o_mux_string = @"mpeg1";
+    if ( [o_mux isEqualToString: @"AVI"] ) o_mux_string = @"avi";
+    else if ( [o_mux isEqualToString: @"Ogg"] ) o_mux_string = @"ogg";
+    else if ( [o_mux isEqualToString: @"MPEG PS"] ) o_mux_string = @"ps";
+    else if ( [o_mux isEqualToString: @"MPEG 4"] ) o_mux_string = @"mp4";
+    else if ( [o_mux isEqualToString: @"MPEG 1"] ) o_mux_string = @"mpeg1";
+    else if ( [o_mux isEqualToString: @"Quicktime"] ) o_mux_string = @"mov";
+    else if ( [o_mux isEqualToString: @"ASF"] ) o_mux_string = @"asf";
     else o_mux_string = @"ts";
 
     if( [o_mode isEqualToString: _NS("File")] )
     {
-        [o_mrl_string appendFormat:
+        if( [o_dump_chkbox state] == NSOnState )
+        {
+            o_mrl_string = [NSMutableString stringWithFormat:
+                            @":demux=demuxdump :demuxdump-file=\"%@\"",
+                            [o_file_field stringValue]];
+            [self setMRL:o_mrl_string];
+            return;
+        }
+        else
+        {
+                [o_mrl_string appendFormat:
                         @"std{access=file,mux=%@,url=\"%@\"}",
                         o_mux_string, [o_file_field stringValue]];
+        }
     }
     else if( [o_mode isEqualToString: _NS("Stream")] )
     {
         o_mode = [o_stream_type titleOfSelectedItem];
-        o_sap = @"";
+        o_announce = @"";
         
-        if ( [o_mode isEqualToString: _NS("HTTP")] )
+        if ( [o_mode isEqualToString: @"HTTP"] )
             o_mode = @"http";
-        else if ( [o_mode isEqualToString: _NS("UDP")] )
+        else if ( [o_mode isEqualToString: @"MMSH"] )
+        {
+            if ( [o_mux isEqualToString: @"ASF"] ) o_mux_string = @"asfh";
+            o_mode = @"mmsh";
+        }
+        else if ( [o_mode isEqualToString: @"UDP"] )
         {
             o_mode = @"udp";
             if( [o_sap_chkbox state] == NSOnState )
             {
-                o_sap = @",sap";
-                if ( ![[o_sap_name stringValue] isEqualToString: @""] )
-                    o_sap = [NSString stringWithFormat:@",sap=%@", [o_sap_name stringValue]];
+                if ( ![[o_channel_name stringValue] isEqualToString: @""] )
+                    o_announce = [NSString stringWithFormat:@",sap=%@", [o_channel_name stringValue]];
+                else
+                    o_announce = @",sap";
+            }
+            if( [o_slp_chkbox state] == NSOnState )
+            {
+                if ( ![[o_channel_name stringValue] isEqualToString: @""] )
+                    o_announce = [o_announce stringByAppendingFormat:@",slp=%@",
+                        [o_channel_name stringValue]];
+                else
+                    o_announce = [o_announce stringByAppendingString: @",slp"];
             }
         }
-        else if ( [o_mode isEqualToString: _NS("RTP")] )
+        else if ( [o_mode isEqualToString: @"RTP"] )
             o_mode = @"rtp";
             
         [o_mrl_string appendFormat:
                         @"std{access=%@,mux=%@,url=\"%@:%@\"%@}",
                         o_mode, o_mux_string, [o_stream_address stringValue],
-                        [o_stream_port stringValue], o_sap];
+                        [o_stream_port stringValue], o_announce];
     }
     if( [o_display state] == NSOnState )
     {
@@ -345,16 +398,20 @@
 {
     NSSavePanel *o_save_panel = [NSSavePanel savePanel];
     NSString *o_mux_string;
-    if ( [[o_mux_selector titleOfSelectedItem] isEqualToString: _NS("MPEG PS")] )
+    if ( [[o_mux_selector titleOfSelectedItem] isEqualToString: @"MPEG PS"] )
         o_mux_string = @"vob";
-    else if ( [[o_mux_selector titleOfSelectedItem] isEqualToString: _NS("MPEG1")] )
+    else if ( [[o_mux_selector titleOfSelectedItem] isEqualToString: @"MPEG 1"] )
         o_mux_string = @"mpg";
-    else if ( [[o_mux_selector titleOfSelectedItem] isEqualToString: _NS("AVI")] )
+    else if ( [[o_mux_selector titleOfSelectedItem] isEqualToString: @"AVI"] )
         o_mux_string = @"avi";
-    else if ( [[o_mux_selector titleOfSelectedItem] isEqualToString: _NS("Ogg")] )
+    else if ( [[o_mux_selector titleOfSelectedItem] isEqualToString: @"ASF"] )
+        o_mux_string = @"asf";
+    else if ( [[o_mux_selector titleOfSelectedItem] isEqualToString: @"Ogg"] )
         o_mux_string = @"ogm";
-    else if ( [[o_mux_selector titleOfSelectedItem] isEqualToString: _NS("mp4")] )
+    else if ( [[o_mux_selector titleOfSelectedItem] isEqualToString: @"MPEG 4"] )
         o_mux_string = @"mp4";
+    else if ( [[o_mux_selector titleOfSelectedItem] isEqualToString: @"Quicktime"] )
+        o_mux_string = @"mov";
     else
         o_mux_string = @"ts";
 
@@ -447,12 +504,10 @@
     [self outputInfoChanged:nil];
 }
 
-- (IBAction)sapChanged:(id)sender
+- (IBAction)announceChanged:(id)sender
 {
-    [o_sap_name setEnabled: [o_sap_chkbox state]];
+    [o_channel_name setEnabled: [o_sap_chkbox state] || [o_slp_chkbox state]];
     [self outputInfoChanged: nil];
 }
 
-
-
 @end
