@@ -1,11 +1,11 @@
 /*****************************************************************************
- * playlist.h: MacOS X interface plugin
+ * playlist.h: MacOS X interface module
  *****************************************************************************
- * Copyright (C) 2002-2003 VideoLAN
- * $Id: playlist.h,v 1.16 2003/12/11 19:34:46 hartman Exp $
+ * Copyright (C) 2002-2004 VideoLAN
+ * $Id: playlist.h,v 1.21 2004/02/06 04:51:02 hartman Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
- *          Derk-Jan Hartman <thedj@users.sourceforge.net>
+ *          Derk-Jan Hartman <hartman at videolan dot org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,14 +37,18 @@
 @interface VLCPlaylist : NSObject
 {
     int i_moveRow;
+    bool b_isSortDescending;
     
     IBOutlet id o_window;
     IBOutlet id o_btn_playlist;
     IBOutlet id o_table_view;
     
     IBOutlet id o_status_field;
+    IBOutlet id o_tc_id;
     IBOutlet id o_tc_name;
     IBOutlet id o_tc_author;
+    IBOutlet id o_tc_duration;
+    IBOutlet id o_tc_sortColumn;
 
     IBOutlet id o_ctx_menu;
     IBOutlet id o_mi_save_playlist;
@@ -59,6 +63,9 @@
     IBOutlet id o_search_keyword;
     IBOutlet id o_search_button;
 
+    NSImage *o_descendingSortingImage;
+    NSImage *o_ascendingSortingImage;
+
 }
 
 - (void)initStrings;
