@@ -1,11 +1,11 @@
 /*****************************************************************************
- * playlist.m: MacOS X interface plugin
+ * playlist.m: MacOS X interface module
  *****************************************************************************
- * Copyright (C) 2002-2003 VideoLAN
- * $Id: playlist.m,v 1.49 2003/12/15 14:25:43 hartman Exp $
+ * Copyright (C) 2002-2004 VideoLAN
+ * $Id: playlist.m,v 1.58 2004/02/26 14:40:29 hartman Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
- *          Derk-Jan Hartman <thedj@users.sourceforge.net>
+ *          Derk-Jan Hartman <hartman at videolan dot org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -104,6 +104,7 @@
     }
 }
 
+
 @end
 
 /*****************************************************************************
@@ -133,6 +134,28 @@
         [NSArray arrayWithObjects: NSFilenamesPboardType, nil]];
 
     [o_window setExcludedFromWindowsMenu: TRUE];
+
+/* We need to check whether _defaultTableHeaderSortImage exists, since it 
+belongs to an Apple hidden private API, and then can "disapear" at any time*/
+
+    if( [[NSTableView class] respondsToSelector:@selector(_defaultTableHeaderSortImage)] )
+    {
+        o_ascendingSortingImage = [[NSTableView class] _defaultTableHeaderSortImage];
+    }
+    else
+    {
+        o_ascendingSortingImage = nil;
+    }
+
+    if( [[NSTableView class] respondsToSelector:@selector(_defaultTableHeaderReverseSortImage)] )
+    {
+        o_descendingSortingImage = [[NSTableView class] _defaultTableHeaderReverseSortImage];
+    }
+    else
+    {
+        o_descendingSortingImage = nil;
+    }
+
     [self initStrings];
 }
 
@@ -145,13 +168,81 @@
     [o_mi_selectall setTitle: _NS("Select All")];
     [[o_tc_name headerCell] setStringValue:_NS("Name")];
     [[o_tc_author headerCell] setStringValue:_NS("Author")];
-    [o_random_ckb setTitle: _NS("Shuffle")];
-    [o_loop_ckb setTitle: _NS("Repeat Playlist")];
-    [o_repeat_ckb setTitle: _NS("Repeat Item")];
+    [[o_tc_duration headerCell] setStringValue:_NS("Duration")];
+    [o_random_ckb setTitle: _NS("Random")];
+    [o_loop_ckb setTitle: _NS("Repeat All")];
+    [o_repeat_ckb setTitle: _NS("Repeat One")];
     [o_search_button setTitle: _NS("Search")];
     [o_btn_playlist setToolTip: _NS("Playlist")];
 }
 
+- (void) tableView:(NSTableView*)o_tv
+                  didClickTableColumn:(NSTableColumn *)o_tc
+{
+    intf_thread_t * p_intf = [NSApp getIntf];
+    playlist_t *p_playlist =
+        (playlist_t *)vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                       FIND_ANYWHERE );
+
+    int max = [[o_table_view tableColumns] count];
+    int i;
+
+    if( p_playlist == NULL )
+    {
+        return;
+    }
+
+    if( o_tc_sortColumn == o_tc )
+    { 
+        b_isSortDescending = !b_isSortDescending;
+    }
+    else if( o_tc == o_tc_name || o_tc == o_tc_author || 
+        o_tc == o_tc_id )
+    {
+        b_isSortDescending = VLC_FALSE;
+        [o_table_view setHighlightedTableColumn:o_tc];
+        o_tc_sortColumn = o_tc;
+        for( i=0 ; i<max ; i++ )
+        {
+            [o_table_view setIndicatorImage:nil inTableColumn:[[o_table_view tableColumns] objectAtIndex:i]];
+        }
+    }
+
+    if( o_tc_id == o_tc && !b_isSortDescending )
+    {    
+        playlist_SortID( p_playlist , ORDER_NORMAL );
+        [o_table_view setIndicatorImage:o_ascendingSortingImage inTableColumn:o_tc];    
+    }
+    else if( o_tc_name == o_tc && !b_isSortDescending )
+    {    
+        playlist_SortTitle( p_playlist , ORDER_NORMAL );
+        [o_table_view setIndicatorImage:o_ascendingSortingImage inTableColumn:o_tc];    
+    }
+    else if( o_tc_author == o_tc && !b_isSortDescending )
+    {
+        playlist_SortAuthor( p_playlist , ORDER_NORMAL );
+        [o_table_view setIndicatorImage:o_ascendingSortingImage inTableColumn:o_tc];
+    }
+    else if( o_tc_id == o_tc && b_isSortDescending )
+    {    
+        playlist_SortID( p_playlist , ORDER_REVERSE );
+        [o_table_view setIndicatorImage:o_ascendingSortingImage inTableColumn:o_tc];    
+    }
+    else if( o_tc_name == o_tc && b_isSortDescending )
+    {    
+        playlist_SortTitle( p_playlist , ORDER_REVERSE );
+        [o_table_view setIndicatorImage:o_descendingSortingImage inTableColumn:o_tc];
+    }
+    else if( o_tc_author == o_tc && b_isSortDescending )
+    {
+        playlist_SortAuthor( p_playlist , ORDER_REVERSE );
+        [o_table_view setIndicatorImage:o_descendingSortingImage inTableColumn:o_tc];
+    } 
+    vlc_object_release( p_playlist );
+    [self playlistUpdated];
+}
+
+
 - (BOOL)tableView:(NSTableView *)o_tv 
                   shouldEditTableColumn:(NSTableColumn *)o_tc
                   row:(int)i_row
@@ -206,7 +297,7 @@
     if( [o_save_panel runModalForDirectory: nil
             file: o_name] == NSOKButton )
     {
-        playlist_SaveFile( p_playlist, [[o_save_panel filename] fileSystemRepresentation] );
+        playlist_Export( p_playlist, [[o_save_panel filename] fileSystemRepresentation], "export-m3u" );
     }
 
 }
@@ -306,7 +397,7 @@
         o_current_name = [NSString stringWithUTF8String: 
             p_playlist->pp_items[i_current]->psz_name];
         o_current_author = [NSString stringWithUTF8String: 
-            p_playlist->pp_items[i_current]->psz_author];
+            playlist_GetInfo(p_playlist, i_current ,_("General"),_("Author") )];
         vlc_mutex_unlock( &p_playlist->object_lock );
 
 
@@ -326,6 +417,7 @@
     vlc_object_release( p_playlist );
 }
 
+
 - (void)appendArray:(NSArray*)o_array atPos:(int)i_position enqueue:(BOOL)b_enqueue
 {
     int i_item;
@@ -342,43 +434,37 @@
     {
         /* One item */
         NSDictionary *o_one_item;
-        int j;
-        int i_total_options = 0;
+        int j, i_total_options = 0, i_new_id = -1;
         int i_mode = PLAYLIST_INSERT;
         BOOL b_rem = FALSE, b_dir = FALSE;
-        NSString *o_url, *o_name;
+        NSString *o_uri, *o_name;
         NSArray *o_options;
+        NSURL *o_true_file;
         char **ppsz_options = NULL;
     
         /* Get the item */
         o_one_item = [o_array objectAtIndex: i_item];
-        o_url = (NSString *)[o_one_item objectForKey: @"ITEM_URL"];
+        o_uri = (NSString *)[o_one_item objectForKey: @"ITEM_URL"];
         o_name = (NSString *)[o_one_item objectForKey: @"ITEM_NAME"];
         o_options = (NSArray *)[o_one_item objectForKey: @"ITEM_OPTIONS"];
         
         /* If no name, then make a guess */
-        if( !o_name) o_name = [[NSFileManager defaultManager] displayNameAtPath: o_url];
+        if( !o_name) o_name = [[NSFileManager defaultManager] displayNameAtPath: o_uri];
     
-        if( [[NSFileManager defaultManager] fileExistsAtPath:o_url isDirectory:&b_dir] && b_dir &&
-            [[NSWorkspace sharedWorkspace] getFileSystemInfoForPath: o_url isRemovable: &b_rem
+        if( [[NSFileManager defaultManager] fileExistsAtPath:o_uri isDirectory:&b_dir] && b_dir &&
+            [[NSWorkspace sharedWorkspace] getFileSystemInfoForPath: o_uri isRemovable: &b_rem
                     isWritable:NULL isUnmountable:NULL description:NULL type:NULL] && b_rem   )
         {
             /* All of this is to make sure CD's play when you D&D them on VLC */
             /* Converts mountpoint to a /dev file */
             struct statfs *buf;
-            char *psz_dev, *temp;
+            char *psz_dev;
             buf = (struct statfs *) malloc (sizeof(struct statfs));
-            statfs( [o_url fileSystemRepresentation], buf );
+            statfs( [o_uri fileSystemRepresentation], buf );
             psz_dev = strdup(buf->f_mntfromname);
-            free( buf );
-            temp = strrchr( psz_dev , 's' );
-            psz_dev[temp - psz_dev] = '\0';
-            o_url = [NSString stringWithCString: psz_dev ];
+            o_uri = [NSString stringWithCString: psz_dev ];
         }
-    
-        if (i_item == 0 && !b_enqueue)
-            i_mode |= PLAYLIST_GO;
-    
+
         if( o_options && [o_options count] > 0 )
         {
             /* Count the input options */
@@ -393,22 +479,30 @@
                 ppsz_options[j] = strdup([[o_options objectAtIndex:j] UTF8String]);
             }
         }
-    
-        playlist_AddExt( p_playlist, [o_url fileSystemRepresentation], [o_name UTF8String], -1, 
-            (ppsz_options != NULL ) ? (const char **)ppsz_options : 0, i_total_options,
-            i_mode, i_position == -1 ? PLAYLIST_END : i_position + i_item);
-    
-        /* clean up */
+
+        /* Add the item */
+        i_new_id = playlist_AddExt( p_playlist, [o_uri fileSystemRepresentation], 
+                      [o_name UTF8String], i_mode, 
+                      i_position == -1 ? PLAYLIST_END : i_position + i_item,
+                      0, (ppsz_options != NULL ) ? (const char **)ppsz_options : 0, i_total_options );
+
+        /* clean up 
         for( j = 0; j < i_total_options; j++ )
             free( ppsz_options[j] );
-        if( ppsz_options ) free( ppsz_options );
-    
+        if( ppsz_options ) free( ppsz_options ); */
+
         /* Recent documents menu */
-        NSURL *o_true_url = [NSURL fileURLWithPath: o_url];
-        if( o_true_url != nil )
+        o_true_file = [NSURL fileURLWithPath: o_uri];
+        if( o_true_file != nil )
         { 
             [[NSDocumentController sharedDocumentController]
-                noteNewRecentDocumentURL: o_true_url]; 
+                noteNewRecentDocumentURL: o_true_file]; 
+        }
+        
+        if( i_item == 0 && !b_enqueue )
+        {
+            playlist_Goto( p_playlist, playlist_GetPositionById( p_playlist, i_new_id ) );
+            playlist_Play( p_playlist );
         }
     }
 
@@ -456,7 +550,7 @@
     [o_table_view selectRow: i_row byExtendingSelection: NO];
     [o_table_view scrollRowToVisible: i_row];
 }
-    
+
 
 @end
 
@@ -501,17 +595,31 @@
     else if( [[o_tc identifier] isEqualToString:@"1"] )
     {
         vlc_mutex_lock( &p_playlist->object_lock );
-        o_value = [[NSString stringWithUTF8String: 
-            p_playlist->pp_items[i_row]->psz_name] lastPathComponent]; 
+        o_value = [NSString stringWithUTF8String: 
+            p_playlist->pp_items[i_row]->psz_name];
         vlc_mutex_unlock( &p_playlist->object_lock );
     }
     else if( [[o_tc identifier] isEqualToString:@"2"] )
     {
         vlc_mutex_lock( &p_playlist->object_lock );
         o_value = [NSString stringWithUTF8String: 
-            p_playlist->pp_items[i_row]->psz_author]; 
+            playlist_GetInfo(p_playlist, i_row ,_("General"),_("Author") )];
         vlc_mutex_unlock( &p_playlist->object_lock );
     }
+    else if( [[o_tc identifier] isEqualToString:@"3"] )
+    {
+        char psz_duration[MSTRTIME_MAX_SIZE];
+        mtime_t dur = p_playlist->pp_items[i_row]->i_duration;
+        if( dur != -1 )
+        {
+            secstotimestr( psz_duration, dur/1000000 );
+            o_value = [NSString stringWithUTF8String: psz_duration];
+        }
+        else
+        {
+            o_value = @"-:--:--";
+        }
+    }
 
     vlc_object_release( p_playlist );
 
