@@ -2,7 +2,7 @@
  * vlcproc.cpp: VlcProc class
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: vlcproc.cpp,v 1.52 2003/10/23 16:00:48 gbazin Exp $
+ * $Id: vlcproc.cpp,v 1.54 2004/02/15 18:58:38 ipkiss Exp $
  *
  * Authors: Olivier Teulire <ipkiss@via.ecp.fr>
  *          Emmanuel Puig    <karibu@via.ecp.fr>
@@ -52,7 +52,7 @@ VlcProc::VlcProc( intf_thread_t *_p_intf )
         VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
     if( p_playlist != NULL )
     {
-        // We want to be notified of playlit changes
+        // We want to be notified of playlist changes
         var_AddCallback( p_playlist, "intf-change", RefreshCallback, this );
 
         // Raise/lower interface with middle click on vout
@@ -162,6 +162,14 @@ bool VlcProc::EventProc( Event *evt )
             PrevStream();
             return true;
 
+        case VLC_SLOWER:
+            SlowStream();
+            return true;
+
+        case VLC_FASTER:
+            FastStream();
+            return true;
+
         case VLC_PLAYLIST_ADD_FILE:
             p_intf->p_sys->p_dialogs->ShowOpen( false );
             InterfaceRefresh();
@@ -444,13 +452,13 @@ void VlcProc::DropFile( unsigned int param1, long param2 )
         if( param2 == 0 )
         {
             // Enqueue the item
-            playlist_Add( p_intf->p_sys->p_playlist, FileName, 0, 0,
+            playlist_Add( p_intf->p_sys->p_playlist, FileName, FileName,
                           PLAYLIST_APPEND, PLAYLIST_END );
         }
         else
         {
             // Enqueue and play the item
-            playlist_Add( p_intf->p_sys->p_playlist, FileName, 0, 0,
+            playlist_Add( p_intf->p_sys->p_playlist, FileName, FileName,
                           PLAYLIST_APPEND | PLAYLIST_GO, PLAYLIST_END );
         }
     }
@@ -532,6 +540,34 @@ void VlcProc::PrevStream()
     InterfaceRefresh();
 }
 //---------------------------------------------------------------------------
+void VlcProc::SlowStream()
+{
+    input_thread_t *p_input =
+        (input_thread_t *)vlc_object_find( p_intf, VLC_OBJECT_INPUT,
+                                           FIND_ANYWHERE );
+    if( p_input )
+    {
+        vlc_value_t val; val.b_bool = VLC_TRUE;
+
+        var_Set( p_input, "rate-slower", val );
+        vlc_object_release( p_input );
+    }
+}
+//---------------------------------------------------------------------------
+void VlcProc::FastStream()
+{
+    input_thread_t *p_input =
+        (input_thread_t *)vlc_object_find( p_intf, VLC_OBJECT_INPUT,
+                                           FIND_ANYWHERE );
+    if( p_input )
+    {
+        vlc_value_t val; val.b_bool = VLC_TRUE;
+
+        var_Set( p_input, "rate-faster", val );
+        vlc_object_release( p_input );
+    }
+}
+//---------------------------------------------------------------------------
 void VlcProc::MoveStream( long Pos )
 {
     if( p_intf->p_sys->p_input == NULL )
