@@ -2,7 +2,7 @@
  * httpd.c
  *****************************************************************************
  * Copyright (C) 2001-2003 VideoLAN
- * $Id: httpd.c,v 1.24 2003/07/10 22:37:02 fenrir Exp $
+ * $Id: httpd.c,v 1.31 2004/02/05 19:51:46 fenrir Exp $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -23,6 +23,9 @@
 
 /*****************************************************************************
  * Preamble
+ *
+ * TODO:
+ *  - make that two distinct host:port use different daemon
  *****************************************************************************/
 #include <stdlib.h>
 #include <vlc/vlc.h>
@@ -111,7 +114,9 @@ static httpd_file_t     *RegisterFile   ( httpd_t *,
                                           httpd_file_callback_args_t *p_args );
 static void             UnregisterFile  ( httpd_t *, httpd_file_t * );
 
-//#define httpd_stream_t              httpd_file_t
+#if 0
+ #define httpd_stream_t              httpd_file_t
+#endif
 static httpd_stream_t   *RegisterStream ( httpd_t *,
                                          char *psz_file, char *psz_mime,
                                          char *psz_user, char *psz_password );
@@ -134,10 +139,15 @@ struct httpd_host_t
 
 };
 
-#define HTTPD_AUTHENTICATE_NONE     0
-#define HTTPD_AUTHENTICATE_BASIC    1
+enum httpd_authenticate_e
+{
+    HTTPD_AUTHENTICATE_NONE = 0,
+    HTTPD_AUTHENTICATE_BASIC = 1
+};
 
-//typedef httpd_file_t httpd_stream_t;
+#if 0
+  typedef httpd_file_t httpd_stream_t;
+#endif
 
 struct httpd_file_t
 {
@@ -163,7 +173,7 @@ struct httpd_file_t
     int         i_buffer_size;      /* buffer size, can't be reallocated smaller */
     uint8_t     *p_buffer;          /* buffer */
     int64_t     i_buffer_pos;       /* absolute position from begining */
-    int         i_buffer_last_pos;  /* a new connection will start with that */
+    int64_t     i_buffer_last_pos;  /* a new connection will start with that */
 
     /* data to be send at connection time (if any) */
     int         i_header_size;
@@ -171,14 +181,24 @@ struct httpd_file_t
 };
 
 
-#define HTTPD_CONNECTION_RECEIVING_REQUEST      1
-#define HTTPD_CONNECTION_SENDING_HEADER         2
-#define HTTPD_CONNECTION_SENDING_FILE           3
-#define HTTPD_CONNECTION_SENDING_STREAM         4
-#define HTTPD_CONNECTION_TO_BE_CLOSED           5
+enum httpd_connection_state_e
+{
+    HTTPD_CONNECTION_RECEIVING_REQUEST = 1,
+    HTTPD_CONNECTION_SENDING_HEADER = 2,
+    HTTPD_CONNECTION_SENDING_FILE = 3,
+    HTTPD_CONNECTION_SENDING_STREAM = 4,
+    HTTPD_CONNECTION_TO_BE_CLOSED = 5
+};
+
+enum httpd_connection_method_e
+{
+    HTTPD_CONNECTION_METHOD_GET     = 1,
+    HTTPD_CONNECTION_METHOD_POST    = 2,
+    HTTPD_CONNECTION_METHOD_HEAD    = 3,
+    /* cludgy, only used when parsing connection request */
+    HTTPD_CONNECTION_METHOD_ASFHEAD = 4
+};
 
-#define HTTPD_CONNECTION_METHOD_GET             1
-#define HTTPD_CONNECTION_METHOD_POST            2
 typedef struct httpd_connection_s
 {
     struct httpd_connection_s *p_next;
@@ -191,12 +211,12 @@ typedef struct httpd_connection_s
     int    i_state;
     int    i_method;       /* get/post */
 
-    char    *psz_file;      // file to be send
-    int     i_http_error;   // error to be send with the file
-    char    *psz_user;      // if Authorization in the request header
+    char    *psz_file;      /* file to be send */
+    int     i_http_error;   /* error to be send with the file */
+    char    *psz_user;      /* if Authorization in the request header */
     char    *psz_password;
 
-    uint8_t *p_request;     // whith get: ?<*>, with post: main data
+    uint8_t *p_request;     /* whith get: ?<*>, with post: main data */
     int      i_request_size;
 
     httpd_file_t    *p_file;
@@ -247,7 +267,9 @@ static void httpd_Thread( httpd_sys_t *p_httpt );
 static void httpd_ConnnectionNew( httpd_sys_t *, int , struct sockaddr_in * );
 static void httpd_ConnnectionClose( httpd_sys_t *, httpd_connection_t * );
 static int httpd_UnbanIP( httpd_sys_t *, httpd_banned_ip_t *);
+#if 0
 static int httpd_BanIP( httpd_sys_t *, char *);
+#endif
 static httpd_banned_ip_t *httpd_GetbannedIP( httpd_sys_t *, char * );
 
 /*****************************************************************************
@@ -270,19 +292,36 @@ static int Open( vlc_object_t *p_this )
     p_httpt->b_error= 0;
 
     /* init httpt_t structure */
-    vlc_mutex_init( p_httpd, &p_httpt->host_lock );
+    if( vlc_mutex_init( p_httpd, &p_httpt->host_lock ) )
+    {
+        msg_Err( p_httpd, "Error in mutex creation");
+        return( VLC_EGENERIC );
+    }
     p_httpt->i_host_count = 0;
     p_httpt->host = NULL;
 
-    vlc_mutex_init( p_httpd, &p_httpt->file_lock );
+    if( vlc_mutex_init( p_httpd, &p_httpt->file_lock ) )
+    {
+        msg_Err( p_httpd, "Error in mutex creation");
+        return( VLC_EGENERIC );
+    }
     p_httpt->i_file_count = 0;
     p_httpt->file = NULL;
 
-    vlc_mutex_init( p_httpd, &p_httpt->connection_lock );
+    if( vlc_mutex_init( p_httpd, &p_httpt->connection_lock ) )
+    {
+        msg_Err( p_httpd, "Error in mutex creation");
+        return( VLC_EGENERIC );
+    }
     p_httpt->i_connection_count = 0;
     p_httpt->p_first_connection = NULL;
 
-    vlc_mutex_init( p_httpd, &p_httpt->ban_lock );
+    if( vlc_mutex_init( p_httpd, &p_httpt->ban_lock ) )
+    {
+        msg_Err( p_httpd, "Error in mutex creation");
+        return( VLC_EGENERIC );
+    }
+
     p_httpt->i_banned_ip_count = 0;
     p_httpt->p_first_banned_ip = NULL;
 
@@ -541,9 +580,24 @@ static httpd_host_t *_RegisterHost( httpd_sys_t *p_httpt, char *psz_host_addr, i
     {
         p_httpt->host = malloc( sizeof( httpd_host_t *) );
     }
+    if( !p_httpt->host )
+    {
+        msg_Err( p_httpt, "Out of memory" );
+        return NULL;
+    }
     p_host                = malloc( sizeof( httpd_host_t ) );
+    if( !p_host )
+    {
+        msg_Err( p_httpt, "Out of memory" );
+        return NULL;
+    }
     p_host->i_ref         = 1;
     p_host->psz_host_addr = strdup( psz_host_addr );
+    if( !p_host->psz_host_addr )
+    {
+        msg_Err( p_httpt, "Out of memory" );
+        return NULL;
+    }
     p_host->i_port        = i_port;
     p_host->sock          = sock;
     p_host->fd            = fd;
@@ -624,6 +678,11 @@ static void            _UnregisterHost( httpd_sys_t *p_httpt, httpd_host_t *p_ho
         p_httpt->i_host_count--;
         p_httpt->host = realloc( p_httpt->host,
                                  p_httpt->i_host_count * sizeof( httpd_host_t * ) );
+        if( !p_httpt->host )
+        {
+            msg_Err( p_httpt, "Out of memory" );
+            return;
+        }
     }
 
     vlc_mutex_unlock( &p_httpt->host_lock );
@@ -645,7 +704,10 @@ static void __RegisterFile( httpd_sys_t *p_httpt, httpd_file_t *p_file )
     {
         p_httpt->file = malloc( sizeof( httpd_file_t *) );
     }
-
+    if( !p_httpt->file )
+    {
+        return;
+    }
     p_httpt->file[p_httpt->i_file_count++] = p_file;
 }
 
@@ -675,14 +737,29 @@ static httpd_file_t    *_RegisterFile( httpd_sys_t *p_httpt,
     }
 
     p_file              = malloc( sizeof( httpd_file_t ) );
+    if( !p_file )
+    {
+        msg_Err( p_httpt, "Out of memory" );
+        return NULL;
+    }
     p_file->i_ref       = 0;
     p_file->psz_file    = strdup( psz_file );
     p_file->psz_mime    = strdup( psz_mime );
+    if( !p_file->psz_file || !p_file->psz_mime )
+    {
+        msg_Err( p_httpt, "Out of memory" );
+        return NULL;
+    }
     if( psz_user && *psz_user )
     {
         p_file->i_authenticate_method = HTTPD_AUTHENTICATE_BASIC;
         p_file->psz_user              = strdup( psz_user );
         p_file->psz_password          = strdup( psz_password );
+        if( !p_file->psz_user || !p_file->psz_password )
+        {
+            msg_Err( p_httpt, "Out of memory" );
+            return NULL;
+        }
     }
     else
     {
@@ -745,14 +822,29 @@ static httpd_stream_t  *_RegisterStream( httpd_sys_t *p_httpt,
     }
 
     p_stream              = malloc( sizeof( httpd_stream_t ) );
+    if( !p_stream )
+    {
+        msg_Err( p_httpt, "Out of memory" );
+        return NULL;
+    }
     p_stream->i_ref       = 0;
     p_stream->psz_file    = strdup( psz_file );
     p_stream->psz_mime    = strdup( psz_mime );
+    if( !p_stream->psz_file || !p_stream->psz_mime )
+    {
+        msg_Err( p_httpt, "Out of memory" );
+        return NULL;
+    }
     if( psz_user && *psz_user )
     {
         p_stream->i_authenticate_method = HTTPD_AUTHENTICATE_BASIC;
         p_stream->psz_user              = strdup( psz_user );
         p_stream->psz_password          = strdup( psz_password );
+        if( !p_stream->psz_user || !p_stream->psz_password )
+        {
+            msg_Err( p_httpt, "Out of memory" );
+            return NULL;
+        }
     }
     else
     {
@@ -770,6 +862,11 @@ static httpd_stream_t  *_RegisterStream( httpd_sys_t *p_httpt,
     p_stream->i_buffer_pos      = 0;
     p_stream->i_buffer_last_pos = 0;
     p_stream->p_buffer        = malloc( p_stream->i_buffer_size );
+    if( !p_stream->p_buffer )
+    {
+        msg_Err( p_httpt, "Out of memory" );
+        return NULL;
+    }
 
     p_stream->i_header_size   = 0;
     p_stream->p_header        = NULL;
@@ -882,8 +979,9 @@ static int             _SendStream( httpd_sys_t *p_httpt, httpd_stream_t *p_stre
     {
         return( VLC_SUCCESS );
     }
-    //fprintf( stderr, "## i_data=%d pos=%lld\n", i_data, p_stream->i_buffer_pos );
-
+#if 0
+    fprintf( stderr, "## i_data=%d pos=%lld\n", i_data, p_stream->i_buffer_pos );
+#endif
     vlc_mutex_lock( &p_httpt->file_lock );
 
     /* save this pointer (to be used by new connection) */
@@ -897,6 +995,7 @@ static int             _SendStream( httpd_sys_t *p_httpt, httpd_stream_t *p_stre
 
         i_copy = __MIN( i_count, p_stream->i_buffer_size - i_pos );
 
+        /* Ok, we can't go past the end of our buffer */
         memcpy( &p_stream->p_buffer[i_pos],
                 p_data,
                 i_copy );
@@ -911,6 +1010,7 @@ static int             _SendStream( httpd_sys_t *p_httpt, httpd_stream_t *p_stre
 
     return( VLC_SUCCESS );
 }
+
 static int             SendStream( httpd_t *p_httpd, httpd_stream_t *p_stream, uint8_t *p_data, int i_data )
 {
     return( _SendStream( p_httpd->p_sys, p_stream, p_data, i_data ) );
@@ -931,6 +1031,11 @@ static int             HeaderStream( httpd_t *p_httpd, httpd_stream_t *p_stream,
     {
         p_stream->i_header_size = i_data;
         p_stream->p_header = malloc( i_data );
+        if( !p_stream->p_header )
+        {
+            msg_Err( p_httpt, "Out of memory" );
+            return( VLC_ENOMEM );
+        }
         memcpy( p_stream->p_header,
                 p_data,
                 i_data );
@@ -952,14 +1057,22 @@ static void httpd_info_add_ss( httpd_info_t *p_info, char *name, char *value )
             realloc( p_info->info,
                      sizeof( httpd_val_t ) * ( p_info->i_count + 1 ) );
     }
+    if( !p_info->info )
+    {
+        return;
+    }
     p_info->info[p_info->i_count].psz_name  = strdup( name );
+    if( ! p_info->info[p_info->i_count].psz_name )
+    {
+        return;
+    }
     p_info->info[p_info->i_count].psz_value = strdup( value ? value : "(null)");
     p_info->i_count++;
 }
 
 static void httpd_info_add_si( httpd_info_t *p_info, char *name, int i_value )
 {
-    char v[40];
+    char v[40]; /* Ok, int is not so long */
 
     sprintf( v, "%d", i_value );
     httpd_info_add_ss( p_info, name, v );
@@ -1076,6 +1189,33 @@ static int Control( httpd_t *p_httpd,
 /****************************************************************************/
 /****************************************************************************/
 
+static int  httpd_page_400_get( httpd_file_callback_args_t *p_args,
+                                uint8_t *p_request, int i_request,
+                                uint8_t **pp_data, int *pi_data )
+{
+    char *p;
+
+    p = *pp_data = malloc( 1024 );
+    if( !p )
+    {
+        return VLC_ENOMEM ;
+    }
+    p += sprintf( p, "<html>\n" );
+    p += sprintf( p, "<head>\n" );
+    p += sprintf( p, "<title>Error 400</title>\n" );
+    p += sprintf( p, "</head>\n" );
+    p += sprintf( p, "<body>\n" );
+    p += sprintf( p, "<h1><center> 400  Bad Request</center></h1>\n" );
+    p += sprintf( p, "<hr />\n" );
+    p += sprintf( p, "<a href=\"http://www.videolan.org/\">VideoLAN</a>\n" );
+    p += sprintf( p, "</body>\n" );
+    p += sprintf( p, "</html>\n" );
+
+    *pi_data = strlen( *pp_data ) + 1;
+
+    return VLC_SUCCESS;
+}
+
 static int  httpd_page_401_get( httpd_file_callback_args_t *p_args,
                                 uint8_t *p_request, int i_request,
                                 uint8_t **pp_data, int *pi_data )
@@ -1083,6 +1223,10 @@ static int  httpd_page_401_get( httpd_file_callback_args_t *p_args,
     char *p;
 
     p = *pp_data = malloc( 1024 );
+    if( !p )
+    {
+        return VLC_ENOMEM ;
+    }
 
     p += sprintf( p, "<html>\n" );
     p += sprintf( p, "<head>\n" );
@@ -1091,7 +1235,7 @@ static int  httpd_page_401_get( httpd_file_callback_args_t *p_args,
     p += sprintf( p, "<body>\n" );
     p += sprintf( p, "<h1><center> 401 authentification needed</center></h1>\n" );
     p += sprintf( p, "<hr />\n" );
-    p += sprintf( p, "<a href=\"http://www.videolan.org\">VideoLAN</a>\n" );
+    p += sprintf( p, "<a href=\"http://www.videolan.org/\">VideoLAN</a>\n" );
     p += sprintf( p, "</body>\n" );
     p += sprintf( p, "</html>\n" );
 
@@ -1106,6 +1250,10 @@ static int  httpd_page_404_get( httpd_file_callback_args_t *p_args,
     char *p;
 
     p = *pp_data = malloc( 1024 );
+    if( !p )
+    {
+        return VLC_ENOMEM ;
+    }
 
     p += sprintf( p, "<html>\n" );
     p += sprintf( p, "<head>\n" );
@@ -1114,7 +1262,7 @@ static int  httpd_page_404_get( httpd_file_callback_args_t *p_args,
     p += sprintf( p, "<body>\n" );
     p += sprintf( p, "<h1><center> 404 Ressource not found</center></h1>\n" );
     p += sprintf( p, "<hr />\n" );
-    p += sprintf( p, "<a href=\"http://www.videolan.org\">VideoLAN</a>\n" );
+    p += sprintf( p, "<a href=\"http://www.videolan.org/\">VideoLAN</a>\n" );
     p += sprintf( p, "</body>\n" );
     p += sprintf( p, "</html>\n" );
 
@@ -1123,7 +1271,7 @@ static int  httpd_page_404_get( httpd_file_callback_args_t *p_args,
     return VLC_SUCCESS;
 }
 
-
+#if 0
 static int httpd_BanIP( httpd_sys_t *p_httpt, char * psz_new_banned_ip)
 {
     httpd_banned_ip_t *p_new_banned_ip ;
@@ -1167,7 +1315,7 @@ static int httpd_BanIP( httpd_sys_t *p_httpt, char * psz_new_banned_ip)
     p_httpt->i_banned_ip_count++;
     return 0;
 }
-
+#endif
 static httpd_banned_ip_t *httpd_GetbannedIP( httpd_sys_t *p_httpt, char *psz_ip )
 {
     httpd_banned_ip_t *p_ip;
@@ -1234,6 +1382,11 @@ static void httpd_ConnnectionNew( httpd_sys_t *p_httpt, int fd, struct sockaddr_
 
     /* create a new connection and link it */
     p_con = malloc( sizeof( httpd_connection_t ) );
+    if( !p_con )
+    {
+        msg_Err( p_httpt, "Out of memory" );
+        return;
+    }
     p_con->i_state  = HTTPD_CONNECTION_RECEIVING_REQUEST;
     p_con->fd       = fd;
     p_con->i_last_activity_date = mdate();
@@ -1251,8 +1404,13 @@ static void httpd_ConnnectionNew( httpd_sys_t *p_httpt, int fd, struct sockaddr_
     p_con->i_buffer = 0;
     p_con->i_buffer_size = 8096;
     p_con->p_buffer = malloc( p_con->i_buffer_size );
+    if( !p_con->p_buffer )
+    {
+        msg_Err( p_httpt, "Out of memory");
+        return ;
+    }
 
-    p_con->i_stream_pos = 0; // updated by httpd_thread */
+    p_con->i_stream_pos = 0; /* updated by httpd_thread */
     p_con->p_next = NULL;
 
     if( p_httpt->p_first_connection )
@@ -1353,7 +1511,7 @@ static int httpd_RequestNextLine( char **pp_buffer, char *p_end )
     return VLC_EGENERIC;
 }
 
-//char b64[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+/*char b64[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";*/
 static void b64_decode( char *dest, char *src )
 {
     int  i_level;
@@ -1417,13 +1575,15 @@ static void httpd_ConnectionParseRequest( httpd_sys_t *p_httpt, httpd_connection
     char *p, *p_end;
 
     int  i;
+    int  b_xplaystream = VLC_FALSE;
+    /* Size is checked for all of these */
     char command[32];
     char url[1024];
     char version[32];
     char user[512] = "";
     char password[512] = "";
 
-    //msg_Dbg( p_httpt, "new request=\n%s", p_con->p_buffer );
+    msg_Dbg( p_httpt, "new request=\n%s", p_con->p_buffer );
 
 
     p = p_con->p_buffer;
@@ -1432,8 +1592,9 @@ static void httpd_ConnectionParseRequest( httpd_sys_t *p_httpt, httpd_connection
     httpd_RequestGetWord( command, 32, &p, p_end );
     httpd_RequestGetWord( url, 1024, &p, p_end );
     httpd_RequestGetWord( version, 32, &p, p_end );
-    //msg_Dbg( p_httpt, "ask =%s= =%s= =%s=", command, url, version );
-
+#if 0
+    msg_Dbg( p_httpt, "ask =%s= =%s= =%s=", command, url, version );
+#endif
     p_con->p_request      = NULL;
     p_con->i_request_size = 0;
     if( !strcmp( command, "GET" ) )
@@ -1444,6 +1605,10 @@ static void httpd_ConnectionParseRequest( httpd_sys_t *p_httpt, httpd_connection
     {
         p_con->i_method = HTTPD_CONNECTION_METHOD_POST;
     }
+    else if( !strcmp( command, "HEAD" ))
+    {
+        p_con->i_method = HTTPD_CONNECTION_METHOD_HEAD;
+    }
     else
     {
         /* unimplemented */
@@ -1468,11 +1633,14 @@ static void httpd_ConnectionParseRequest( httpd_sys_t *p_httpt, httpd_connection
 
         if( httpd_RequestNextLine( &p, p_end ) )
         {
-            //msg_Dbg( p_httpt, "failled new line" );
+#if 0
+            msg_Dbg( p_httpt, "failled new line" );
+#endif
             break;;
         }
-        //msg_Dbg( p_httpt, "new line=%s", p );
-
+#if 0
+        msg_Dbg( p_httpt, "new line=%s", p );
+#endif
         httpd_RequestGetWord( header, 1024, &p, p_end );
         if( !strcmp( header, "\r\n" ) || !strcmp( header, "\n" ) )
         {
@@ -1490,9 +1658,13 @@ static void httpd_ConnectionParseRequest( httpd_sys_t *p_httpt, httpd_connection
                 char decoded[1024];
 
                 httpd_RequestGetWord( basic, 1024, &p, p_end );
-//                msg_Dbg( p_httpt, "Authorization: basic:%s", basic );
+#if 0
+                msg_Dbg( p_httpt, "Authorization: basic:%s", basic );
+#endif
                 b64_decode( decoded, basic );
-//                msg_Dbg( p_httpt, "Authorization: decoded:%s", decoded );
+#if 0
+                msg_Dbg( p_httpt, "Authorization: decoded:%s", decoded );
+#endif
                 if( strchr( decoded, ':' ) )
                 {
                     char *p = strchr( decoded, ':' );
@@ -1503,6 +1675,16 @@ static void httpd_ConnectionParseRequest( httpd_sys_t *p_httpt, httpd_connection
                 }
             }
         }
+        else if( !strcmp( header, "Pragma:" ) )
+        {
+            char method[128];
+
+            httpd_RequestGetWord( method, 128, &p, p_end );
+            if( !strcasecmp( method, "xPlayStrm=1" ) )
+            {
+                b_xplaystream = VLC_TRUE;
+            }
+        }
     }
 
     if( strchr( url, '?' ) )
@@ -1538,7 +1720,11 @@ static void httpd_ConnectionParseRequest( httpd_sys_t *p_httpt, httpd_connection
         {
             p_con->i_request_size = p_end - p_request;
             p_con->p_request = malloc( p_con->i_request_size + 1);
-
+            if( !p_con->p_request)
+            {
+                msg_Err( p_httpt, "Out of memory" );
+                return;
+            }
             memcpy( p_con->p_request,
                     p_request,
                     p_con->i_request_size );
@@ -1549,20 +1735,27 @@ static void httpd_ConnectionParseRequest( httpd_sys_t *p_httpt, httpd_connection
     p_con->i_http_error = 200;
 
 create_header:
-    //msg_Dbg( p_httpt, "ask %s %s %d", command, p_con->psz_file, p_con->i_http_error );
+#if 0
+    msg_Dbg( p_httpt, "ask %s %s %d", command, p_con->psz_file, p_con->i_http_error );
+#endif
     FREE( p_con->p_buffer );
     p_con->i_buffer = 0;
     p_con->i_buffer_size = 0;
 
-    //vlc_mutex_lock( &p_httpt->file_lock );
+#if 0
+    vlc_mutex_lock( &p_httpt->file_lock );
+#endif
 search_file:
     /* search file */
     p_con->p_file = NULL;
     for( i = 0; i < p_httpt->i_file_count; i++ )
     {
+        /* Our strdup call failed */
+        if( !p_con->psz_file ) return;
         if( !strcmp( p_httpt->file[i]->psz_file, p_con->psz_file ) )
         {
             if( p_httpt->file[i]->b_stream ||
+                p_con->i_method == HTTPD_CONNECTION_METHOD_HEAD ||
                 ( p_con->i_method == HTTPD_CONNECTION_METHOD_GET  && p_httpt->file[i]->pf_get ) ||
                 ( p_con->i_method == HTTPD_CONNECTION_METHOD_POST && p_httpt->file[i]->pf_post ) )
             {
@@ -1576,33 +1769,48 @@ search_file:
     {
         p_con->psz_file = strdup( "/404.html" );
         p_con->i_http_error = 404;
+        p_con->i_method = HTTPD_CONNECTION_METHOD_GET;
 
         /* XXX be sure that "/404.html" exist else ... */
         goto search_file;
     }
-
     if( p_con->p_file->i_authenticate_method == HTTPD_AUTHENTICATE_BASIC )
     {
-        if( strcmp( user, p_con->p_file->psz_user ) || strcmp( password, p_con->p_file->psz_password ) )
+        if( strcmp( user, p_con->p_file->psz_user ) ||
+            strcmp( password, p_con->p_file->psz_password ) )
         {
             p_con->psz_file = strdup( "/401.html" );
             strcpy( user, p_con->p_file->psz_user );
             p_con->i_http_error = 401;
+            p_con->i_method = HTTPD_CONNECTION_METHOD_GET;
 
             /* XXX do not put password on 404 else ... */
             goto search_file;
         }
     }
+    if( !strcmp( p_con->p_file->psz_mime, "video/x-ms-asf-stream" ) &&
+        p_con->i_method == HTTPD_CONNECTION_METHOD_POST )
+    {
+        p_con->psz_file = strdup( "/400.html" );
+        p_con->i_http_error = 400;
+        p_con->i_method = HTTPD_CONNECTION_METHOD_GET;
+
+        goto search_file;
+    }
 
     p_con->p_file->i_ref++;
-//    vlc_mutex_unlock( &p_httpt->file_lock );
+#if 0
+    vlc_mutex_unlock( &p_httpt->file_lock );
+#endif
 
     switch( p_con->i_http_error )
     {
         case 200:
             psz_status = "OK";
             break;
-
+        case 400:
+            psz_status = "Bad Request";
+            break;
         case 401:
             psz_status = "Authorization Required";
             break;
@@ -1623,19 +1831,56 @@ search_file:
     }
 
     p = p_con->p_buffer = malloc( p_con->i_buffer_size );
-
+    if( !p)
+    {
+        msg_Err( p_httpt, "Out of memory" );
+        return;
+    }
     p += sprintf( p, "HTTP/1.0 %d %s\r\n", p_con->i_http_error, psz_status );
-    p += sprintf( p, "Content-type: %s\r\n", p_con->p_file->psz_mime );
+
+    /* Special mmsh case cludgy but ...*/
+    if( !strcmp( p_con->p_file->psz_mime, "video/x-ms-asf-stream" ) &&
+        p_con->i_method == HTTPD_CONNECTION_METHOD_GET )
+    {
+        p += sprintf( p, "Content-type: application/octet-stream\r\n" );
+        p += sprintf( p, "Server: Cougar 4.1.0.3921\r\n" );
+        p += sprintf( p, "Cache-Control: no-cache\r\n" );
+        p += sprintf( p, "Pragma: no-cache\r\n" );
+        p += sprintf( p, "Pragma: client-id=%d\r\n", rand()&0x7fff );
+        p += sprintf( p, "Pragma: features=\"broadcast\"\r\n" );
+
+        if( !b_xplaystream )
+        {
+            p_con->i_method = HTTPD_CONNECTION_METHOD_ASFHEAD;
+        }
+    }
+    else
+    {
+        p += sprintf( p, "Content-type: %s\r\n", p_con->p_file->psz_mime );
+        p += sprintf( p, "Cache-Control: no-cache\r\n" );
+    }
     if( p_con->i_http_error == 401 )
     {
         p += sprintf( p, "WWW-Authenticate: Basic realm=\"%s\"\r\n", user );
     }
-    p += sprintf( p, "Cache-Control: no-cache\r\n" );
+#if 0
+    if( p_con->i_method == HTTPD_CONNECTION_METHOD_HEAD )
+    {
+        p += sprintf( p, "Content-Length: 0\r\n" );
+    }
+    else if( p_con->i_method == HTTPD_CONNECTION_METHOD_ASFHEAD &&
+             p_con->p_file->i_header_size > 0 )
+    {
+        p += sprintf( p, "Content-Length: %d\r\n", p_con->p_file->i_header_size );
+    }
+#endif
     p += sprintf( p, "\r\n" );
 
     p_con->i_buffer_size = strlen( p_con->p_buffer );// + 1;
 
-    if( p_con->i_http_error == 200 && p_con->p_file->b_stream && p_con->p_file->i_header_size > 0 )
+    if( p_con->i_http_error == 200 &&
+        p_con->i_method != HTTPD_CONNECTION_METHOD_HEAD &&
+        p_con->p_file->b_stream && p_con->p_file->i_header_size > 0 )
     {
         /* add stream header */
         memcpy( &p_con->p_buffer[p_con->i_buffer_size],
@@ -1644,11 +1889,18 @@ search_file:
         p_con->i_buffer_size += p_con->p_file->i_header_size;
     }
 
-    //msg_Dbg( p_httpt, "answer=\n%s", p_con->p_buffer );
+    if( p_con->i_method == HTTPD_CONNECTION_METHOD_ASFHEAD )
+    {
+        p_con->i_method = HTTPD_CONNECTION_METHOD_HEAD;
+    }
+#if 0
+    msg_Dbg( p_httpt, "answer=\n%s", p_con->p_buffer );
+#endif
 }
 #define HTTPD_STREAM_PACKET 10000
 static void httpd_Thread( httpd_sys_t *p_httpt )
 {
+    httpd_file_t    *p_page_400;
     httpd_file_t    *p_page_401;
     httpd_file_t    *p_page_404;
 
@@ -1656,6 +1908,13 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
 
     msg_Info( p_httpt, "httpd started" );
 
+    p_page_400 = _RegisterFile( p_httpt,
+                                "/400.html", "text/html",
+                                NULL, NULL,
+                                httpd_page_400_get,
+                                NULL,
+                                (httpd_file_callback_args_t*)NULL );
+
     p_page_401 = _RegisterFile( p_httpt,
                                 "/401.html", "text/html",
                                 NULL, NULL,
@@ -1845,7 +2104,9 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
                 {
                     i_len = 0;
                 }
-//                msg_Warn( p_httpt, "on %d send %d bytes %s", p_con->i_buffer_size, i_len, p_con->p_buffer + p_con->i_buffer );
+#if 0
+                msg_Warn( p_httpt, "on %d send %d bytes %s", p_con->i_buffer_size, i_len, p_con->p_buffer + p_con->i_buffer );
+#endif
 
 #if defined( WIN32 ) || defined( UNDER_CE )
                 if( ( i_len < 0 && WSAGetLastError() != WSAEWOULDBLOCK ) || ( i_len == 0 ) )
@@ -1871,9 +2132,9 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
                             p_con->i_buffer = 0;
                             FREE( p_con->p_buffer );
 
-                            if( !p_con->p_file->b_stream )
+                            if( !p_con->p_file->b_stream || p_con->i_method == HTTPD_CONNECTION_METHOD_HEAD )
                             {
-                                p_con->i_state = HTTPD_CONNECTION_SENDING_FILE; // be sure to out from HTTPD_CONNECTION_SENDING_HEADER
+                                p_con->i_state = HTTPD_CONNECTION_SENDING_FILE; /* be sure to out from HTTPD_CONNECTION_SENDING_HEADER */
                                 if( p_con->i_method == HTTPD_CONNECTION_METHOD_GET )
                                 {
                                     p_con->p_file->pf_get( p_con->p_file->p_sys,
@@ -1888,6 +2149,7 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
                                 }
                                 else
                                 {
+                                    /* HTTPD_CONNECTION_METHOD_HEAD for example */
                                     p_con->p_buffer = NULL;
                                     p_con->i_buffer_size = 0;
                                 }
@@ -1983,6 +2245,7 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
 
     msg_Info( p_httpt, "httpd stopped" );
 
+    _UnregisterFile( p_httpt, p_page_400 );
     _UnregisterFile( p_httpt, p_page_401 );
     _UnregisterFile( p_httpt, p_page_404 );
 }
