@@ -2,10 +2,11 @@
  * ipv4.c: IPv4 network abstraction layer
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: ipv4.c,v 1.20 2003/07/31 23:44:49 fenrir Exp $
+ * $Id: ipv4.c,v 1.25 2004/02/22 23:09:25 titer Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Mathias Kretschmer <mathias@research.att.com>
+ *          Alexis de Lattre <alexis@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,6 +74,7 @@
 #   define IN_MULTICAST(a) IN_CLASSD(a)
 #endif
 
+
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
@@ -152,6 +154,18 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
     socklen_t i_opt_size;
     struct sockaddr_in sock;
 
+    /* If IP_ADD_SOURCE_MEMBERSHIP is not defined in the headers
+       (because it's not in glibc for example), we have to define the
+       headers required for IGMPv3 here */
+#ifndef IP_ADD_SOURCE_MEMBERSHIP
+    #define IP_ADD_SOURCE_MEMBERSHIP  39
+    struct ip_mreq_source {
+        struct in_addr  imr_multiaddr;
+        struct in_addr  imr_interface;
+        struct in_addr  imr_sourceaddr;
+     };
+#endif
+
     /* Open a SOCK_DGRAM (UDP) socket, in the AF_INET domain, automatic (0)
      * protocol */
     if( (i_handle = socket( AF_INET, SOCK_DGRAM, 0 )) == -1 )
@@ -182,12 +196,8 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
     /* Increase the receive buffer size to 1/2MB (8Mb/s during 1/2s) to avoid
      * packet loss caused by scheduling problems */
     i_opt = 0x80000;
-#if defined( SYS_BEOS )
-    if( setsockopt( i_handle, SOL_SOCKET, SO_NONBLOCK,
-#else
-    if( setsockopt( i_handle, SOL_SOCKET, SO_RCVBUF,
-#endif
-                    (void *) &i_opt, sizeof( i_opt ) ) == -1 )
+#if !defined( SYS_BEOS )
+    if( setsockopt( i_handle, SOL_SOCKET, SO_RCVBUF, (void *) &i_opt, sizeof( i_opt ) ) == -1 )
     {
 #ifdef HAVE_ERRNO_H
         msg_Dbg( p_this, "cannot configure socket (SO_RCVBUF: %s)",
@@ -196,18 +206,15 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
         msg_Warn( p_this, "cannot configure socket (SO_RCVBUF)" );
 #endif
     }
+#endif
 
+#if !defined( SYS_BEOS )
     /* Check if we really got what we have asked for, because Linux, etc.
      * will silently limit the max buffer size to net.core.rmem_max which
      * is typically only 65535 bytes */
     i_opt = 0;
     i_opt_size = sizeof( i_opt );
-#if defined( SYS_BEOS )
-    if( getsockopt( i_handle, SOL_SOCKET, SO_NONBLOCK,
-#else
-    if( getsockopt( i_handle, SOL_SOCKET, SO_RCVBUF,
-#endif
-                    (void*) &i_opt, &i_opt_size ) == -1 )
+    if( getsockopt( i_handle, SOL_SOCKET, SO_RCVBUF, (void*) &i_opt, &i_opt_size ) == -1 )
     {
 #ifdef HAVE_ERRNO_H
         msg_Warn( p_this, "cannot query socket (SO_RCVBUF: %s)",
@@ -221,6 +228,7 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
         msg_Dbg( p_this, "socket buffer size is 0x%x instead of 0x%x",
                          i_opt, 0x80000 );
     }
+#endif
 
 
     /* Build the local socket */
@@ -264,16 +272,12 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
     }
 #endif
 
+#if !defined( SYS_BEOS )
     /* Allow broadcast reception if we bound on INADDR_ANY */
     if( !*psz_bind_addr )
     {
         i_opt = 1;
-#if defined( SYS_BEOS )
-        if( setsockopt( i_handle, SOL_SOCKET, SO_NONBLOCK,
-#else
-        if( setsockopt( i_handle, SOL_SOCKET, SO_BROADCAST,
-#endif
-                        (void*) &i_opt, sizeof( i_opt ) ) == -1 )
+        if( setsockopt( i_handle, SOL_SOCKET, SO_BROADCAST, (void*) &i_opt, sizeof( i_opt ) ) == -1 )
         {
 #ifdef HAVE_ERRNO_H
             msg_Warn( p_this, "cannot configure socket (SO_BROADCAST: %s)",
@@ -283,42 +287,88 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
 #endif
         }
     }
+#endif
 
 #if !defined( UNDER_CE ) && !defined( SYS_BEOS )
     /* Join the multicast group if the socket is a multicast address */
     if( IN_MULTICAST( ntohl(sock.sin_addr.s_addr) ) )
     {
-        struct ip_mreq imr;
-
         /* Determine interface to be used for multicast */
         char * psz_if_addr = config_GetPsz( p_this, "iface-addr" );
-        imr.imr_multiaddr.s_addr = sock.sin_addr.s_addr;
-        if( psz_if_addr != NULL && *psz_if_addr
-            && inet_addr(psz_if_addr) != INADDR_NONE )
-        {
-            imr.imr_interface.s_addr = inet_addr(psz_if_addr);
-        }
-        else
-        {
-            imr.imr_interface.s_addr = INADDR_ANY;
-        }
-        if( psz_if_addr != NULL ) free( psz_if_addr );
 
-        /* Join Multicast group */
-        if( setsockopt( i_handle, IPPROTO_IP, IP_ADD_MEMBERSHIP,
-                        (char*)&imr, sizeof(struct ip_mreq) ) == -1 )
+        /* If we have a source address, we use IP_ADD_SOURCE_MEMBERSHIP
+           so that IGMPv3 aware OSes running on IGMPv3 aware networks
+           will do an IGMPv3 query on the network */
+        if( *psz_server_addr )
         {
+            struct ip_mreq_source imr;
+
+            imr.imr_multiaddr.s_addr = sock.sin_addr.s_addr;
+            imr.imr_sourceaddr.s_addr = inet_addr(psz_server_addr);
+
+            if( psz_if_addr != NULL && *psz_if_addr
+                && inet_addr(psz_if_addr) != INADDR_NONE )
+            {
+                imr.imr_interface.s_addr = inet_addr(psz_if_addr);
+            }
+            else
+            {
+                imr.imr_interface.s_addr = INADDR_ANY;
+            }
+            if( psz_if_addr != NULL ) free( psz_if_addr );
+
+            msg_Dbg( p_this, "IP_ADD_SOURCE_MEMBERSHIP multicast request" );
+            /* Join Multicast group with source filter */
+            if( setsockopt( i_handle, IPPROTO_IP, IP_ADD_SOURCE_MEMBERSHIP,
+                         (char*)&imr,
+                         sizeof(struct ip_mreq_source) ) == -1 )
+            {
 #ifdef HAVE_ERRNO_H
-            msg_Err( p_this, "failed to join IP multicast group (%s)",
-                              strerror(errno) );
+                msg_Err( p_this, "failed to join IP multicast group (%s)",
+                                  strerror(errno) );
+                msg_Err( p_this, "are you sure your OS supports IGMPv3?" );
 #else
-            msg_Err( p_this, "failed to join IP multicast group" );
+                msg_Err( p_this, "failed to join IP multicast group" );
+                msg_Err( p_this, "are you sure your OS supports IGMPv3?" );
 #endif
-            close( i_handle );
-            return( -1 );
-        }
+                close( i_handle );
+                return( -1 );
+            }
+         }
+         /* If there is no source address, we use IP_ADD_MEMBERSHIP */
+         else
+         {
+             struct ip_mreq imr;
+
+             imr.imr_multiaddr.s_addr = sock.sin_addr.s_addr;
+             if( psz_if_addr != NULL && *psz_if_addr
+                && inet_addr(psz_if_addr) != INADDR_NONE )
+            {
+                imr.imr_interface.s_addr = inet_addr(psz_if_addr);
+            }
+            else
+            {
+                imr.imr_interface.s_addr = INADDR_ANY;
+            }
+            if( psz_if_addr != NULL ) free( psz_if_addr );
+
+            msg_Dbg( p_this, "IP_ADD_MEMBERSHIP multicast request" );
+            /* Join Multicast group without source filter */
+            if( setsockopt( i_handle, IPPROTO_IP, IP_ADD_MEMBERSHIP,
+                            (char*)&imr, sizeof(struct ip_mreq) ) == -1 )
+            {
+#ifdef HAVE_ERRNO_H
+                msg_Err( p_this, "failed to join IP multicast group (%s)",
+                                  strerror(errno) );
+#else
+                msg_Err( p_this, "failed to join IP multicast group" );
+#endif
+                close( i_handle );
+                return( -1 );
+            }
+         }
     }
-#endif /* UNDER_CE, SYS_BEOS */
+#endif
 
     if( *psz_server_addr )
     {
@@ -366,7 +416,7 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
                 return( -1 );
             }
         }
-#endif /* UNDER_CE, SYS_BEOS */
+#endif
     }
 
     p_socket->i_handle = i_handle;
@@ -404,34 +454,116 @@ static int OpenTCP( vlc_object_t * p_this, network_socket_t * p_socket )
 #else
         msg_Warn( p_this, "cannot create socket" );
 #endif
-        return( -1 );
+        goto error;
     }
 
     /* Build remote address */
     if ( BuildAddr( &sock, psz_server_addr, i_server_port ) == -1 )
     {
         msg_Dbg( p_this, "could not build local address" );
-        close( i_handle );
-        return( -1 );
+        goto error;
     }
 
-    /* Connect the socket */
-    if( connect( i_handle, (struct sockaddr *) &sock,
-                 sizeof( sock ) ) == (-1) )
+    /* Set to non-blocking */
+#if defined( WIN32 ) || defined( UNDER_CE )
     {
-#ifdef HAVE_ERRNO_H
-        msg_Warn( p_this, "cannot connect socket (%s)", strerror(errno) );
-#else
-        msg_Warn( p_this, "cannot connect socket" );
+        unsigned long i_dummy = 1;
+        if( ioctlsocket( i_handle, FIONBIO, &i_dummy ) != 0 )
+        {
+            msg_Err( p_this, "cannot set socket to non-blocking mode" );
+        }
+    }
+#elif defined( HAVE_ERRNO_H )
+    {
+        int i_flags;
+        if( ( i_flags = fcntl( i_handle, F_GETFL, 0 ) ) < 0 ||
+            fcntl( i_handle, F_SETFL, i_flags | O_NONBLOCK ) < 0 )
+        {
+            msg_Err( p_this, "cannot set socket to non-blocking mode" );
+        }
+    }
 #endif
-        close( i_handle );
-        return( -1 );
+
+    /* Connect the socket */
+    if( connect( i_handle, (struct sockaddr *) &sock, sizeof( sock ) ) == -1 )
+    {
+#if defined( WIN32 ) || defined( UNDER_CE )
+        if( WSAGetLastError() == WSAEWOULDBLOCK )
+#elif defined( HAVE_ERRNO_H )
+        if( errno == EINPROGRESS )
+#else
+        if( 0 )
+#endif
+        {
+            int i_ret;
+            int i_opt;
+            int i_opt_size = sizeof( i_opt );
+            struct timeval  timeout;
+            fd_set          fds;
+
+            msg_Dbg( p_this, "connection in progress" );
+            do
+            {
+                if( p_this->b_die )
+                {
+                    msg_Dbg( p_this, "connection aborted" );
+                    goto error;
+                }
+
+                /* Initialize file descriptor set */
+                FD_ZERO( &fds );
+                FD_SET( i_handle, &fds );
+
+                /* We'll wait 0.1 second if nothing happens */
+                timeout.tv_sec = 0;
+                timeout.tv_usec = 100000;
+
+            } while( ( i_ret = select( i_handle + 1, NULL, &fds, NULL,
+                                       &timeout ) ) == 0 ||
+#if defined( WIN32 ) || defined( UNDER_CE )
+                     ( i_ret < 0 && WSAGetLastError() == WSAEWOULDBLOCK ) );
+#elif defined( HAVE_ERRNO_H )
+                     ( i_ret < 0 && errno == EINTR ) );
+#else
+                     ( i_ret < 0 ) );
+#endif
+
+            if( i_ret < 0 )
+            {
+                msg_Warn( p_this, "cannot connect socket (select failed)" );
+                goto error;
+            }
+
+#if !defined( SYS_BEOS )
+            if( getsockopt( i_handle, SOL_SOCKET, SO_ERROR, (void*)&i_opt,
+                            &i_opt_size ) == -1 || i_opt != 0 )
+            {
+                msg_Warn( p_this, "cannot connect socket (SO_ERROR)" );
+                goto error;
+            }
+#endif
+        }
+        else
+        {
+#if defined( HAVE_ERRNO_H )
+            msg_Warn( p_this, "cannot connect socket (%s)", strerror(errno) );
+#else
+            msg_Warn( p_this, "cannot connect socket" );
+#endif
+            goto error;
+        }
     }
 
     p_socket->i_handle = i_handle;
     p_socket->i_mtu = 0; /* There is no MTU notion in TCP */
+    return VLC_SUCCESS;
 
-    return( 0 );
+error:
+    if( i_handle > 0 )
+    {
+        close( i_handle );
+    }
+    return VLC_EGENERIC;
 }
 
 /*****************************************************************************
