@@ -2,7 +2,7 @@
  * ts.c: MPEG-II TS Muxer
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: ts.c,v 1.41 2003/11/27 22:44:50 massiot Exp $
+ * $Id: ts.c,v 1.45 2004/02/22 15:57:41 fenrir Exp $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Eric Petit <titer@videolan.org>
@@ -32,9 +32,11 @@
 #include <vlc/input.h>
 #include <vlc/sout.h>
 
-#include "codecs.h"
+#include "iso_lang.h"
+
 #include "bits.h"
 #include "pes.h"
+#include "csa.h"
 
 #if defined MODULE_NAME_IS_mux_ts_dvbpsi
 #   ifdef HAVE_DVBPSI_DR_H
@@ -102,6 +104,7 @@ static int     Mux      ( sout_mux_t * );
  * Local prototypes
  *****************************************************************************/
 #define SOUT_BUFFER_FLAGS_PRIVATE_PCR  ( 1 << SOUT_BUFFER_FLAGS_PRIVATE_SHIFT )
+#define SOUT_BUFFER_FLAGS_PRIVATE_CSA  ( 2 << SOUT_BUFFER_FLAGS_PRIVATE_SHIFT )
 typedef struct
 {
     int           i_depth;
@@ -173,6 +176,9 @@ typedef struct ts_stream_s
     int             i_decoder_specific_info;
     uint8_t         *p_decoder_specific_info;
 
+    /* language is iso639-2T */
+    uint8_t         lang[3];
+
     sout_buffer_chain_t chain_pes;
     mtime_t             i_pes_dts;
     mtime_t             i_pes_length;
@@ -212,6 +218,8 @@ struct sout_mux_sys_t
     int64_t             i_dts_delay;
 
     mtime_t             i_pcr;  /* last PCR emited */
+
+    csa_t               *csa;
 };
 
 
@@ -366,6 +374,43 @@ static int Open( vlc_object_t *p_this )
 
     /* for TS generation */
     p_sys->i_pcr    = 0;
+
+    p_sys->csa      = NULL;
+    if( ( val = sout_cfg_find_value( p_mux->p_cfg, "csa-ck" ) ) )
+    {
+        /* skip 0x */
+        if( val[0] == '0' && ( val[1] == 'x' || val[1] == 'X' ) )
+        {
+            val += 2;
+        }
+        if( strlen( val ) != 16 )
+        {
+            msg_Dbg( p_mux, "invalid csa ck (it must be 16 chars long)" );
+        }
+        else
+        {
+            /* Avoid using strtoll */
+            uint64_t i_ck;
+            uint8_t  ck[8];
+            int      i;
+
+            ck[0] = val[8];
+            val[8] = 0;
+            i_ck = ((int64_t)strtol( val, NULL, 16 )) << 32;
+            val[8] = ck[0];
+            i_ck += strtol( &val[8], NULL, 16 );
+            for( i = 0; i < 8; i++ )
+            {
+                ck[i] = ( i_ck >> ( 56 - 8*i) )&0xff;
+            }
+
+            msg_Dbg( p_mux, "using CSA scrambling with ck=%x:%x:%x:%x:%x:%x:%x:%x",
+                     ck[0], ck[1], ck[2], ck[3], ck[4], ck[5], ck[6], ck[7] );
+
+            p_sys->csa = csa_New();
+            csa_SetCW( p_sys->csa, ck, ck );
+        }
+    }
     return VLC_SUCCESS;
 }
 
@@ -378,6 +423,10 @@ static void Close( vlc_object_t * p_this )
     sout_mux_sys_t      *p_sys = p_mux->p_sys;
 
     msg_Dbg( p_mux, "Close" );
+    if( p_sys->csa )
+    {
+        csa_Delete( p_sys->csa );
+    }
 
     free( p_sys );
 }
@@ -506,6 +555,40 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
             return VLC_EGENERIC;
     }
 
+    p_stream->lang[0] =
+    p_stream->lang[1] =
+    p_stream->lang[2] = '\0';
+    if( p_input->p_fmt->psz_language )
+    {
+        char *psz = p_input->p_fmt->psz_language;
+        const iso639_lang_t *pl = NULL;
+
+        if( strlen( psz ) == 2 )
+        {
+            pl = GetLang_1( psz );
+        }
+        else if( strlen( psz ) == 3 )
+        {
+            pl = GetLang_2B( psz );
+            if( !strcmp( pl->psz_iso639_1, "??" ) )
+            {
+                pl = GetLang_2T( psz );
+            }
+        }
+        if( pl && strcmp( pl->psz_iso639_1, "??" ) )
+        {
+            p_stream->lang[0] = pl->psz_iso639_2T[0];
+            p_stream->lang[1] = pl->psz_iso639_2T[1];
+            p_stream->lang[2] = pl->psz_iso639_2T[2];
+
+            msg_Dbg( p_mux, "    - lang=%c%c%c",
+                     p_stream->lang[0],
+                     p_stream->lang[1],
+                     p_stream->lang[2] );
+        }
+    }
+
+
     /* Copy extra data (VOL for MPEG-4 and extra BitMapInfoHeader for VFW */
     p_stream->i_decoder_specific_info = p_input->p_fmt->i_extra;
     if( p_stream->i_decoder_specific_info > 0 )
@@ -721,6 +804,12 @@ static int Mux( sout_mux_t *p_mux )
                     }
                     else
                     {
+                        if( p_data->i_length < 0 || p_data->i_length > 2000000 )
+                        {
+                            /* FIXME choose a better value, but anyway we should never
+                             * have to do that */
+                            p_data->i_length = 1000;
+                        }
                         p_stream->i_pes_length += p_data->i_length;
                         if( p_stream->i_pes_dts == 0 )
                         {
@@ -825,6 +914,10 @@ static int Mux( sout_mux_t *p_mux )
 
             /* Build the TS packet */
             p_ts = TSNew( p_mux, p_stream, b_pcr );
+            if( p_sys->csa )
+            {
+                p_ts->i_flags |= SOUT_BUFFER_FLAGS_PRIVATE_CSA;
+            }
             i_packet_pos++;
 
             /* */
@@ -846,6 +939,10 @@ static int Mux( sout_mux_t *p_mux )
                 /* msg_Dbg( p_mux, "pcr=%lld ms", p_ts->i_dts / 1000 ); */
                 TSSetPCR( p_ts, p_ts->i_dts - p_sys->i_dts_delay );
             }
+            if( p_ts->i_flags&SOUT_BUFFER_FLAGS_PRIVATE_CSA )
+            {
+                csa_Encrypt( p_sys->csa, p_ts->p_buffer, 0 );
+            }
 
             /* latency */
             p_ts->i_dts += 3*p_sys->i_caching_delay/2;
@@ -1580,6 +1677,20 @@ static void GetPMT( sout_mux_t *p_mux,
             /* "registration" descriptor : "AC-3" */
             dvbpsi_PMTESAddDescriptor( p_es, 0x05, 4, format );
         }
+
+        if( p_stream->lang[0] != 0 )
+        {
+            uint8_t data[4];
+
+            /* I construct the content myself, way faster than looking at
+             * over complicated/mind broken libdvbpsi way */
+            data[0] = p_stream->lang[0];
+            data[1] = p_stream->lang[1];
+            data[2] = p_stream->lang[2];
+            data[3] = 0x00; /* audio type: 0x00 undefined */
+
+            dvbpsi_PMTESAddDescriptor( p_es, 0x0a, 4, data );
+        }
     }
 
     p_section = dvbpsi_GenPMTSections( &pmt );
