@@ -1,8 +1,8 @@
 /*****************************************************************************
- * transcode.c
+ * transcode.c: transcoding stream output module
  *****************************************************************************
- * Copyright (C) 2001, 2002 VideoLAN
- * $Id: transcode.c,v 1.65 2004/01/03 00:39:07 gbazin Exp $
+ * Copyright (C) 2003-2004 VideoLAN
+ * $Id: transcode.c,v 1.80 2004/02/24 17:42:07 gbazin Exp $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@netcourrier.com>
@@ -61,6 +61,8 @@ static int  transcode_video_ffmpeg_process( sout_stream_t *, sout_stream_id_t *,
 
 static int  transcode_video_ffmpeg_getframebuf( struct AVCodecContext *, AVFrame *);
 
+static int  EncoderThread( struct sout_stream_sys_t * p_sys );
+
 static int pi_channels_maps[6] =
 {
     0,
@@ -76,15 +78,25 @@ static int pi_channels_maps[6] =
  * Module descriptor
  *****************************************************************************/
 vlc_module_begin();
-    set_description( _("Transcode stream") );
+    set_description( _("Transcode stream output") );
     set_capability( "sout stream", 50 );
     add_shortcut( "transcode" );
     set_callbacks( Open, Close );
 vlc_module_end();
 
+#define PICTURE_RING_SIZE 64
+
 struct sout_stream_sys_t
 {
-    sout_stream_t   *p_out;
+    VLC_COMMON_MEMBERS
+
+    sout_stream_t * p_out;
+    sout_stream_id_t * id_video;
+    sout_buffer_t * p_buffers;
+    vlc_mutex_t     lock_out;
+    vlc_cond_t      cond;
+    picture_t *     pp_pics[PICTURE_RING_SIZE];
+    int             i_first_pic, i_last_pic;
 
     vlc_fourcc_t    i_acodec;   /* codec audio (0 if not transcode) */
     int             i_sample_rate;
@@ -103,9 +115,17 @@ struct sout_stream_sys_t
     int             i_qmax;
     vlc_bool_t      i_hq;
     vlc_bool_t      b_deinterlace;
+    vlc_bool_t      b_interlace;
     vlc_bool_t      b_strict_rc;
     vlc_bool_t      b_pre_me;
     vlc_bool_t      b_hurry_up;
+    int             i_rc_buffer_size;
+    float           f_rc_buffer_aggressivity;
+    float           f_i_quant_factor;
+    int             i_noise_reduction;
+    vlc_bool_t      b_mpeg4_matrix;
+    int             i_threads;
+    vlc_bool_t      b_trellis;
 
     int             i_crop_top;
     int             i_crop_bottom;
@@ -125,8 +145,7 @@ static int Open( vlc_object_t *p_this )
     sout_stream_sys_t *p_sys;
     char *codec;
 
-    p_sys = malloc( sizeof( sout_stream_sys_t ) );
-    memset( p_sys, 0, sizeof(struct sout_stream_sys_t) );
+    p_sys = vlc_object_create( p_this, sizeof( sout_stream_sys_t ) );
     p_sys->p_out = sout_stream_new( p_stream->p_sout, p_stream->psz_next );
 
     p_sys->f_scale      = 1;
@@ -134,11 +153,16 @@ static int Open( vlc_object_t *p_this )
     p_sys->i_key_int    = -1;
     p_sys->i_qmin       = 2;
     p_sys->i_qmax       = 31;
+    p_sys->f_i_quant_factor = 0.0;
 #if LIBAVCODEC_BUILD >= 4673
     p_sys->i_hq         = FF_MB_DECISION_SIMPLE;
 #else
     p_sys->i_hq         = VLC_FALSE;
 #endif
+    p_sys->i_rc_buffer_size = 224*1024*8 * 3/2;
+    p_sys->f_rc_buffer_aggressivity = 0.1;
+    p_sys->i_threads = 0;
+    p_sys->b_trellis = 0;
 
     if( ( codec = sout_cfg_find_value( p_stream->p_cfg, "acodec" ) ) )
     {
@@ -168,7 +192,7 @@ static int Open( vlc_object_t *p_this )
 
         msg_Dbg( p_stream, "codec audio=%4.4s %dHz %d channels %dKb/s", fcc,
                  p_sys->i_sample_rate, p_sys->i_channels,
-                 p_sys->i_abitrate / 1024 );
+                 p_sys->i_abitrate / 1000 );
     }
 
     if( ( codec = sout_cfg_find_value( p_stream->p_cfg, "vcodec" ) ) )
@@ -208,6 +232,10 @@ static int Open( vlc_object_t *p_this )
         {
             p_sys->b_deinterlace = VLC_TRUE;
         }
+        if( sout_cfg_find( p_stream->p_cfg, "interlace" ) )
+        {
+            p_sys->b_interlace = VLC_TRUE;
+        }
         if( sout_cfg_find( p_stream->p_cfg, "strict_rc" ) )
         {
             p_sys->b_strict_rc = VLC_TRUE;
@@ -219,6 +247,28 @@ static int Open( vlc_object_t *p_this )
         if( sout_cfg_find( p_stream->p_cfg, "hurry_up" ) )
         {
             p_sys->b_hurry_up = VLC_TRUE;
+            /* hurry up mode needs noise reduction, even small */
+            p_sys->i_noise_reduction = 1;
+        }
+        if( ( val = sout_cfg_find_value( p_stream->p_cfg, "rc_buffer_size" ) ) )
+        {
+            p_sys->i_rc_buffer_size = atoi( val );
+        }
+        if( ( val = sout_cfg_find_value( p_stream->p_cfg, "rc_buffer_aggressivity" ) ) )
+        {
+            p_sys->f_rc_buffer_aggressivity = atof( val );
+        }
+        if( ( val = sout_cfg_find_value( p_stream->p_cfg, "i_quant_factor" ) ) )
+        {
+            p_sys->f_i_quant_factor = atof( val );
+        }
+        if( ( val = sout_cfg_find_value( p_stream->p_cfg, "noise_reduction" ) ) )
+        {
+            p_sys->i_noise_reduction = atoi( val );
+        }
+        if( ( val = sout_cfg_find_value( p_stream->p_cfg, "mpeg4_matrix" ) ) )
+        {
+            p_sys->b_mpeg4_matrix = VLC_TRUE;
         }
         /* crop */
         if( ( val = sout_cfg_find_value( p_stream->p_cfg, "croptop" ) ) )
@@ -279,10 +329,18 @@ static int Open( vlc_object_t *p_this )
         {
             p_sys->i_qmax   = atoi( val );
         }
+        if( ( val = sout_cfg_find_value( p_stream->p_cfg, "threads" ) ) )
+        {
+            p_sys->i_threads = atoi( val );
+        }
+        if( sout_cfg_find( p_stream->p_cfg, "trellis" ) )
+        {
+            p_sys->b_trellis = VLC_TRUE;
+        }
 
         msg_Dbg( p_stream, "codec video=%4.4s %dx%d scaling: %f %dkb/s",
                  fcc, p_sys->i_width, p_sys->i_height, p_sys->f_scale,
-                 p_sys->i_vbitrate / 1024 );
+                 p_sys->i_vbitrate / 1000 );
     }
 
     if( !p_sys->p_out )
@@ -315,7 +373,7 @@ static void Close( vlc_object_t * p_this )
     sout_stream_sys_t   *p_sys = p_stream->p_sys;
 
     sout_stream_delete( p_sys->p_out );
-    free( p_sys );
+    vlc_object_destroy( p_sys );
 }
 
 struct sout_stream_id_t
@@ -373,8 +431,10 @@ static sout_stream_id_t * Add( sout_stream_t *p_stream, es_format_t *p_fmt )
         memcpy( &id->f_src, p_fmt, sizeof( es_format_t ) );
 
         /* create dst format */
-        id->f_dst.i_cat    = AUDIO_ES;
-        id->f_dst.i_codec  = p_sys->i_acodec;
+        es_format_Init( &id->f_dst, AUDIO_ES, p_sys->i_acodec );
+        id->f_dst.i_id    = id->f_src.i_id;
+        id->f_dst.i_group = id->f_src.i_group;
+        if( id->f_src.psz_language ) id->f_dst.psz_language = strdup( id->f_src.psz_language );
         id->f_dst.audio.i_rate = p_sys->i_sample_rate  > 0 ? p_sys->i_sample_rate : id->f_src.audio.i_rate;
         id->f_dst.audio.i_channels    = p_sys->i_channels > 0 ? p_sys->i_channels : id->f_src.audio.i_channels;
         id->f_dst.i_bitrate     = p_sys->i_abitrate > 0 ? p_sys->i_abitrate : 64000;
@@ -410,8 +470,10 @@ static sout_stream_id_t * Add( sout_stream_t *p_stream, es_format_t *p_fmt )
         memcpy( &id->f_src, p_fmt, sizeof( es_format_t ) );
 
         /* create dst format */
-        id->f_dst.i_cat         = VIDEO_ES;
-        id->f_dst.i_codec       = p_sys->i_vcodec;
+        es_format_Init( &id->f_dst, VIDEO_ES, p_sys->i_vcodec );
+        id->f_dst.i_id    = id->f_src.i_id;
+        id->f_dst.i_group = id->f_src.i_group;
+        if( id->f_src.psz_language ) id->f_dst.psz_language = strdup( id->f_src.psz_language );
         id->f_dst.video.i_width = p_sys->i_width;
         id->f_dst.video.i_height= p_sys->i_height;
         id->f_dst.i_bitrate     = p_sys->i_vbitrate > 0 ? p_sys->i_vbitrate : 800*1000;
@@ -564,6 +626,7 @@ static struct
     { VLC_FOURCC( 'R', 'V', '3', '2' ), CODEC_ID_RAWVIDEO },
     { VLC_FOURCC( 'Y', 'U', 'Y', '2' ), CODEC_ID_RAWVIDEO },
     { VLC_FOURCC( 'Y', 'V', '1', '2' ), CODEC_ID_RAWVIDEO },
+    { VLC_FOURCC( 'I', 'Y', 'U', 'V' ), CODEC_ID_RAWVIDEO },
 
     { VLC_FOURCC(   0,   0,   0,   0 ), 0 }
 };
@@ -588,6 +651,7 @@ static inline int get_ff_chroma( vlc_fourcc_t i_chroma )
     switch( i_chroma )
     {
         case VLC_FOURCC( 'Y', 'V', '1', '2' ):
+        case VLC_FOURCC( 'I', 'Y', 'U', 'V' ):
         case VLC_FOURCC( 'I', '4', '2', '0' ):
             return PIX_FMT_YUV420P;
         case VLC_FOURCC( 'I', '4', '2', '2' ):
@@ -599,7 +663,7 @@ static inline int get_ff_chroma( vlc_fourcc_t i_chroma )
         case VLC_FOURCC( 'R', 'V', '1', '6' ):
             return PIX_FMT_RGB565;
         case VLC_FOURCC( 'R', 'V', '2', '4' ):
-            return PIX_FMT_RGB24;
+            return PIX_FMT_BGR24;
         case VLC_FOURCC( 'R', 'V', '3', '2' ):
             return PIX_FMT_RGBA32;
         case VLC_FOURCC( 'G', 'R', 'E', 'Y' ):
@@ -792,6 +856,12 @@ static int transcode_audio_ffmpeg_process( sout_stream_t *p_stream,
 
             i_buffer -= i_used;
             p_buffer += i_used;
+
+            if ( id->i_buffer_pos < 0 )
+            {
+                msg_Warn( p_stream, "weird error audio decoding");
+                break;
+            }
         }
         else
         {
@@ -827,38 +897,30 @@ static int transcode_audio_ffmpeg_process( sout_stream_t *p_stream,
                 int16_t *sin = (int16_t*)p_buffer;
                 int i_used = __MIN( id->i_buffer, i_buffer );
                 int i_samples = i_used / 2;
-                int b_invert_indianness;
 
+                /* first copy */
+                memcpy( sout, sin, i_used );
+
+#ifdef WORDS_BIGENDIAN
                 if( id->f_src.i_codec == VLC_FOURCC( 's', '1', '6', 'l' ) )
-#ifdef WORDS_BIGENDIAN
-                    b_invert_indianness = 1;
 #else
-                    b_invert_indianness = 0;
+                if( id->f_src.i_codec == VLC_FOURCC( 's', '1', '6', 'b' ) )
 #endif
-                else
-#ifdef WORDS_BIGENDIAN
-                    b_invert_indianness = 0;
-#else
-                    b_invert_indianness = 1;
-#endif
-
-                if( b_invert_indianness )
                 {
+                    uint8_t *dat = (uint8_t*)sout;
+
                     while( i_samples > 0 )
                     {
-                        uint8_t tmp[2];
+                        uint8_t tmp;
+                        tmp    = dat[0];
+                        dat[0] = dat[1];
+                        dat[1] = tmp;
+
+                        dat += 2;
 
-                        tmp[1] = *sin++;
-                        tmp[0] = *sin++;
-                        *sout++ = *(int16_t*)tmp;
                         i_samples--;
                     }
                 }
-                else
-                {
-                    memcpy( sout, sin, i_used );
-                    sout += i_samples;
-                }
 
                 i_buffer -= i_used;
                 p_buffer += i_used;
@@ -967,6 +1029,7 @@ static int transcode_video_ffmpeg_new( sout_stream_t *p_stream,
         id->f_src.i_codec == VLC_FOURCC( 'I', '4', '4', '4' ) ||
         id->f_src.i_codec == VLC_FOURCC( 'Y', 'V', '1', '2' ) ||
         id->f_src.i_codec == VLC_FOURCC( 'Y', 'U', 'Y', '2' ) ||
+        id->f_src.i_codec == VLC_FOURCC( 'I', 'Y', 'U', 'V' ) ||
         id->f_src.i_codec == VLC_FOURCC( 'R', 'V', '1', '5' ) ||
         id->f_src.i_codec == VLC_FOURCC( 'R', 'V', '1', '6' ) ||
         id->f_src.i_codec == VLC_FOURCC( 'R', 'V', '2', '4' ) ||
@@ -978,6 +1041,16 @@ static int transcode_video_ffmpeg_new( sout_stream_t *p_stream,
         id->ff_dec_c->width     = id->f_src.video.i_width;
         id->ff_dec_c->height    = id->f_src.video.i_height;
         id->ff_dec_c->pix_fmt   = get_ff_chroma( id->f_src.i_codec );
+
+#if LIBAVCODEC_BUILD >= 4687
+        if( id->ff_dec_c->width )
+        id->ff_dec_c->sample_aspect_ratio =
+            av_d2q( id->f_src.video.i_aspect / (double)VOUT_ASPECT_FACTOR *
+                    id->ff_dec_c->height / id->ff_dec_c->width, 255 );
+#else
+        id->ff_dec_c->aspect_ratio =
+            id->f_src.video.i_aspect / (float)VOUT_ASPECT_FACTOR;
+#endif
     }
     else
     {
@@ -1085,6 +1158,14 @@ static int transcode_video_ffmpeg_new( sout_stream_t *p_stream,
     id->p_encoder->b_strict_rc = p_sys->b_strict_rc;
     id->p_encoder->b_pre_me = p_sys->b_pre_me;
     id->p_encoder->b_hurry_up = p_sys->b_hurry_up;
+    id->p_encoder->b_interlace = p_sys->b_interlace;
+    id->p_encoder->i_rc_buffer_size = p_sys->i_rc_buffer_size;
+    id->p_encoder->f_rc_buffer_aggressivity = p_sys->f_rc_buffer_aggressivity;
+    id->p_encoder->f_i_quant_factor = p_sys->f_i_quant_factor;
+    id->p_encoder->i_noise_reduction = p_sys->i_noise_reduction;
+    id->p_encoder->b_mpeg4_matrix = p_sys->b_mpeg4_matrix;
+    id->p_encoder->i_threads = p_sys->i_threads;
+    id->p_encoder->b_trellis = p_sys->b_trellis;
 
     id->p_ff_pic         = avcodec_alloc_frame();
     id->p_ff_pic_tmp0    = NULL;
@@ -1105,15 +1186,46 @@ static int transcode_video_ffmpeg_new( sout_stream_t *p_stream,
     /* Close the encoder.
      * We'll open it only when we have the first frame */
     module_Unneed( id->p_encoder, id->p_encoder->p_module );
+    id->p_encoder->p_module = NULL;
 
     id->b_enc_inited = VLC_FALSE;
 
+    if ( p_sys->i_threads >= 1 )
+    {
+        p_sys->id_video = id;
+        vlc_mutex_init( p_stream, &p_sys->lock_out );
+        vlc_cond_init( p_stream, &p_sys->cond );
+        memset( p_sys->pp_pics, 0, sizeof(p_sys->pp_pics) );
+        p_sys->i_first_pic = 0;
+        p_sys->i_last_pic = 0;
+        p_sys->p_buffers = NULL;
+        p_sys->b_die = p_sys->b_error = 0;
+        if( vlc_thread_create( p_sys, "encoder", EncoderThread,
+                               VLC_THREAD_PRIORITY_VIDEO, VLC_FALSE ) )
+        {
+            vlc_object_destroy( id->p_encoder );
+            msg_Err( p_stream, "cannot spawn encoder thread" );
+            return VLC_EGENERIC;
+        }
+    }
+
     return VLC_SUCCESS;
 }
 
 static void transcode_video_ffmpeg_close ( sout_stream_t *p_stream,
                                            sout_stream_id_t *id )
 {
+    if ( p_stream->p_sys->i_threads >= 1 )
+    {
+       vlc_mutex_lock( &p_stream->p_sys->lock_out );
+       p_stream->p_sys->b_die = 1;
+       vlc_cond_signal( &p_stream->p_sys->cond );
+       vlc_mutex_unlock( &p_stream->p_sys->lock_out );
+       vlc_thread_join( p_stream->p_sys );
+       vlc_mutex_destroy( &p_stream->p_sys->lock_out );
+       vlc_cond_destroy( &p_stream->p_sys->cond );
+    }
+
     /* Close decoder */
     if( id->ff_dec )
     {
@@ -1122,7 +1234,8 @@ static void transcode_video_ffmpeg_close ( sout_stream_t *p_stream,
     }
 
     /* Close encoder */
-    module_Unneed( id->p_encoder, id->p_encoder->p_module );
+    if( id->p_encoder->p_module )
+        module_Unneed( id->p_encoder, id->p_encoder->p_module );
     vlc_object_destroy( id->p_encoder );
 
     /* Misc cleanup */
@@ -1148,7 +1261,7 @@ static void transcode_video_ffmpeg_close ( sout_stream_t *p_stream,
     }
     if( id->p_vresample )
     {
-        free( id->p_vresample );
+        img_resample_close( id->p_vresample );
     }
 }
 
@@ -1171,7 +1284,7 @@ static int transcode_video_ffmpeg_process( sout_stream_t *p_stream,
     for( ;; )
     {
         block_t *p_block;
-        picture_t pic;
+        picture_t * p_pic;
         int i_plane;
 
         /* decode frame */
@@ -1335,6 +1448,10 @@ static int transcode_video_ffmpeg_process( sout_stream_t *p_stream,
                                    id->ff_dec_c->pix_fmt,
                                    id->ff_dec_c->width, id->ff_dec_c->height );
 
+#if LIBAVCODEC_BUILD >= 4685
+            id->p_ff_pic_tmp0->interlaced_frame = 0;
+#endif
+            id->p_ff_pic_tmp0->repeat_pict = frame->repeat_pict;
             frame = id->p_ff_pic_tmp0;
         }
 
@@ -1361,6 +1478,11 @@ static int transcode_video_ffmpeg_process( sout_stream_t *p_stream,
                          (AVPicture*)frame, id->ff_dec_c->pix_fmt,
                          id->ff_dec_c->width, id->ff_dec_c->height );
 
+            id->p_ff_pic_tmp1->repeat_pict = frame->repeat_pict;
+#if LIBAVCODEC_BUILD >= 4685
+            id->p_ff_pic_tmp1->interlaced_frame = frame->interlaced_frame;
+            id->p_ff_pic_tmp1->top_field_first = frame->top_field_first;
+#endif
             frame = id->p_ff_pic_tmp1;
         }
 
@@ -1398,29 +1520,46 @@ static int transcode_video_ffmpeg_process( sout_stream_t *p_stream,
             img_resample( id->p_vresample, (AVPicture*)id->p_ff_pic_tmp2,
                           (AVPicture*)frame );
 
+            id->p_ff_pic_tmp2->repeat_pict = frame->repeat_pict;
+#if LIBAVCODEC_BUILD >= 4685
+            id->p_ff_pic_tmp2->interlaced_frame = frame->interlaced_frame;
+            id->p_ff_pic_tmp2->top_field_first = frame->top_field_first;
+#endif
             frame = id->p_ff_pic_tmp2;
         }
 
         /* Encoding */
-        vout_InitPicture( VLC_OBJECT(p_stream), &pic,
+        p_pic = malloc(sizeof(picture_t));
+        vout_InitPicture( VLC_OBJECT(p_stream), p_pic,
                           id->p_encoder->fmt_in.i_codec,
                           id->f_dst.video.i_width, id->f_dst.video.i_height,
                           id->f_dst.video.i_width * VOUT_ASPECT_FACTOR /
                           id->f_dst.video.i_height );
 
-        for( i_plane = 0; i_plane < pic.i_planes; i_plane++ )
+        for( i_plane = 0; i_plane < p_pic->i_planes; i_plane++ )
         {
-            pic.p[i_plane].p_pixels = frame->data[i_plane];
-            pic.p[i_plane].i_pitch = frame->linesize[i_plane];
+            p_pic->p[i_plane].i_pitch = frame->linesize[i_plane];
+            if ( p_sys->i_threads >= 1 )
+            {
+                p_pic->p[i_plane].p_pixels = malloc(p_pic->p[i_plane].i_lines *
+                                                    p_pic->p[i_plane].i_pitch);
+                p_stream->p_vlc->pf_memcpy( p_pic->p[i_plane].p_pixels,
+                    frame->data[i_plane], p_pic->p[i_plane].i_lines *
+                     p_pic->p[i_plane].i_pitch );
+            }
+            else
+            {
+                p_pic->p[i_plane].p_pixels = frame->data[i_plane];
+            }
         }
 
         /* Set the pts of the frame being encoded */
-        pic.date = p_sys->i_output_pts;
+        p_pic->date = p_sys->i_output_pts;
 
-        pic.b_progressive = 1; /* ffmpeg doesn't support interlaced encoding */
-        pic.i_nb_fields = frame->repeat_pict;
-#if LIBAVCODEC_BUILD >= 4684
-        pic.b_top_field_first = frame->top_field_first;
+        p_pic->i_nb_fields = frame->repeat_pict;
+#if LIBAVCODEC_BUILD >= 4685
+        p_pic->b_progressive = !frame->interlaced_frame;
+        p_pic->b_top_field_first = frame->top_field_first;
 #endif
 
         /* Interpolate the next PTS
@@ -1431,7 +1570,78 @@ static int transcode_video_ffmpeg_process( sout_stream_t *p_stream,
               id->ff_dec_c->frame_rate_base / (2 * id->ff_dec_c->frame_rate);
         }
 
-        p_block = id->p_encoder->pf_encode_video( id->p_encoder, &pic );
+        if ( p_sys->i_threads >= 1 )
+        {
+            vlc_mutex_lock( &p_sys->lock_out );
+            p_sys->pp_pics[p_sys->i_last_pic++] = p_pic;
+            p_sys->i_last_pic %= PICTURE_RING_SIZE;
+            *out = p_sys->p_buffers;
+            p_sys->p_buffers = NULL;
+            vlc_cond_signal( &p_sys->cond );
+            vlc_mutex_unlock( &p_sys->lock_out );
+        }
+        else
+        {
+            block_t *p_block;
+            p_block = id->p_encoder->pf_encode_video( id->p_encoder, p_pic );
+            while( p_block )
+            {
+                sout_buffer_t *p_out;
+                block_t *p_prev_block = p_block;
+
+                p_out = sout_BufferNew( p_stream->p_sout, p_block->i_buffer );
+                memcpy( p_out->p_buffer, p_block->p_buffer, p_block->i_buffer);
+                p_out->i_dts = p_block->i_dts;
+                p_out->i_pts = p_block->i_pts;
+                p_out->i_length = p_block->i_length;
+                sout_BufferChain( out, p_out );
+
+                p_block = p_block->p_next;
+                block_Release( p_prev_block );
+            }
+            free( p_pic );
+        }
+
+        if( i_data <= 0 )
+        {
+            return VLC_SUCCESS;
+        }
+    }
+
+    return VLC_SUCCESS;
+}
+
+static int EncoderThread( sout_stream_sys_t * p_sys )
+{
+    sout_stream_t * p_stream = p_sys->p_out;
+    sout_stream_id_t * id = p_sys->id_video;
+    picture_t * p_pic;
+    int i_plane;
+    sout_buffer_t * p_buffer;
+
+    while ( !p_sys->b_die && !p_sys->b_error )
+    {
+        block_t *p_block;
+
+        vlc_mutex_lock( &p_sys->lock_out );
+        while ( p_sys->i_last_pic == p_sys->i_first_pic )
+        {
+            vlc_cond_wait( &p_sys->cond, &p_sys->lock_out );
+            if ( p_sys->b_die || p_sys->b_error )
+                break;
+        }
+        if ( p_sys->b_die || p_sys->b_error )
+        {
+            vlc_mutex_unlock( &p_sys->lock_out );
+            break;
+        }
+
+        p_pic = p_sys->pp_pics[p_sys->i_first_pic++];
+        p_sys->i_first_pic %= PICTURE_RING_SIZE;
+        vlc_mutex_unlock( &p_sys->lock_out );
+
+        p_block = id->p_encoder->pf_encode_video( id->p_encoder, p_pic );
+        vlc_mutex_lock( &p_sys->lock_out );
         while( p_block )
         {
             sout_buffer_t *p_out;
@@ -1442,19 +1652,41 @@ static int transcode_video_ffmpeg_process( sout_stream_t *p_stream,
             p_out->i_dts = p_block->i_dts;
             p_out->i_pts = p_block->i_pts;
             p_out->i_length = p_block->i_length;
-            sout_BufferChain( out, p_out );
+            sout_BufferChain( &p_sys->p_buffers, p_out );
 
             p_block = p_block->p_next;
             block_Release( p_prev_block );
         }
+        vlc_mutex_unlock( &p_sys->lock_out );
 
-        if( i_data <= 0 )
+        for( i_plane = 0; i_plane < p_pic->i_planes; i_plane++ )
         {
-            return VLC_SUCCESS;
+            free( p_pic->p[i_plane].p_pixels );
         }
+        free( p_pic );
     }
 
-    return VLC_SUCCESS;
+    while ( p_sys->i_last_pic != p_sys->i_first_pic )
+    {
+        p_pic = p_sys->pp_pics[p_sys->i_first_pic++];
+        p_sys->i_first_pic %= PICTURE_RING_SIZE;
+
+        for( i_plane = 0; i_plane < p_pic->i_planes; i_plane++ )
+        {
+            free( p_pic->p[i_plane].p_pixels );
+        }
+        free( p_pic );
+    }
+
+    p_buffer = p_sys->p_buffers;
+    while ( p_buffer != NULL )
+    {
+        sout_buffer_t * p_next = p_buffer->p_next;
+        sout_BufferDelete( p_stream->p_sout, p_buffer );
+        p_buffer = p_next;
+    }
+
+    return 0;
 }
 
 /*****************************************************************************
