@@ -2,7 +2,7 @@
  * clone.c : Clone video plugin for vlc
  *****************************************************************************
  * Copyright (C) 2002, 2003 VideoLAN
- * $Id: clone.c,v 1.10 2003/05/15 22:27:37 massiot Exp $
+ * $Id: clone.c,v 1.12 2004/01/25 20:05:28 hartman Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -60,11 +60,12 @@ static int  SendEvents( vlc_object_t *, char const *,
 #define VOUTLIST_LONGTEXT N_("Select the specific vout modules that you want to activate")
 
 vlc_module_begin();
-    add_category_hint( N_("Clone"), NULL, VLC_FALSE );
+    set_description( _("Clone video filter") );
+    set_capability( "video filter", 0 );
+    
     add_integer( "clone-count", 2, NULL, COUNT_TEXT, COUNT_LONGTEXT, VLC_FALSE );
     add_string ( "clone-vout-list", NULL, NULL, VOUTLIST_TEXT, VOUTLIST_LONGTEXT, VLC_FALSE );
-    set_description( _("clone video filter") );
-    set_capability( "video filter", 0 );
+    
     add_shortcut( "clone" );
     set_callbacks( Create, Destroy );
 vlc_module_end();
@@ -243,6 +244,8 @@ static int Init( vout_thread_t *p_vout )
     if( psz_default_vout ) free( psz_default_vout );
     ALLOCATE_DIRECTBUFFERS( VOUT_MAX_PICTURES );
 
+    ADD_PARENT_CALLBACKS( SendEventsToChild );
+
     return VLC_SUCCESS;
 }
 
@@ -272,6 +275,8 @@ static void Destroy( vlc_object_t *p_this )
 
     RemoveAllVout( p_vout );
 
+    DEL_PARENT_CALLBACKS( SendEventsToChild );
+
     free( p_vout->p_sys->pp_vout );
     free( p_vout->p_sys );
 }
@@ -367,3 +372,22 @@ static int SendEvents( vlc_object_t *p_this, char const *psz_var,
 
     return VLC_SUCCESS;
 }
+
+/*****************************************************************************
+ * SendEventsToChild: forward events to the child/children vout
+ *****************************************************************************/
+static int SendEventsToChild( vlc_object_t *p_this, char const *psz_var,
+                       vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    vout_thread_t *p_vout = (vout_thread_t *)p_this;
+    int i_vout;
+
+    for( i_vout = 0; i_vout < p_vout->p_sys->i_clones; i_vout++ )
+    {
+        var_Set( p_vout->p_sys->pp_vout[ i_vout ], psz_var, newval );
+
+        if( !strcmp( psz_var, "fullscreen" ) ) break;
+    }
+
+    return VLC_SUCCESS;
+}
