@@ -2,7 +2,7 @@
  * wall.c : Wall video plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000, 2001, 2002, 2003 VideoLAN
- * $Id: wall.c,v 1.10 2003/05/15 22:27:37 massiot Exp $
+ * $Id: wall.c,v 1.13 2004/01/25 20:05:28 hartman Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -59,16 +59,17 @@ static int  SendEvents( vlc_object_t *, char const *,
     "which to split the video")
 
 #define ACTIVE_TEXT N_("Active windows")
-#define ACTIVE_LONGTEXT N_("comma separated list of active windows, " \
+#define ACTIVE_LONGTEXT N_("Comma separated list of active windows, " \
     "defaults to all")
 
 vlc_module_begin();
-    add_category_hint( N_("Miscellaneous"), NULL, VLC_FALSE );
+    set_description( _("wall video filter") );
+    set_capability( "video filter", 0 );
+
     add_integer( "wall-cols", 3, NULL, COLS_TEXT, COLS_LONGTEXT, VLC_FALSE );
     add_integer( "wall-rows", 3, NULL, ROWS_TEXT, ROWS_LONGTEXT, VLC_FALSE );
     add_string( "wall-active", NULL, NULL, ACTIVE_TEXT, ACTIVE_LONGTEXT, VLC_FALSE );
-    set_description( _("wall video filter") );
-    set_capability( "video filter", 0 );
+
     add_shortcut( "wall" );
     set_callbacks( Create, Destroy );
 vlc_module_end();
@@ -273,6 +274,8 @@ static int Init( vout_thread_t *p_vout )
 
     ALLOCATE_DIRECTBUFFERS( VOUT_MAX_PICTURES );
 
+    ADD_PARENT_CALLBACKS( SendEventsToChild );
+
     return VLC_SUCCESS;
 }
 
@@ -302,6 +305,8 @@ static void Destroy( vlc_object_t *p_this )
 
     RemoveAllVout( p_vout );
 
+    DEL_PARENT_CALLBACKS( SendEventsToChild );
+
     free( p_vout->p_sys->pp_vout );
     free( p_vout->p_sys );
 }
@@ -474,3 +479,24 @@ static int SendEvents( vlc_object_t *p_this, char const *psz_var,
     return VLC_SUCCESS;
 }
 
+/*****************************************************************************
+ * SendEventsToChild: forward events to the child/children vout
+ *****************************************************************************/
+static int SendEventsToChild( vlc_object_t *p_this, char const *psz_var,
+                       vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    vout_thread_t *p_vout = (vout_thread_t *)p_this;
+    int i_row, i_col, i_vout = 0;
+
+    for( i_row = 0; i_row < p_vout->p_sys->i_row; i_row++ )
+    {
+        for( i_col = 0; i_col < p_vout->p_sys->i_col; i_col++ )
+        {
+            var_Set( p_vout->p_sys->pp_vout[ i_vout ].p_vout, psz_var, newval);
+            if( !strcmp( psz_var, "fullscreen" ) ) break;
+            i_vout++;
+        }
+    }
+
+    return VLC_SUCCESS;
+}
