@@ -1,8 +1,8 @@
 /*****************************************************************************
  * events.c: Windows DirectX video output events handler
  *****************************************************************************
- * Copyright (C) 2001 VideoLAN
- * $Id: events.c,v 1.21 2003/07/29 21:46:44 gbazin Exp $
+ * Copyright (C) 2001-2004 VideoLAN
+ * $Id: events.c,v 1.38 2004/02/26 13:58:23 gbazin Exp $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -28,6 +28,7 @@
  *****************************************************************************/
 #include <errno.h>                                                 /* ENOMEM */
 #include <stdlib.h>                                                /* free() */
+#include <ctype.h>                                              /* tolower() */
 #include <string.h>                                            /* strerror() */
 
 #include <vlc/vlc.h>
@@ -41,6 +42,7 @@
 
 #include <ddraw.h>
 
+#include "vlc_keys.h"
 #include "vout.h"
 
 /*****************************************************************************
@@ -48,8 +50,23 @@
  *****************************************************************************/
 static int  DirectXCreateWindow( vout_thread_t *p_vout );
 static void DirectXCloseWindow ( vout_thread_t *p_vout );
-static long FAR PASCAL DirectXEventProc ( HWND hwnd, UINT message,
-                                          WPARAM wParam, LPARAM lParam );
+static long FAR PASCAL DirectXEventProc( HWND, UINT, WPARAM, LPARAM );
+static long FAR PASCAL DirectXVideoEventProc( HWND, UINT, WPARAM, LPARAM );
+
+static void DirectXPopupMenu( event_thread_t *p_event, vlc_bool_t b_open )
+{
+    playlist_t *p_playlist =
+        vlc_object_find( p_event, VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
+    if( p_playlist != NULL )
+    {
+        vlc_value_t val;
+        val.b_bool = b_open;
+        var_Set( p_playlist, "intf-popupmenu", val );
+        vlc_object_release( p_playlist );
+    }
+}
+
+static int DirectXConvertKey( int i_key );
 
 /*****************************************************************************
  * DirectXEventThread: Create video window & handle its messages
@@ -77,28 +94,41 @@ void DirectXEventThread( event_thread_t *p_event )
         p_event->b_dead = VLC_TRUE;
     }
 
-    /* signal the creation of the window */
+    /* Signal the creation of the window */
     vlc_thread_ready( p_event );
 
     /* Main loop */
     /* GetMessage will sleep if there's no message in the queue */
-    while( !p_event->b_die
-           && GetMessage( &msg, p_event->p_vout->p_sys->hwnd, 0, 0 ) )
+    while( !p_event->b_die && ( p_event->p_vout->p_sys->hparent ||
+           GetMessage( &msg, p_event->p_vout->p_sys->hwnd, 0, 0 ) ) )
     {
         /* Check if we are asked to exit */
         if( p_event->b_die )
             break;
 
+        if( p_event->p_vout->p_sys->hparent )
+        {
+            /* Parent window was created in another thread so we can't
+             * access the window messages. */
+            msleep( INTF_IDLE_SLEEP );
+            continue;
+        }
+
         switch( msg.message )
         {
 
-        case WM_NCMOUSEMOVE:
         case WM_MOUSEMOVE:
             vout_PlacePicture( p_event->p_vout,
                                p_event->p_vout->p_sys->i_window_width,
                                p_event->p_vout->p_sys->i_window_height,
                                &i_x, &i_y, &i_width, &i_height );
 
+            if( msg.hwnd != p_event->p_vout->p_sys->hwnd )
+            {
+                /* Child window */
+                i_x = i_y = 0;
+            }
+
             val.i_int = ( GET_X_LPARAM(msg.lParam) - i_x )
                          * p_event->p_vout->render.i_width / i_width;
             var_Set( p_event->p_vout, "mouse-x", val );
@@ -109,6 +139,7 @@ void DirectXEventThread( event_thread_t *p_event )
             val.b_bool = VLC_TRUE;
             var_Set( p_event->p_vout, "mouse-moved", val );
 
+        case WM_NCMOUSEMOVE:
             if( (abs(GET_X_LPARAM(msg.lParam) - old_mouse_pos.x) > 2 ||
                 (abs(GET_Y_LPARAM(msg.lParam) - old_mouse_pos.y)) > 2 ) )
             {
@@ -132,6 +163,7 @@ void DirectXEventThread( event_thread_t *p_event )
             var_Get( p_event->p_vout, "mouse-button-down", &val );
             val.i_int |= 1;
             var_Set( p_event->p_vout, "mouse-button-down", val );
+            DirectXPopupMenu( p_event, VLC_FALSE );
             break;
 
         case WM_LBUTTONUP:
@@ -151,6 +183,7 @@ void DirectXEventThread( event_thread_t *p_event )
             var_Get( p_event->p_vout, "mouse-button-down", &val );
             val.i_int |= 2;
             var_Set( p_event->p_vout, "mouse-button-down", val );
+            DirectXPopupMenu( p_event, VLC_FALSE );
             break;
 
         case WM_MBUTTONUP:
@@ -163,127 +196,57 @@ void DirectXEventThread( event_thread_t *p_event )
             var_Get( p_event->p_vout, "mouse-button-down", &val );
             val.i_int |= 4;
             var_Set( p_event->p_vout, "mouse-button-down", val );
+            DirectXPopupMenu( p_event, VLC_FALSE );
             break;
 
         case WM_RBUTTONUP:
             var_Get( p_event->p_vout, "mouse-button-down", &val );
             val.i_int &= ~4;
             var_Set( p_event->p_vout, "mouse-button-down", val );
-            {
-                playlist_t *p_playlist =
-                    vlc_object_find( p_event, VLC_OBJECT_PLAYLIST,
-                                     FIND_ANYWHERE );
-                if( p_playlist != NULL )
-                {
-                    vlc_value_t val;
-                    val.b_bool = VLC_TRUE; /* make compiler happy */
-                    var_Set( p_playlist, "intf-popupmenu", val );
-                    vlc_object_release( p_playlist );
-                }
-            }
+            DirectXPopupMenu( p_event, VLC_TRUE );
             break;
 
         case WM_KEYDOWN:
-            /* the key events are first processed here. The next
-             * message processed by this main message loop will be the
-             * char translation of the key event */
-            msg_Dbg( p_event, "WM_KEYDOWN" );
-            switch( msg.wParam )
+        case WM_SYSKEYDOWN:
+            /* The key events are first processed here and not translated
+             * into WM_CHAR events because we need to know the status of the
+             * modifier keys. */
+            val.i_int = DirectXConvertKey( msg.wParam );
+            if( !val.i_int )
             {
-            case VK_ESCAPE:
-                if( p_event->p_vout->b_fullscreen )
+                /* This appears to be a "normal" (ascii) key */
+                val.i_int = tolower( MapVirtualKey( msg.wParam, 2 ) );
+            }
+
+            if( val.i_int )
+            {
+                if( GetKeyState(VK_CONTROL) & 0x8000 )
                 {
-                    p_event->p_vout->i_changes |= VOUT_FULLSCREEN_CHANGE;
+                    val.i_int |= KEY_MODIFIER_CTRL;
                 }
-                else
+                if( GetKeyState(VK_SHIFT) & 0x8000 )
                 {
-                    /* close video window */
-                    PostMessage( msg.hwnd, WM_CLOSE, 0, 0 );
+                    val.i_int |= KEY_MODIFIER_SHIFT;
                 }
-                break;
-
-            case VK_MENU:
+                if( GetKeyState(VK_MENU) & 0x8000 )
                 {
-                    playlist_t *p_playlist =
-                        vlc_object_find( p_event, VLC_OBJECT_PLAYLIST,
-                                         FIND_ANYWHERE );
-                    if( p_playlist != NULL )
-                    {
-                        vlc_value_t val;
-                        var_Set( p_playlist, "intf-popupmenu", val );
-                        vlc_object_release( p_playlist );
-                    }
+                    val.i_int |= KEY_MODIFIER_ALT;
                 }
-                break;
 
-            case VK_LEFT:
-                /* input_Seek( p_event->p_vout, -5,
-                   INPUT_SEEK_SECONDS | INPUT_SEEK_CUR ); */
-                val.psz_string = "LEFT";
-                var_Set( p_event->p_vout, "key-pressed", val );
-                break;
-            case VK_RIGHT:
-                /* input_Seek( p_event->p_vout, 5,
-                   INPUT_SEEK_SECONDS | INPUT_SEEK_CUR ); */
-                val.psz_string = "RIGHT";
-                var_Set( p_event->p_vout, "key-pressed", val );
-                break;
-            case VK_UP:
-                /* input_Seek( p_event->p_vout, 60,
-                   INPUT_SEEK_SECONDS | INPUT_SEEK_CUR ); */
-                val.psz_string = "UP";
-                var_Set( p_event->p_vout, "key-pressed", val );
-                break;
-            case VK_DOWN:
-                /* input_Seek( p_event->p_vout, -60,
-                   INPUT_SEEK_SECONDS | INPUT_SEEK_CUR ); */
-                val.psz_string = "DOWN";
-                var_Set( p_event->p_vout, "key-pressed", val );
-                break;
-            case VK_RETURN:
-                val.psz_string = "ENTER";
-                var_Set( p_event->p_vout, "key-pressed", val );
-                break;
-            case VK_HOME:
-                input_Seek( p_event->p_vout, 0,
-                            INPUT_SEEK_BYTES | INPUT_SEEK_SET );
-                break;
-            case VK_END:
-                input_Seek( p_event->p_vout, 0,
-                            INPUT_SEEK_BYTES | INPUT_SEEK_END );
-                break;
-            case VK_PRIOR:
-                input_Seek( p_event->p_vout, 10,
-                            INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
-                break;
-            case VK_NEXT:
-                input_Seek( p_event->p_vout, -10,
-                            INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
-                break;
-            case VK_SPACE:
-                input_SetStatus( p_event->p_vout, INPUT_STATUS_PAUSE );
-                break;
+                var_Set( p_event->p_vlc, "key-pressed", val );
             }
-            TranslateMessage(&msg);
             break;
 
-        case WM_CHAR:
-            switch( msg.wParam )
-            {
-            case 'q':
-            case 'Q':
-                /* exit application */
-                p_event->p_vlc->b_die = VLC_TRUE;
-                break;
-
-            case 'f':                            /* switch to fullscreen */
-            case 'F':
-                p_event->p_vout->p_sys->i_changes |= VOUT_FULLSCREEN_CHANGE;
-                break;
-
-            default:
-                break;
-            }
+        case WM_VLC_CHANGE_TEXT:
+            if( p_event->p_vout->p_sys->b_using_overlay )
+                SetWindowText( p_event->p_vout->p_sys->hwnd,
+                    VOUT_TITLE " (hardware YUV overlay DirectX output)" );
+            else if( p_event->p_vout->p_sys->b_hw_yuv )
+                SetWindowText( p_event->p_vout->p_sys->hwnd,
+                    VOUT_TITLE " (hardware YUV DirectX output)" );
+            else SetWindowText( p_event->p_vout->p_sys->hwnd,
+                    VOUT_TITLE " (software RGB DirectX output)" );
+            break;
 
         default:
             /* Messages we don't handle directly are dispatched to the
@@ -323,8 +286,6 @@ void DirectXEventThread( event_thread_t *p_event )
 static int DirectXCreateWindow( vout_thread_t *p_vout )
 {
     HINSTANCE  hInstance;
-    COLORREF   colorkey;
-    HDC        hdc;
     HMENU      hMenu;
     RECT       rect_window;
 
@@ -335,43 +296,6 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
     /* Get this module's instance */
     hInstance = GetModuleHandle(NULL);
 
-    /* Create a BRUSH that will be used by Windows to paint the window
-     * background.
-     * This window background is important for us as it will be used by the
-     * graphics card to display the overlay.
-     * This is why we carefully choose the color for this background, the goal
-     * being to choose a color which isn't complete black but nearly. We
-     * obviously don't want to use black as a colorkey for the overlay because
-     * black is one of the most used color and thus would give us undesirable
-     * effects */
-    /* the first step is to find the colorkey we want to use. The difficulty
-     * comes from the potential dithering (depends on the display depth)
-     * because we need to know the real RGB value of the chosen colorkey */
-    hdc = GetDC( NULL );
-    for( colorkey = 0x0a; colorkey < 0xff /* all shades of red */; colorkey++ )
-    {
-        if( colorkey == GetNearestColor( hdc, colorkey ) )
-        {
-            break;
-        }
-    }
-    msg_Dbg( p_vout, "background color: %i", colorkey );
-
-    /* Create the actual brush */
-    p_vout->p_sys->hbrush = CreateSolidBrush(colorkey);
-    p_vout->p_sys->i_rgb_colorkey = (int)colorkey;
-
-    /* Get the current size of the display and its colour depth */
-    p_vout->p_sys->rect_display.right = GetDeviceCaps( hdc, HORZRES );
-    p_vout->p_sys->rect_display.bottom = GetDeviceCaps( hdc, VERTRES );
-    p_vout->p_sys->i_display_depth = GetDeviceCaps( hdc, BITSPIXEL );
-    msg_Dbg( p_vout, "screen dimensions %ix%i colour depth %i",
-                      p_vout->p_sys->rect_display.right,
-                      p_vout->p_sys->rect_display.bottom,
-                      p_vout->p_sys->i_display_depth );
-
-    ReleaseDC( NULL, hdc );
-
     /* If an external window was specified, we'll draw in it. */
     var_Get( p_vout->p_vlc, "drawable", &val );
     p_vout->p_sys->hparent = p_vout->p_sys->hwnd =
@@ -386,7 +310,7 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
         SetClassLong( p_vout->p_sys->hwnd,
                       GCL_STYLE, CS_DBLCLKS );
         SetClassLong( p_vout->p_sys->hwnd,
-                      GCL_HBRBACKGROUND, (LONG)p_vout->p_sys->hbrush );
+                      GCL_HBRBACKGROUND, (LONG)GetStockObject(BLACK_BRUSH) );
         SetClassLong( p_vout->p_sys->hwnd,
                       GCL_HCURSOR, (LONG)LoadCursor(NULL, IDC_ARROW) );
         /* Store a p_vout pointer into the window local storage (for later
@@ -398,8 +322,7 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
                                        GWL_WNDPROC, (LONG)DirectXEventProc );
 
         /* Blam! Erase everything that might have been there. */
-        RedrawWindow( p_vout->p_sys->hwnd, NULL, NULL,
-                      RDW_INVALIDATE | RDW_ERASE );
+        InvalidateRect( p_vout->p_sys->hwnd, NULL, TRUE );
     }
     else
     {
@@ -423,7 +346,7 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
         wc.hInstance     = hInstance;                            /* instance */
         wc.hIcon         = vlc_icon;                /* load the vlc big icon */
         wc.hCursor       = LoadCursor(NULL, IDC_ARROW);    /* default cursor */
-        wc.hbrBackground = p_vout->p_sys->hbrush;        /* background color */
+        wc.hbrBackground = GetStockObject(BLACK_BRUSH);  /* background color */
         wc.lpszMenuName  = NULL;                                  /* no menu */
         wc.lpszClassName = "VLC DirectX";             /* use a special class */
         wc.hIconSm       = vlc_icon;              /* load the vlc small icon */
@@ -433,13 +356,6 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
         {
             WNDCLASS wndclass;
 
-            /* Free window background brush */
-            if( p_vout->p_sys->hbrush )
-            {
-                DeleteObject( p_vout->p_sys->hbrush );
-                p_vout->p_sys->hbrush = NULL;
-            }
-
             if( vlc_icon )
             {
                 DestroyIcon( vlc_icon );
@@ -468,7 +384,8 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
         p_vout->p_sys->hwnd =
             CreateWindow( "VLC DirectX",             /* name of window class */
                     VOUT_TITLE " (DirectX Output)", /* window title bar text */
-                    WS_OVERLAPPEDWINDOW | WS_SIZEBOX,        /* window style */
+                    WS_OVERLAPPEDWINDOW | WS_SIZEBOX | WS_VISIBLE |
+                    WS_CLIPCHILDREN,                         /* window style */
                     CW_USEDEFAULT,                   /* default X coordinate */
                     0,                               /* default Y coordinate */
                     rect_window.right - rect_window.left,    /* window width */
@@ -485,15 +402,20 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
         }
     }
 
+    /* Now display the window */
+    ShowWindow( p_vout->p_sys->hwnd, SW_SHOW );
+
+    /* Create video sub-window. This sub window will always exactly match
+     * the size of the video, which allows us to use crazy overlay colorkeys
+     * without having them shown outside of the video area. */
+    SendMessage( p_vout->p_sys->hwnd, WM_VLC_CREATE_VIDEO_WIN, 0, 0 );
+
     /* Append a "Always On Top" entry in the system menu */
     hMenu = GetSystemMenu( p_vout->p_sys->hwnd, FALSE );
     AppendMenu( hMenu, MF_SEPARATOR, 0, "" );
     AppendMenu( hMenu, MF_STRING | MF_UNCHECKED,
                        IDM_TOGGLE_ON_TOP, "Always on &Top" );
 
-    /* Now display the window */
-    ShowWindow( p_vout->p_sys->hwnd, SW_SHOW );
-
     return VLC_SUCCESS;
 }
 
@@ -510,6 +432,19 @@ static void DirectXCloseWindow( vout_thread_t *p_vout )
     {
         DestroyWindow( p_vout->p_sys->hwnd );
     }
+    else if( p_vout->p_sys->hparent )
+    {
+        /* Get rid of the video sub-window */
+        PostMessage( p_vout->p_sys->hvideownd, WM_VLC_DESTROY_VIDEO_WIN, 0, 0);
+
+        /* We don't want our windowproc to be called anymore */
+        SetWindowLong( p_vout->p_sys->hwnd,
+                       GWL_WNDPROC, (LONG)p_vout->p_sys->pf_wndproc );
+        SetWindowLong( p_vout->p_sys->hwnd, GWL_USERDATA, (LONG)NULL );
+
+        /* Blam! Erase everything that might have been there. */
+        InvalidateRect( p_vout->p_sys->hwnd, NULL, TRUE );
+    }
 
     p_vout->p_sys->hwnd = NULL;
 
@@ -563,20 +498,26 @@ void DirectXUpdateRects( vout_thread_t *p_vout, vlc_bool_t b_force )
     vout_PlacePicture( p_vout, rect.right, rect.bottom,
                        &i_x, &i_y, &i_width, &i_height );
 
+    if( p_vout->p_sys->hvideownd )
+        SetWindowPos( p_vout->p_sys->hvideownd, HWND_TOP,
+                      i_x, i_y, i_width, i_height, 0 );
+
     /* Destination image position and dimensions */
     rect_dest.left = point.x + i_x;
     rect_dest.right = rect_dest.left + i_width;
     rect_dest.top = point.y + i_y;
     rect_dest.bottom = rect_dest.top + i_height;
 
-
     /* UpdateOverlay directdraw function doesn't automatically clip to the
      * display size so we need to do it otherwise it will fail */
 
     /* Clip the destination window */
-    IntersectRect( &rect_dest_clipped,
-                   &rect_dest,
-                   &p_vout->p_sys->rect_display );
+    if( !IntersectRect( &rect_dest_clipped, &rect_dest,
+                        &p_vout->p_sys->rect_display ) )
+    {
+        SetRectEmpty( &rect_src_clipped );
+        return;
+    }
 
 #if 0
     msg_Dbg( p_vout, "DirectXUpdateRects image_dst_clipped coords:"
@@ -611,6 +552,13 @@ void DirectXUpdateRects( vout_thread_t *p_vout, vlc_bool_t b_force )
       (rect_dest.bottom - rect_dest_clipped.bottom) * p_vout->render.i_height /
       (rect_dest.bottom - rect_dest.top);
 
+    /* The destination coordinates need to be relative to the current
+     * directdraw primary surface (display) */
+    rect_dest_clipped.left -= p_vout->p_sys->rect_display.left;
+    rect_dest_clipped.right -= p_vout->p_sys->rect_display.left;
+    rect_dest_clipped.top -= p_vout->p_sys->rect_display.top;
+    rect_dest_clipped.bottom -= p_vout->p_sys->rect_display.top;
+
 #if 0
     msg_Dbg( p_vout, "DirectXUpdateRects image_src_clipped"
                      " coords: %i,%i,%i,%i",
@@ -618,18 +566,8 @@ void DirectXUpdateRects( vout_thread_t *p_vout, vlc_bool_t b_force )
                      rect_src_clipped.right, rect_src_clipped.bottom );
 #endif
 
-    /* Signal the size change */
-    if( !p_vout->p_sys->p_event->b_die )
-    {
-        if( p_vout->p_sys->b_using_overlay )
-        {
-            DirectXUpdateOverlay( p_vout );
-        }
-        else
-        {
-            p_vout->p_sys->i_changes |= VOUT_SIZE_CHANGE;
-        }
-    }
+    /* Signal the change in size/position */
+    p_vout->p_sys->i_changes |= DX_POSITION_CHANGE;
 
 #undef rect_src
 #undef rect_src_clipped
@@ -664,6 +602,13 @@ static long FAR PASCAL DirectXEventProc( HWND hwnd, UINT message,
         p_vout = (vout_thread_t *)GetWindowLong( hwnd, GWL_USERDATA );
     }
 
+    if( !p_vout )
+    {
+        /* Hmmm mozilla does manage somehow to save the pointer to our
+         * windowproc and still calls it after the vout has been closed. */
+        return DefWindowProc(hwnd, message, wParam, lParam);
+    }
+
     switch( message )
     {
 
@@ -707,35 +652,36 @@ static long FAR PASCAL DirectXEventProc( HWND hwnd, UINT message,
                 msg_Dbg( p_vout, "WinProc WM_SYSCOMMAND: IDM_TOGGLE_ON_TOP");
 
                 /* Get the current value... */
-                if( var_Get( p_vout, "directx-on-top", &val ) < 0 )
+                if( var_Get( p_vout, "video-on-top", &val ) < 0 )
                     return 0;
                 /* ...and change it */
                 val.b_bool = !val.b_bool;
-                var_Set( p_vout, "directx-on-top", val );
+                var_Set( p_vout, "video-on-top", val );
                 return 0;
                 break;
             }
         }
         break;
 
-    case WM_ERASEBKGND:
-        if( !p_vout->p_sys->b_using_overlay )
+    case WM_VLC_CREATE_VIDEO_WIN:
+        /* Create video sub-window */
+        p_vout->p_sys->hvideownd =
+            CreateWindow( "STATIC", "",   /* window class and title bar text */
+                    WS_CHILD | WS_VISIBLE,                   /* window style */
+                    CW_USEDEFAULT, CW_USEDEFAULT,     /* default coordinates */
+                    CW_USEDEFAULT, CW_USEDEFAULT,
+                    hwnd,                                   /* parent window */
+                    NULL, GetModuleHandle(NULL), NULL );
+
+        if( !p_vout->p_sys->hvideownd )
         {
-            /* We want to eliminate unnecessary background redraws which create
-             * an annoying flickering */
-            int i_width, i_height, i_x, i_y;
-            RECT rect_temp;
-            GetClipBox( (HDC)wParam, &rect_temp );
-#if 0
-            msg_Dbg( p_vout, "WinProc WM_ERASEBKGND %i,%i,%i,%i",
-                          rect_temp.left, rect_temp.top,
-                          rect_temp.right, rect_temp.bottom );
-#endif
-            vout_PlacePicture( p_vout, p_vout->p_sys->i_window_width,
-                               p_vout->p_sys->i_window_height,
-                               &i_x, &i_y, &i_width, &i_height );
-            ExcludeClipRect( (HDC)wParam, i_x, i_y,
-                             i_x + i_width, i_y + i_height );
+            msg_Warn( p_vout, "Can't create video sub-window" );
+        }
+        else
+        {
+            msg_Dbg( p_vout, "Created video sub-window" );
+            SetWindowLong( p_vout->p_sys->hvideownd,
+                           GWL_WNDPROC, (LONG)DirectXVideoEventProc );
         }
         break;
 
@@ -746,3 +692,64 @@ static long FAR PASCAL DirectXEventProc( HWND hwnd, UINT message,
 
     return DefWindowProc(hwnd, message, wParam, lParam);
 }
+static long FAR PASCAL DirectXVideoEventProc( HWND hwnd, UINT message,
+                                              WPARAM wParam, LPARAM lParam )
+{
+    switch( message )
+    {
+    case WM_VLC_DESTROY_VIDEO_WIN:
+        /* Destroy video sub-window */
+        DestroyWindow( hwnd );
+        break;
+    }
+
+    return DefWindowProc(hwnd, message, wParam, lParam);
+}
+
+static struct
+{
+    int i_dxkey;
+    int i_vlckey;
+
+} dxkeys_to_vlckeys[] =
+{
+    { VK_F1, KEY_F1 }, { VK_F2, KEY_F2 }, { VK_F3, KEY_F3 }, { VK_F4, KEY_F4 },
+    { VK_F5, KEY_F5 }, { VK_F6, KEY_F6 }, { VK_F7, KEY_F7 }, { VK_F8, KEY_F8 },
+    { VK_F9, KEY_F9 }, { VK_F10, KEY_F10 }, { VK_F11, KEY_F11 },
+    { VK_F12, KEY_F12 },
+
+    { VK_RETURN, KEY_ENTER },
+    { VK_SPACE, KEY_SPACE },
+    { VK_ESCAPE, KEY_ESC },
+
+    { VK_LEFT, KEY_LEFT },
+    { VK_RIGHT, KEY_RIGHT },
+    { VK_UP, KEY_UP },
+    { VK_DOWN, KEY_DOWN },
+
+    { VK_HOME, KEY_HOME },
+    { VK_END, KEY_END },
+    { VK_PRIOR, KEY_PAGEUP },
+    { VK_NEXT, KEY_PAGEDOWN },
+
+    { VK_CONTROL, 0 },
+    { VK_SHIFT, 0 },
+    { VK_MENU, 0 },
+
+    { 0, 0 }
+};
+
+static int DirectXConvertKey( int i_key )
+{
+    int i;
+
+    for( i = 0; dxkeys_to_vlckeys[i].i_dxkey != 0; i++ )
+    {
+        if( dxkeys_to_vlckeys[i].i_dxkey == i_key )
+        {
+            return dxkeys_to_vlckeys[i].i_vlckey;
+        }
+    }
+
+    return 0;
+}
