@@ -2,7 +2,7 @@
  * qte.cpp : QT Embedded plugin for vlc
  *****************************************************************************
  * Copyright (C) 1998-2003 VideoLAN
- * $Id: qte.cpp,v 1.17 2003/03/30 18:14:39 gbazin Exp $
+ * $Id: qte.cpp,v 1.21 2004/01/26 16:45:02 zorglub Exp $
  *
  * Authors: Gerald Hansink <gerald.hansink@ordain.nl>
  *          Jean-Paul Saman <jpsaman@wxs.nl>
@@ -71,7 +71,6 @@ extern "C"
 
 extern "C"
 {
-#include "netutils.h"                                 /* network_ChannelJoin */
 #include "qte.h"
 
 /*****************************************************************************
@@ -79,8 +78,8 @@ extern "C"
  *****************************************************************************/
 #define DISPLAY_TEXT N_("QT Embedded display name")
 #define DISPLAY_LONGTEXT N_( \
-    "Specify the Qt Embedded hardware display you want to use. By default VLC will " \
-    "use the value of the DISPLAY environment variable.")
+    "Specify the Qt Embedded hardware display you want to use. " \
+    "By default VLC will use the value of the DISPLAY environment variable.")
 
 /*****************************************************************************
  * Local prototypes
@@ -177,7 +176,7 @@ static void Close ( vlc_object_t *p_this )
 {
     vout_thread_t * p_vout = (vout_thread_t *)p_this;
 
-    msg_Dbg( p_vout, "Close" );
+    msg_Dbg( p_vout, "close" );
     if( p_vout->p_sys->p_event )
     {
         vlc_object_detach( p_vout->p_sys->p_event );
@@ -191,7 +190,7 @@ static void Close ( vlc_object_t *p_this )
     }
 
 #ifdef NEED_QTE_MAIN
-    msg_Dbg( p_vout, "Releasing gui-helper" );
+    msg_Dbg( p_vout, "releasing gui-helper" );
     module_Unneed( p_vout, p_vout->p_sys->p_qte_main );
 #endif
 
@@ -291,14 +290,14 @@ static void Display( vout_thread_t *p_vout, picture_t *p_pic )
                        &x, &y, &w, &h );
 #if 0
     msg_Dbg(p_vout, "+qte::Display( p_vout, i_width=%d, i_height=%d, x=%u, y=%u, w=%u, h=%u",
-	p_vout->output.i_width, p_vout->output.i_height, x, y, w, h );
+        p_vout->output.i_width, p_vout->output.i_height, x, y, w, h );
 #endif
 
     if(p_vout->p_sys->p_VideoWidget)
     {
 // shameless borrowed from opie mediaplayer....
 #ifndef USE_DIRECT_PAINTER
-        msg_Dbg(p_vout, "Not using direct painter");
+        msg_Dbg(p_vout, "not using direct painter");
         QPainter p(p_vout->p_sys->p_VideoWidget);
 
         /* rotate frame */
@@ -353,7 +352,7 @@ static void Display( vout_thread_t *p_vout, picture_t *p_pic )
  *****************************************************************************/
 static int Manage( vout_thread_t *p_vout )
 {
-    msg_Dbg( p_vout, "Manage" );
+//    msg_Dbg( p_vout, "Manage" );
 
     /* Fullscreen change */
     if( p_vout->i_changes & VOUT_FULLSCREEN_CHANGE )
@@ -392,6 +391,9 @@ static int Manage( vout_thread_t *p_vout )
 //        p_vout->p_sys->b_cursor_autohidden = 1;
 //        SDL_ShowCursor( 0 );
 //    }
+//
+//    if( p_vout->p_vlc->b_die )
+//        p_vout->p_sys->bRunning = FALSE;
 
     return 0;
 }
@@ -486,7 +488,7 @@ static void ToggleFullScreen ( vout_thread_t *p_vout )
     if ( p_vout->b_fullscreen )
        p_vout->p_sys->p_VideoWidget->showFullScreen();
     else
-      p_vout->p_sys->p_VideoWidget->showNormal();
+       p_vout->p_sys->p_VideoWidget->showNormal();
 
     p_vout->b_fullscreen = !p_vout->b_fullscreen;
 }
@@ -562,7 +564,7 @@ static int OpenDisplay( vout_thread_t *p_vout )
 static void CloseDisplay( vout_thread_t *p_vout )
 {
     // quit qt application loop
-    msg_Dbg( p_vout, "Destroying Qt Window" );
+    msg_Dbg( p_vout, "destroying Qt Window" );
 #ifdef NEED_QTE_MAIN
     if(p_vout->p_sys->p_QApplication)
     {
@@ -605,7 +607,7 @@ static void RunQtThread(event_thread_t *p_event)
         }
         QWidget* pWidget = new QWidget();
         if (pWidget)
-	{
+            {
             p_event->p_vout->p_sys->p_VideoWidget = pWidget;
         }
     }
@@ -617,31 +619,31 @@ static void RunQtThread(event_thread_t *p_event)
     if (p_event->p_vout->p_sys->p_QApplication)
     {
         /* Set default window width and heigh to exactly preferred size. */
-    	QWidget *desktop = p_event->p_vout->p_sys->p_QApplication->desktop();
-	p_event->p_vout->p_sys->p_VideoWidget->setMinimumWidth( 10 );
- 	p_event->p_vout->p_sys->p_VideoWidget->setMinimumHeight( 10 );
-	p_event->p_vout->p_sys->p_VideoWidget->setBaseSize( p_event->p_vout->p_sys->i_width,
-	p_event->p_vout->p_sys->i_height );
+            QWidget *desktop = p_event->p_vout->p_sys->p_QApplication->desktop();
+            p_event->p_vout->p_sys->p_VideoWidget->setMinimumWidth( 10 );
+             p_event->p_vout->p_sys->p_VideoWidget->setMinimumHeight( 10 );
+            p_event->p_vout->p_sys->p_VideoWidget->setBaseSize( p_event->p_vout->p_sys->i_width,
+            p_event->p_vout->p_sys->i_height );
         p_event->p_vout->p_sys->p_VideoWidget->setMaximumWidth( desktop->width() );
         p_event->p_vout->p_sys->p_VideoWidget->setMaximumHeight( desktop->height() );
         /* Check on fullscreen */
         if (p_event->p_vout->b_fullscreen)
-	    p_event->p_vout->p_sys->p_VideoWidget->showFullScreen();
+                  p_event->p_vout->p_sys->p_VideoWidget->showFullScreen();
         else
-	   p_event->p_vout->p_sys->p_VideoWidget->showNormal();
+                p_event->p_vout->p_sys->p_VideoWidget->showNormal();
 
         p_event->p_vout->p_sys->p_VideoWidget->show();
         p_event->p_vout->p_sys->bRunning = TRUE;
 
 #ifdef NEED_QTE_MAIN
         while(!p_event->b_die && p_event->p_vout->p_sys->bRunning)
-	{
-    	   /* Check if we are asked to exit */
+              {
+               /* Check if we are asked to exit */
            if( p_event->b_die )
                break;
 
-	   msleep(100);
-	}
+               msleep(100);
+            }
 #else
         // run the main loop of qtapplication until someone says: 'quit'
         p_event->p_vout->p_sys->pcQApplication->exec();
