@@ -2,18 +2,18 @@
  * xvideo.c : Xvideo plugin for vlc
  *****************************************************************************
  * Copyright (C) 1998-2001 VideoLAN
- * $Id: xvideo.c,v 1.5 2003/02/20 01:52:46 sigmunau Exp $
+ * $Id: xvideo.c,v 1.9 2004/01/26 16:45:03 zorglub Exp $
  *
  * Authors: Shane Harper <shanegh@optusnet.com.au>
  *          Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
  *          David Kennedy <dkennedy@tinytoad.com>
- *      
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -46,11 +46,11 @@ extern void E_(Deactivate) ( vlc_object_t * );
     "If you graphics card provides several adaptors, this option allows you " \
     "to choose which one will be used (you shouldn't have to change this).")
 
-#define ALT_FS_TEXT N_("alternate fullscreen method")
+#define ALT_FS_TEXT N_("Alternate fullscreen method")
 #define ALT_FS_LONGTEXT N_( \
     "There are two ways to make a fullscreen window, unfortunately each one " \
     "has its drawbacks.\n" \
-    "1) Let the window manager handle your fullscreen window (default). But " \
+    "1) Let the window manager handle your fullscreen window (default), but " \
     "things like taskbars will likely show on top of the video.\n" \
     "2) Completly bypass the window manager, but then nothing will be able " \
     "to show on top of the video.")
@@ -65,12 +65,16 @@ extern void E_(Deactivate) ( vlc_object_t * );
     "Force the XVideo renderer to use a specific chroma format instead of " \
     "trying to improve performances by using the most efficient one.")
 
-#define SHM_TEXT N_("use shared memory")
+#define SHM_TEXT N_("Use shared memory")
 #define SHM_LONGTEXT N_( \
     "Use shared memory to communicate between VLC and the X server.")
 
+#define SCREEN_TEXT N_("Screen to be used for fullscreen mode.")
+#define SCREEN_LONGTEXT N_( \
+    "Choose the screen you want to use in fullscreen mode. For instance " \
+    "set it to 0 for first screen, 1 for the second.")
+
 vlc_module_begin();
-    add_category_hint( N_("XVideo"), NULL, VLC_TRUE );
     add_string( "xvideo-display", NULL, NULL, DISPLAY_TEXT, DISPLAY_LONGTEXT, VLC_TRUE );
     add_integer( "xvideo-adaptor", -1, NULL, ADAPTOR_TEXT, ADAPTOR_LONGTEXT, VLC_TRUE );
     add_bool( "xvideo-altfullscreen", 0, NULL, ALT_FS_TEXT, ALT_FS_LONGTEXT, VLC_TRUE );
@@ -78,7 +82,11 @@ vlc_module_begin();
 #ifdef HAVE_SYS_SHM_H
     add_bool( "xvideo-shm", 1, NULL, SHM_TEXT, SHM_LONGTEXT, VLC_TRUE );
 #endif
-    set_description( _("XVideo extension module") );
+#ifdef HAVE_XINERAMA
+    add_integer ( "xvideo-xineramascreen", 0, NULL, SCREEN_TEXT, SCREEN_LONGTEXT, VLC_TRUE );
+#endif
+
+    set_description( _("XVideo extension video output") );
     set_capability( "video output", 150 );
     set_callbacks( E_(Activate), E_(Deactivate) );
 vlc_module_end();
