@@ -2,7 +2,7 @@
  * xosd.c : X On Screen Display interface
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: xosd.c,v 1.11 2003/06/29 22:07:40 gbazin Exp $
+ * $Id: xosd.c,v 1.12 2004/01/25 20:40:59 gbazin Exp $
  *
  * Authors: Loc Minier <lool@videolan.org>
  *
@@ -72,14 +72,13 @@ static void Run          ( intf_thread_t * );
 #define FONT_LONGTEXT N_("Font used to display text in the xosd output")
 
 vlc_module_begin();
-    add_category_hint( N_("XOSD module"), NULL, VLC_TRUE );
+    set_description( _("XOSD interface") );
     add_bool( "xosd-position", 1, NULL, POSITION_TEXT, POSITION_LONGTEXT, VLC_TRUE );
     add_integer( "xosd-text-offset", 0, NULL, TXT_OFS_TEXT, TXT_OFS_LONGTEXT, VLC_TRUE );
     add_integer( "xosd-shadow-offset", 1, NULL,
                  SHD_OFS_TEXT, SHD_OFS_LONGTEXT, VLC_TRUE );
     add_string( "xosd-font", "-misc-fixed-medium-r-*-*-*-300-*-*-*-*-*-*",
                 NULL, FONT_TEXT, FONT_LONGTEXT, VLC_TRUE );
-    set_description( _("xosd interface") );
     set_capability( "interface", 10 );
     set_callbacks( Open, Close );
 vlc_module_end();
