@@ -2,7 +2,7 @@
  * vlcplugin.h: a VLC plugin for Mozilla
  *****************************************************************************
  * Copyright (C) 2002 VideoLAN
- * $Id: vlcplugin.h,v 1.11 2003/07/17 09:25:58 sigmunau Exp $
+ * $Id: vlcplugin.h,v 1.13 2003/10/23 17:04:40 sam Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -24,6 +24,7 @@
 /*******************************************************************************
  * Instance state information about the plugin.
  ******************************************************************************/
+
 class VlcPlugin
 {
 public:
@@ -53,6 +54,12 @@ public:
     Display *p_display;
 #endif
 
+#ifdef XP_MACOSX
+    /* MACOS data members */
+    NPWindow *window;
+#endif
+
+
     /* vlc data members */
     int      i_vlc;
     int      b_stream;
