@@ -1,8 +1,8 @@
 /*****************************************************************************
  * dec.c : audio output API towards decoders
  *****************************************************************************
- * Copyright (C) 2002 VideoLAN
- * $Id: dec.c,v 1.10 2003/05/22 16:01:02 gbazin Exp $
+ * Copyright (C) 2002-2004 VideoLAN
+ * $Id: dec.c,v 1.14 2004/01/06 12:02:05 zorglub Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -49,6 +49,7 @@ static aout_input_t * DecNew( vlc_object_t * p_this, aout_instance_t * p_aout,
 {
     aout_input_t * p_input;
     input_thread_t * p_input_thread;
+    vlc_value_t val;
 
     /* We can only be called by the decoder, so no need to lock
      * p_input->lock. */
@@ -82,14 +83,8 @@ static aout_input_t * DecNew( vlc_object_t * p_this, aout_instance_t * p_aout,
     {
         int i;
 
-        if ( var_Type( p_aout, "audio-device" ) != 0 )
-        {
-            var_Destroy( p_aout, "audio-device" );
-        }
-        if ( var_Type( p_aout, "audio-channels" ) != 0 )
-        {
-            var_Destroy( p_aout, "audio-channels" );
-        }
+        var_Destroy( p_aout, "audio-device" );
+        var_Destroy( p_aout, "audio-channels" );
 
         /* Recreate the output using the new format. */
         if ( aout_OutputNew( p_aout, p_format ) < 0 )
@@ -125,24 +120,26 @@ static aout_input_t * DecNew( vlc_object_t * p_this, aout_instance_t * p_aout,
         return NULL;
     }
 
-    aout_MixerNew( p_aout );
-
     aout_InputNew( p_aout, p_input );
 
     vlc_mutex_unlock( &p_aout->mixer_lock );
 
+    var_Create( p_this, "audio-desync", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    var_Get( p_this, "audio-desync", &val );
+    p_input->i_desync = val.i_int * 1000;
+
     p_input_thread = (input_thread_t *)vlc_object_find( p_this,
                                            VLC_OBJECT_INPUT, FIND_PARENT );
     if( p_input_thread )
     {
-        p_aout->i_pts_delay = p_input_thread->i_pts_delay;
-        p_aout->i_pts_delay += p_aout->p_vlc->i_desync;
+        p_input->i_pts_delay = p_input_thread->i_pts_delay;
+        p_input->i_pts_delay += p_input->i_desync;
         vlc_object_release( p_input_thread );
     }
     else
     {
-        p_aout->i_pts_delay = DEFAULT_PTS_DELAY;
-        p_aout->i_pts_delay += p_aout->p_vlc->i_desync;
+        p_input->i_pts_delay = DEFAULT_PTS_DELAY;
+        p_input->i_pts_delay += p_input->i_desync;
     }
 
     return p_input;
@@ -167,6 +164,7 @@ aout_input_t * __aout_DecNew( vlc_object_t * p_this,
             {
                 return NULL;
             }
+            vlc_object_attach( *pp_aout, p_this->p_vlc );
         }
         else
         {
@@ -302,10 +300,10 @@ int aout_DecPlay( aout_instance_t * p_aout, aout_input_t * p_input,
     }
 
     /* Apply the desynchronisation requested by the user */
-    p_buffer->start_date += p_aout->p_vlc->i_desync;
-    p_buffer->end_date += p_aout->p_vlc->i_desync;
+    p_buffer->start_date += p_input->i_desync;
+    p_buffer->end_date += p_input->i_desync;
 
-    if ( p_buffer->start_date > mdate() + p_aout->i_pts_delay +
+    if ( p_buffer->start_date > mdate() + p_input->i_pts_delay +
          AOUT_MAX_ADVANCE_TIME )
     {
         msg_Warn( p_aout, "received buffer in the future ("I64Fd")",
