@@ -3,8 +3,8 @@
  * Read a stream, demultiplex and parse it before sending it to
  * decoders.
  *****************************************************************************
- * Copyright (C) 1998-2002 VideoLAN
- * $Id: input.c,v 1.271 2003/12/03 22:14:38 sigmunau Exp $
+ * Copyright (C) 1998-2004 VideoLAN
+ * $Id: input.c,v 1.290 2004/02/16 17:14:15 zorglub Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -43,6 +43,7 @@
 
 #include "vlc_interface.h"
 #include "codecs.h"
+#include "vlc_meta.h"
 #include "../../modules/demux/util/sub.h"
 
 /*****************************************************************************
@@ -81,11 +82,11 @@ static int RateCallback    ( vlc_object_t *p_this, char const *psz_cmd,
  * This function creates a new input, and returns a pointer
  * to its description. On error, it returns NULL.
  *****************************************************************************/
-input_thread_t *__input_CreateThread( vlc_object_t *p_parent,
-                                      playlist_item_t *p_item )
+input_thread_t *__input_CreateThread( vlc_object_t *p_parent, char *psz_uri,
+                                      char **ppsz_options, int i_options )
+
 {
     input_thread_t *    p_input;                        /* thread descriptor */
-    input_info_category_t * p_info;
     vlc_value_t val;
     int i;
 
@@ -98,9 +99,10 @@ input_thread_t *__input_CreateThread( vlc_object_t *p_parent,
     }
 
     /* Parse input options */
-    for( i = 0; i < p_item->i_options; i++ )
+    for( i = 0; i < i_options; i++ )
     {
-        ParseOption( p_input, p_item->ppsz_options[i] );
+        msg_Dbg( p_input, "option: %s", ppsz_options[i] );
+        ParseOption( p_input, ppsz_options[i] );
     }
 
     /* Create a few object variables we'll need later on */
@@ -118,6 +120,9 @@ input_thread_t *__input_CreateThread( vlc_object_t *p_parent,
     var_Create( p_input, "sout-video", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
     var_Create( p_input, "sout-keep",  VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
 
+    /* repeat variable */
+    var_Create( p_input, "input-repeat", VLC_VAR_INTEGER|VLC_VAR_DOINHERIT );
+
     /* decoders */
     var_Create( p_input, "minimize-threads", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
 
@@ -170,7 +175,7 @@ input_thread_t *__input_CreateThread( vlc_object_t *p_parent,
     p_input->p_sys      = NULL;
 
     /* Set target */
-    p_input->psz_source = strdup( p_item->psz_uri );
+    p_input->psz_source = strdup( psz_uri );
 
     /* Stream */
     p_input->s = NULL;
@@ -233,23 +238,15 @@ input_thread_t *__input_CreateThread( vlc_object_t *p_parent,
     p_input->stream.control.i_status = INIT_S;
     p_input->stream.control.i_rate = DEFAULT_RATE;
     p_input->stream.control.b_mute = 0;
-    p_input->stream.control.b_grayscale = config_GetInt( p_input, "grayscale" );
-
-    /* Initialize input info */
-    p_input->stream.p_info = malloc( sizeof( input_info_category_t ) );
-    if( !p_input->stream.p_info )
-    {
-        msg_Err( p_input, "No memory!" );
-        return NULL;
-    }
-    p_input->stream.p_info->psz_name = strdup("General") ;
-    p_input->stream.p_info->p_info = NULL;
-    p_input->stream.p_info->p_next = NULL;
+    p_input->stream.control.b_grayscale = config_GetInt( p_input, "grayscale");
 
     msg_Info( p_input, "playlist item `%s'", p_input->psz_source );
 
-    p_info = input_InfoCategory( p_input, _("General") );
-    input_AddInfo( p_info, _("Playlist Item"), p_input->psz_source );
+    /* Initialize input info */
+    p_input->stream.p_info = NULL;
+    p_input->stream.p_info = input_InfoCategory( p_input, _("General") );
+    input_AddInfo( p_input->stream.p_info, _("Playlist Item"),
+                   p_input->psz_source );
     vlc_object_attach( p_input, p_parent );
 
     /* Create thread and wait for its readiness. */
@@ -257,6 +254,7 @@ input_thread_t *__input_CreateThread( vlc_object_t *p_parent,
                            VLC_THREAD_PRIORITY_INPUT, VLC_TRUE ) )
     {
         msg_Err( p_input, "cannot create input thread" );
+        input_DelInfo( p_input );
         free( p_input );
         return NULL;
     }
@@ -319,6 +317,8 @@ static int RunThread( input_thread_t *p_input )
         /* Tell we're dead */
         p_input->b_dead = 1;
 
+        input_DelInfo( p_input );
+
         return 0;
     }
 
@@ -469,10 +469,28 @@ static int RunThread( input_thread_t *p_input )
 
         if( i_count == 0 )
         {
-            /* End of file - we do not set b_die because only the
-             * playlist is allowed to do so. */
-            msg_Info( p_input, "EOF reached" );
-            p_input->b_eof = 1;
+            vlc_value_t repeat;
+
+            var_Get( p_input, "input-repeat", &repeat );
+            if( repeat.i_int == 0 || p_input->stream.i_area_nb <= 0 )
+            {
+                /* End of file - we do not set b_die because only the
+                 * playlist is allowed to do so. */
+                msg_Info( p_input, "EOF reached" );
+                p_input->b_eof = 1;
+            }
+            else
+            {
+                msg_Dbg( p_input, "repeating the same input (%d)", repeat.i_int );
+                if( repeat.i_int > 0 )
+                {
+                    repeat.i_int--;
+                    var_Set( p_input, "input-repeat", repeat );
+                }
+
+                p_input->stream.p_new_area = p_input->stream.pp_areas[0];
+                p_input->stream.p_new_area->i_seek = 0;
+            }
         }
         else if( i_count < 0 )
         {
@@ -528,16 +546,19 @@ static int RunThread( input_thread_t *p_input )
  *****************************************************************************/
 static int InitThread( input_thread_t * p_input )
 {
+    vlc_meta_t *meta;
     float f_fps;
     playlist_t *p_playlist;
     mtime_t i_length;
-    
+
     /* Parse source string. Syntax : [[<access>][/<demux>]:][<source>] */
     char * psz_parser = p_input->psz_dupsource = strdup(p_input->psz_source);
     vlc_value_t val;
-    subtitle_demux_t *p_sub;
     int64_t i_microsecondperframe;
 
+    subtitle_demux_t *p_sub_toselect = NULL;
+    char             *psz_sub_file = NULL;
+
     /* Skip the plug-in names */
     while( *psz_parser && *psz_parser != ':' )
     {
@@ -686,6 +707,7 @@ static int InitThread( input_thread_t * p_input )
         {
             free( p_input->psz_dupsource );
         }
+        input_EsOutDelete( p_input->p_es_out );
         return VLC_EGENERIC;
     }
 
@@ -723,18 +745,19 @@ static int InitThread( input_thread_t * p_input )
                 {
                     free( p_input->psz_dupsource );
                 }
+                input_EsOutDelete( p_input->p_es_out );
                 return VLC_EGENERIC;
             }
         }
     }
 
     /* Create the stream_t facilities */
-    p_input->s = stream_OpenInput( p_input );
+    p_input->s = input_StreamNew( p_input );
     if( p_input->s == NULL )
     {
-        /* should nver occur yet */
+        /* should never occur yet */
 
-        msg_Err( p_input, "cannot create stream_t !" );
+        msg_Err( p_input, "cannot create stream_t" );
 
         module_Unneed( p_input, p_input->p_access );
         if ( p_input->stream.p_sout != NULL )
@@ -747,6 +770,7 @@ static int InitThread( input_thread_t * p_input )
         {
             free( p_input->psz_dupsource );
         }
+        input_EsOutDelete( p_input->p_es_out );
         return VLC_EGENERIC;
     }
 
@@ -761,7 +785,7 @@ static int InitThread( input_thread_t * p_input )
         msg_Err( p_input, "no suitable demux module for `%s/%s://%s'",
                  p_input->psz_access, p_input->psz_demux, p_input->psz_name );
 
-        stream_Release( p_input->s );
+        input_StreamDelete( p_input->s );
         module_Unneed( p_input, p_input->p_access );
         if ( p_input->stream.p_sout != NULL )
         {
@@ -773,6 +797,7 @@ static int InitThread( input_thread_t * p_input )
         {
             free( p_input->psz_dupsource );
         }
+        input_EsOutDelete( p_input->p_es_out );
         return VLC_EGENERIC;
     }
 
@@ -781,6 +806,84 @@ static int InitThread( input_thread_t * p_input )
     p_input->p_sys->i_sub = 0;
     p_input->p_sys->sub   = NULL;
 
+    /* get meta informations */
+    if( !demux_Control( p_input, DEMUX_GET_META, &meta ) )
+    {
+        playlist_t *p_playlist = (playlist_t *)vlc_object_find( p_input,
+                                         VLC_OBJECT_PLAYLIST,  FIND_PARENT);
+        playlist_item_t *p_item = NULL;
+        input_info_category_t *p_cat;
+        int i;
+
+        if( p_playlist )
+        {
+            vlc_mutex_lock( &p_playlist->object_lock );
+            p_item = playlist_ItemGetByPos( p_playlist, -1 );
+            if( p_item )
+            {
+                vlc_mutex_lock( &p_item->lock );
+            }
+            vlc_mutex_unlock( &p_playlist->object_lock );
+        }
+
+        msg_Dbg( p_input, "meta informations:" );
+        if( meta->i_meta > 0 )
+        {
+            p_cat = input_InfoCategory( p_input, _("File") );
+            for( i = 0; i < meta->i_meta; i++ )
+            {
+                msg_Dbg( p_input, "  - '%s' = '%s'", _(meta->name[i]), meta->value[i] );
+                if( !strcmp( meta->name[i], VLC_META_TITLE ) )
+                {
+                    playlist_ItemSetName( p_item, meta->value[i] );
+                }
+                if( !strcmp( meta->name[i], VLC_META_AUTHOR ) )
+                {
+                    playlist_ItemAddInfo( p_item, _("General"), _("Author"),
+                                            meta->value[i] );
+                }
+                input_AddInfo( p_cat, _(meta->name[i]), "%s", meta->value[i] );
+                if( p_item )
+                {
+                    playlist_ItemAddInfo( p_item, _("File"),
+                                          _(meta->name[i]), "%s", meta->value[i] );
+                }
+            }
+        }
+        for( i = 0; i < meta->i_track; i++ )
+        {
+            vlc_meta_t *tk = meta->track[i];
+            int j;
+
+            msg_Dbg( p_input, "  - track[%d]:", i );
+            if( tk->i_meta > 0 )
+            {
+                char *psz_cat = malloc( strlen(_("Stream")) + 10 );
+                sprintf( psz_cat, "%s %d", _("Stream"), i );
+                p_cat = input_InfoCategory( p_input, psz_cat );
+
+                for( j = 0; j < tk->i_meta; j++ )
+                {
+                    msg_Dbg( p_input, "     - '%s' = '%s'", _(tk->name[j]), tk->value[j] );
+                    input_AddInfo( p_cat, _(tk->name[j]), "%s", tk->value[j] );
+                    if( p_item )
+                    {
+                        playlist_ItemAddInfo( p_item, psz_cat,
+                                              _(tk->name[j]), "%s", tk->value[j] );
+                    }
+                }
+            }
+        }
+
+        if( p_item )
+        {
+            vlc_mutex_unlock( &p_item->lock );
+        }
+        if( p_playlist ) vlc_object_release( p_playlist );
+
+        vlc_meta_Delete( meta );
+    }
+
     /* get length */
     if( !demux_Control( p_input, DEMUX_GET_LENGTH, &i_length ) && i_length > 0 )
     {
@@ -789,15 +892,13 @@ static int InitThread( input_thread_t * p_input )
                                                    FIND_PARENT );
         if( p_playlist )
         {
-            vlc_mutex_lock( &p_playlist->object_lock );
-            p_playlist->pp_items[ p_playlist->i_index ]->i_duration = i_length;
-            val.b_bool = VLC_TRUE;
-            vlc_mutex_unlock( &p_playlist->object_lock );
-            var_Set( p_playlist, "intf-change", val );
+            playlist_SetDuration( p_playlist, -1 , i_length );
+            val.b_bool = p_playlist->i_index;
+            var_Set( p_playlist, "item-change", val );
             vlc_object_release( p_playlist );
         }
-    }            
-       
+    }
+
 
     /* get fps */
     if( demux_Control( p_input, DEMUX_GET_FPS, &f_fps ) || f_fps < 0.1 )
@@ -813,31 +914,40 @@ static int InitThread( input_thread_t * p_input )
     var_Get( p_input, "sub-file", &val );
     if( val.psz_string && *val.psz_string )
     {
-        if( ( p_sub = subtitle_New( p_input, strdup(val.psz_string), i_microsecondperframe, 0 ) ) )
-        {
-            /* Select this ES by default */
-            es_out_Control( p_input->p_es_out, ES_OUT_SET_ES_STATE, p_sub->p_es, VLC_TRUE );
+        subtitle_demux_t *p_sub;
 
+        msg_Dbg( p_input, "force subtitle: %s", val.psz_string );
+        if( ( p_sub = subtitle_New( p_input, strdup(val.psz_string),
+                                    i_microsecondperframe ) ) )
+        {
+            p_sub_toselect = p_sub;
             TAB_APPEND( p_input->p_sys->i_sub, p_input->p_sys->sub, p_sub );
         }
     }
-    if( val.psz_string ) free( val.psz_string );
+    psz_sub_file = val.psz_string;
 
     var_Get( p_input, "sub-autodetect-file", &val );
     if( val.b_bool )
     {
+        subtitle_demux_t *p_sub;
         int i;
-        char **tmp = subtitles_Detect( p_input, "", p_input->psz_source );
+        char **tmp = subtitles_Detect( p_input, "", p_input->psz_name );
         char **tmp2 = tmp;
         for( i = 0; *tmp2 != NULL; i++ )
         {
-            if( ( p_sub = subtitle_New( p_input, strdup(*tmp2++), i_microsecondperframe, i ) ) )
+            if( psz_sub_file == NULL || strcmp( psz_sub_file, *tmp2 ) )
             {
-                TAB_APPEND( p_input->p_sys->i_sub, p_input->p_sys->sub, p_sub );
+                if( ( p_sub = subtitle_New( p_input, *tmp2,
+                                            i_microsecondperframe ) ) )
+                {
+                    TAB_APPEND( p_input->p_sys->i_sub, p_input->p_sys->sub, p_sub );
+                }
             }
+            free( *tmp2++ );
         }
-        free(tmp);
+        free( tmp );
     }
+    if( psz_sub_file ) free( psz_sub_file );
 
     es_out_Control( p_input->p_es_out, ES_OUT_SET_ACTIVE, VLC_TRUE );
     val.b_bool =  VLC_FALSE;
@@ -847,6 +957,11 @@ static int InitThread( input_thread_t * p_input )
     }
     es_out_Control( p_input->p_es_out, ES_OUT_SET_MODE,
                     val.b_bool ? ES_OUT_MODE_ALL : ES_OUT_MODE_AUTO );
+    if( p_sub_toselect )
+    {
+        es_out_Control( p_input->p_es_out, ES_OUT_SET_ES,
+                        p_sub_toselect->p_es, VLC_TRUE );
+    }
 
     return VLC_SUCCESS;
 }
@@ -932,7 +1047,7 @@ static void EndThread( input_thread_t * p_input )
     }
 
     /* Destroy the stream_t facilities */
-    if( p_input->s ) stream_Release( p_input->s );
+    if( p_input->s ) input_StreamDelete( p_input->s );
 
     /* Destroy es out */
     if( p_input->p_es_out ) input_EsOutDelete( p_input->p_es_out );
