@@ -1,8 +1,8 @@
 /*****************************************************************************
  * input_dec.c: Functions for the management of decoders
  *****************************************************************************
- * Copyright (C) 1999-2001 VideoLAN
- * $Id: input_dec.c,v 1.83 2003/12/24 09:46:08 gbazin Exp $
+ * Copyright (C) 1999-2004 VideoLAN
+ * $Id: input_dec.c,v 1.92 2004/02/25 18:47:02 gbazin Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Gildas Bazin <gbazin@netcourrier.com>
@@ -66,7 +66,8 @@ struct decoder_owner_sys_t
 
     vout_thread_t   *p_vout;
 
-    sout_packetizer_input_t *p_sout;
+    sout_instance_t         *p_sout;
+    sout_packetizer_input_t *p_sout_input;
 
     /* Current format in use by the output */
     video_format_t video;
@@ -78,46 +79,34 @@ struct decoder_owner_sys_t
 
     /* */
     input_buffers_t *p_method_data;
+    es_descriptor_t *p_es_descriptor;
 };
 
 
-/*****************************************************************************
- * input_RunDecoder: spawns a new decoder thread
- *****************************************************************************/
+/**
+ * Spawns a new decoder thread
+ *
+ * \param p_input the input thread
+ * \param p_es the es descriptor
+ * \return the spawned decoder object
+ */
 decoder_t * input_RunDecoder( input_thread_t * p_input, es_descriptor_t * p_es )
 {
     decoder_t      *p_dec = NULL;
     vlc_value_t    val;
 
     /* If we are in sout mode, search for packetizer module */
-    var_Get( p_input, "sout", &val );
-    if( !p_es->b_force_decoder && val.psz_string && *val.psz_string )
+    if( !p_es->b_force_decoder && p_input->stream.p_sout )
     {
-        free( val.psz_string );
-        val.b_bool = VLC_TRUE;
-
-        if( p_es->i_cat == AUDIO_ES )
+        /* Create the decoder configuration structure */
+        p_dec = CreateDecoder( p_input, p_es, VLC_OBJECT_PACKETIZER );
+        if( p_dec == NULL )
         {
-            var_Get( p_input, "sout-audio", &val );
-        }
-        else if( p_es->i_cat == VIDEO_ES )
-        {
-            var_Get( p_input, "sout-video", &val );
+            msg_Err( p_input, "could not create packetizer" );
+            return NULL;
         }
 
-        if( val.b_bool )
-        {
-            /* Create the decoder configuration structure */
-            p_dec = CreateDecoder( p_input, p_es, VLC_OBJECT_PACKETIZER );
-            if( p_dec == NULL )
-            {
-                msg_Err( p_input, "could not create packetizer" );
-                return NULL;
-            }
-
-            p_dec->p_module =
-                module_Need( p_dec, "packetizer", "$packetizer" );
-        }
+        p_dec->p_module = module_Need( p_dec, "packetizer", "$packetizer" );
     }
     else
     {
@@ -131,11 +120,9 @@ decoder_t * input_RunDecoder( input_thread_t * p_input, es_descriptor_t * p_es )
 
         /* default Get a suitable decoder module */
         p_dec->p_module = module_Need( p_dec, "decoder", "$codec" );
-
-        if( val.psz_string ) free( val.psz_string );
     }
 
-    if( !p_dec || !p_dec->p_module )
+    if( !p_dec->p_module )
     {
         msg_Err( p_dec, "no suitable decoder module for fourcc `%4.4s'.\n"
                  "VLC probably does not support this sound or video format.",
@@ -179,9 +166,13 @@ decoder_t * input_RunDecoder( input_thread_t * p_input, es_descriptor_t * p_es )
     return p_dec;
 }
 
-/*****************************************************************************
- * input_EndDecoder: kills a decoder thread and waits until it's finished
- *****************************************************************************/
+/**
+ * Kills a decoder thread and waits until it's finished
+ *
+ * \param p_input the input thread
+ * \param p_es the es descriptor
+ * \return nothing
+ */
 void input_EndDecoder( input_thread_t * p_input, es_descriptor_t * p_es )
 {
     decoder_t *p_dec = p_es->p_dec;
@@ -233,11 +224,13 @@ void input_EndDecoder( input_thread_t * p_input, es_descriptor_t * p_es )
     p_input->stream.b_changed = 1;
 }
 
-/*****************************************************************************
- * input_DecodePES
- *****************************************************************************
+/**
  * Put a PES in the decoder's fifo.
- *****************************************************************************/
+ *
+ * \param p_dec the decoder object
+ * \param p_pes the pes packet
+ * \return nothing
+ */
 void input_DecodePES( decoder_t * p_dec, pes_packet_t * p_pes )
 {
     data_packet_t *p_data;
@@ -264,7 +257,8 @@ void input_DecodePES( decoder_t * p_dec, pes_packet_t * p_pes )
             }
             p_block->i_pts = p_pes->i_pts;
             p_block->i_dts = p_pes->i_dts;
-            p_block->b_discontinuity = p_pes->b_discontinuity;
+            if( p_pes->b_discontinuity )
+                p_block->i_flags |= BLOCK_FLAG_DISCONTINUITY;
             p_block->i_rate = p_pes->i_rate;
 
             input_DecodeBlock( p_dec, p_block );
@@ -273,11 +267,13 @@ void input_DecodePES( decoder_t * p_dec, pes_packet_t * p_pes )
 
     input_DeletePES( p_dec->p_owner->p_method_data, p_pes );
 }
-/*****************************************************************************
- * input_DecodeBlock
- *****************************************************************************
+
+/**
  * Put a block_t in the decoder's fifo.
- *****************************************************************************/
+ *
+ * \param p_dec the decoder object
+ * \param p_block the data block
+ */
 void input_DecodeBlock( decoder_t * p_dec, block_t *p_block )
 {
     if( p_dec->p_owner->b_own_thread )
@@ -297,26 +293,32 @@ void input_DecodeBlock( decoder_t * p_dec, block_t *p_block )
     }
 }
 
-/*****************************************************************************
+/**
  * Create a NULL packet for padding in case of a data loss
- *****************************************************************************/
+ *
+ * \param p_input the input thread
+ * \param p_es es descriptor
+ * \return nothing
+ */
 static void input_NullPacket( input_thread_t * p_input,
                               es_descriptor_t * p_es )
 {
     block_t *p_block = block_New( p_input, PADDING_PACKET_SIZE );
-
     if( p_block )
     {
         memset( p_block->p_buffer, 0, PADDING_PACKET_SIZE );
-        p_block->b_discontinuity = 1;
+        p_block->i_flags |= BLOCK_FLAG_DISCONTINUITY;
 
         block_FifoPut( p_es->p_dec->p_owner->p_fifo, p_block );
     }
 }
 
-/*****************************************************************************
- * input_EscapeDiscontinuity: send a NULL packet to the decoders
- *****************************************************************************/
+/**
+ * Send a NULL packet to the decoders
+ *
+ * \param p_input the input thread
+ * \return nothing
+ */
 void input_EscapeDiscontinuity( input_thread_t * p_input )
 {
     unsigned int i_es, i;
@@ -335,9 +337,12 @@ void input_EscapeDiscontinuity( input_thread_t * p_input )
     }
 }
 
-/*****************************************************************************
- * input_EscapeAudioDiscontinuity: send a NULL packet to the audio decoders
- *****************************************************************************/
+/**
+ * Send a NULL packet to the audio decoders
+ *
+ * \param p_input the input thread
+ * \return nothing
+ */
 void input_EscapeAudioDiscontinuity( input_thread_t * p_input )
 {
     unsigned int i_es, i;
@@ -356,9 +361,14 @@ void input_EscapeAudioDiscontinuity( input_thread_t * p_input )
     }
 }
 
-/*****************************************************************************
- * CreateDecoder: create a decoder object
- *****************************************************************************/
+/**
+ * Create a decoder object
+ *
+ * \param p_input the input thread
+ * \param p_es the es descriptor
+ * \param i_object_type Object type as define in include/vlc_objects.h
+ * \return the decoder object
+ */
 static decoder_t * CreateDecoder( input_thread_t * p_input,
                                   es_descriptor_t * p_es, int i_object_type )
 {
@@ -455,7 +465,9 @@ static decoder_t * CreateDecoder( input_thread_t * p_input,
     p_dec->p_owner->p_aout = NULL;
     p_dec->p_owner->p_aout_input = NULL;
     p_dec->p_owner->p_vout = NULL;
-    p_dec->p_owner->p_sout = NULL;
+    p_dec->p_owner->p_sout = p_input->stream.p_sout;
+    p_dec->p_owner->p_sout_input = NULL;
+    p_dec->p_owner->p_es_descriptor = p_es;
     /* decoder fifo */
     if( ( p_dec->p_owner->p_fifo = block_FifoNew( p_dec ) ) == NULL )
     {
@@ -477,9 +489,12 @@ static decoder_t * CreateDecoder( input_thread_t * p_input,
     return p_dec;
 }
 
-/*****************************************************************************
- * DecoderThread: the decoding main loop
- *****************************************************************************/
+/**
+ * The decoding main loop
+ *
+ * \param p_dec the decoder
+ * \return 0
+ */
 static int DecoderThread( decoder_t * p_dec )
 {
     block_t       *p_block;
@@ -521,9 +536,13 @@ static int DecoderThread( decoder_t * p_dec )
     return 0;
 }
 
-/*****************************************************************************
- * DecoderDecode: decode a block
- *****************************************************************************/
+/**
+ * Decode a block
+ *
+ * \param p_dec the decoder object
+ * \param p_block the block to decode
+ * \return VLC_SUCCESS or an error code
+ */
 static int DecoderDecode( decoder_t *p_dec, block_t *p_block )
 {
     if( p_block->i_buffer <= 0 )
@@ -538,14 +557,24 @@ static int DecoderDecode( decoder_t *p_dec, block_t *p_block )
 
         while( (p_sout_block = p_dec->pf_packetize( p_dec, &p_block )) )
         {
-            if( !p_dec->p_owner->p_sout )
+            if( !p_dec->p_owner->p_sout_input )
             {
                 es_format_Copy( &p_dec->p_owner->sout, &p_dec->fmt_out );
+                if( p_dec->p_owner->p_es_descriptor->p_pgrm )
+                {
+                    p_dec->p_owner->sout.i_group =
+                        p_dec->p_owner->p_es_descriptor->p_pgrm->i_number;
+                }
+                p_dec->p_owner->sout.i_id = p_dec->p_owner->p_es_descriptor->i_id - 1;
+                if( p_dec->fmt_in.psz_language )
+                {
+                    p_dec->p_owner->sout.psz_language = strdup( p_dec->fmt_in.psz_language );
+                }
 
-                p_dec->p_owner->p_sout =
-                    sout_InputNew( p_dec, &p_dec->p_owner->sout );
+                p_dec->p_owner->p_sout_input =
+                    sout_InputNew( p_dec->p_owner->p_sout, &p_dec->p_owner->sout );
 
-                if( p_dec->p_owner->p_sout == NULL )
+                if( p_dec->p_owner->p_sout_input == NULL )
                 {
                     msg_Err( p_dec, "cannot create packetizer output" );
                     p_dec->b_error = VLC_TRUE;
@@ -566,7 +595,7 @@ static int DecoderDecode( decoder_t *p_dec, block_t *p_block )
                 sout_buffer_t *p_sout_buffer;
 
                 p_sout_buffer =
-                    sout_BufferNew( p_dec->p_owner->p_sout->p_sout,
+                    sout_BufferNew( p_dec->p_owner->p_sout_input->p_sout,
                                     p_sout_block->i_buffer );
                 if( p_sout_buffer == NULL )
                 {
@@ -583,7 +612,7 @@ static int DecoderDecode( decoder_t *p_dec, block_t *p_block )
 
                 block_Release( p_sout_block );
 
-                sout_InputSendBuffer( p_dec->p_owner->p_sout, p_sout_buffer );
+                sout_InputSendBuffer( p_dec->p_owner->p_sout_input, p_sout_buffer );
 
                 p_sout_block = p_next;
             }
@@ -615,16 +644,19 @@ static int DecoderDecode( decoder_t *p_dec, block_t *p_block )
     }
     else
     {
-        msg_Err( p_dec, "unknown ES format !!" );
+        msg_Err( p_dec, "unknown ES format" );
         p_dec->b_error = 1;
     }
 
     return p_dec->b_error ? VLC_EGENERIC : VLC_SUCCESS;
 }
 
-/*****************************************************************************
- * DeleteDecoder: destroys a decoder object
- *****************************************************************************/
+/**
+ * Destroys a decoder object
+ *
+ * \param p_dec the decoder object
+ * \return nothing
+ */
 static void DeleteDecoder( decoder_t * p_dec )
 {
     vlc_object_detach( p_dec );
@@ -662,9 +694,9 @@ static void DeleteDecoder( decoder_t * p_dec )
         vout_Request( p_dec, p_dec->p_owner->p_vout, 0, 0, 0, 0 );
     }
 
-    if( p_dec->p_owner->p_sout )
+    if( p_dec->p_owner->p_sout_input )
     {
-        sout_InputDelete( p_dec->p_owner->p_sout );
+        sout_InputDelete( p_dec->p_owner->p_sout_input );
         if( p_dec->p_owner->sout.i_extra ) free(p_dec->p_owner->sout.p_extra);
     }
 
@@ -760,7 +792,7 @@ static picture_t *vout_new_buffer( decoder_t *p_dec )
     /* Get a new picture */
     while( !(p_pic = vout_CreatePicture( p_sys->p_vout, 0, 0, 0 ) ) )
     {
-        int i_pic;
+        int i_pic, i_ready_pic = 0;
 
         if( p_dec->b_die || p_dec->b_error )
         {
@@ -772,14 +804,17 @@ static picture_t *vout_new_buffer( decoder_t *p_dec )
         for( i_pic = 0; i_pic < p_dec->p_owner->p_vout->render.i_pictures;
              i_pic++ )
         {
+            if( p_pic->i_status == READY_PICTURE && i_ready_pic++ > 0 ) break;
+
             if( p_pic->i_status != DISPLAYED_PICTURE &&
-                p_pic->i_status != RESERVED_PICTURE ) break;
+                p_pic->i_status != RESERVED_PICTURE &&
+                p_pic->i_status != READY_PICTURE ) break;
 
             if( !p_pic->i_refcount ) break;
         }
         if( i_pic == p_dec->p_owner->p_vout->render.i_pictures )
         {
-            msg_Err( p_dec, "decoder is leaking pictures, reseting the heap" );
+            msg_Err( p_dec, "decoder is leaking pictures, resetting the heap" );
 
             /* Just free all the pictures */
             for( i_pic = 0; i_pic < p_dec->p_owner->p_vout->render.i_pictures;
