@@ -1,8 +1,8 @@
 /*****************************************************************************
  * libvlc.c: main libvlc source
  *****************************************************************************
- * Copyright (C) 1998-2002 VideoLAN
- * $Id: libvlc.c,v 1.94 2003/07/23 01:13:48 gbazin Exp $
+ * Copyright (C) 1998-2004 VideoLAN
+ * $Id: libvlc.c,v 1.117 2004/02/22 15:41:27 massiot Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -99,6 +99,25 @@ static void ShowConsole   ( void );
 #endif
 static int  ConsoleWidth  ( void );
 
+static int  VerboseCallback( vlc_object_t *, char const *,
+                             vlc_value_t, vlc_value_t, void * );
+
+/*****************************************************************************
+ * vlc_current_object: return the current object.
+ *****************************************************************************
+ * If i_object is non-zero, return the corresponding object. Otherwise,
+ * return the statically allocated p_vlc object.
+ *****************************************************************************/
+vlc_t * vlc_current_object( int i_object )
+{
+    if( i_object )
+    {
+         return vlc_object_get( p_libvlc, i_object );
+    }
+
+    return p_static_vlc;
+}
+
 /*****************************************************************************
  * VLC_Version: return the libvlc version.
  *****************************************************************************
@@ -185,6 +204,7 @@ int VLC_Create( void )
     {
         return VLC_EGENERIC;
     }
+    p_vlc->thread_id = 0;
     vlc_thread_set_priority( p_vlc, VLC_THREAD_PRIORITY_LOW );
 
     p_vlc->psz_object_name = "root";
@@ -221,13 +241,11 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
     char *       psz_parser;
     char *       psz_language;
     vlc_bool_t   b_exit = VLC_FALSE;
-    vlc_t *      p_vlc;
+    vlc_t *      p_vlc = vlc_current_object( i_object );
     module_t    *p_help_module;
     playlist_t  *p_playlist;
     vlc_value_t  lockval;
 
-    p_vlc = i_object ? vlc_object_get( p_libvlc, i_object ) : p_static_vlc;
-
     if( !p_vlc )
     {
         return VLC_ENOOBJ;
@@ -270,8 +288,8 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
     vlc_mutex_lock( lockval.p_address );
     if( libvlc.p_module_bank == NULL )
     {
-        module_InitBank( p_libvlc );
-        module_LoadMain( p_libvlc );
+        module_InitBank( p_vlc );
+        module_LoadMain( p_vlc );
     }
     vlc_mutex_unlock( lockval.p_address );
     var_Destroy( p_libvlc, "libvlc" );
@@ -285,6 +303,7 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
         return VLC_EGENERIC;
     }
     p_help_module->psz_object_name = "help";
+    p_help_module->psz_longname = N_("Help options");
     config_Duplicate( p_help_module, p_help_config );
     vlc_object_attach( p_help_module, libvlc.p_module_bank );
     /* End hack */
@@ -302,7 +321,7 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
     /* Check for short help option */
     if( config_GetInt( p_vlc, "help" ) )
     {
-        fprintf( stdout, _("Usage: %s [options] [items]...\n\n"),
+        fprintf( stdout, _("Usage: %s [options] [items]...\n"),
                          p_vlc->psz_object_name );
         Usage( p_vlc, "main" );
         Usage( p_vlc, "help" );
@@ -315,6 +334,10 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
         b_exit = VLC_TRUE;
     }
 
+    /* Set the config file stuff */
+    p_vlc->psz_homedir = config_GetHomeDir();
+    p_vlc->psz_configfile = config_GetPsz( p_vlc, "config" );
+
     /* Hack: remove the help module here */
     vlc_object_detach( p_help_module );
     /* End hack */
@@ -334,7 +357,6 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
 
     /* This ain't really nice to have to reload the config here but it seems
      * the only way to do it. */
-    p_vlc->psz_homedir = config_GetHomeDir();
     config_LoadConfigFile( p_vlc, "main" );
     config_LoadCmdLine( p_vlc, &i_argc, ppsz_argv, VLC_TRUE );
 
@@ -350,16 +372,13 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
 
         textdomain( PACKAGE );
 
-#if defined( SYS_BEOS ) || defined ( SYS_DARWIN ) || \
-    ( defined( WIN32 ) && !defined( HAVE_INCLUDED_GETTEXT ) )
-        /* BeOS only support UTF8 strings */
-        /* Mac OS X prefers UTF8 */
+#if defined( ENABLE_UTF8 )
         bind_textdomain_codeset( PACKAGE, "UTF-8" );
 #endif
 
         module_EndBank( p_vlc );
-        module_InitBank( p_libvlc );
-        module_LoadMain( p_libvlc );
+        module_InitBank( p_vlc );
+        module_LoadMain( p_vlc );
         config_LoadCmdLine( p_vlc, &i_argc, ppsz_argv, VLC_TRUE );
     }
     if( psz_language ) free( psz_language );
@@ -371,8 +390,13 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
      * list of configuration options exported by each module and loads their
      * default values.
      */
-    module_LoadBuiltins( p_libvlc );
-    module_LoadPlugins( p_libvlc );
+    module_LoadBuiltins( p_vlc );
+    module_LoadPlugins( p_vlc );
+    if( p_vlc->b_die )
+    {
+        b_exit = VLC_TRUE;
+    }
+
     msg_Dbg( p_vlc, "module bank initialized, found %i modules",
                     libvlc.p_module_bank->i_children );
 
@@ -400,14 +424,32 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
         b_exit = VLC_TRUE;
     }
 
+    /* Check for config file options */
+    if( config_GetInt( p_vlc, "reset-config" ) )
+    {
+        vlc_object_detach( p_help_module );
+        config_ResetAll( p_vlc );
+        config_LoadCmdLine( p_vlc, &i_argc, ppsz_argv, VLC_TRUE );
+        config_SaveConfigFile( p_vlc, NULL );
+        vlc_object_attach( p_help_module, libvlc.p_module_bank );
+    }
+    if( config_GetInt( p_vlc, "save-config" ) )
+    {
+        vlc_object_detach( p_help_module );
+        config_LoadConfigFile( p_vlc, NULL );
+        config_LoadCmdLine( p_vlc, &i_argc, ppsz_argv, VLC_TRUE );
+        config_SaveConfigFile( p_vlc, NULL );
+        vlc_object_attach( p_help_module, libvlc.p_module_bank );
+    }
+
     /* Hack: remove the help module here */
     vlc_object_detach( p_help_module );
-    config_Free( p_help_module );
-    vlc_object_destroy( p_help_module );
     /* End hack */
 
     if( b_exit )
     {
+        config_Free( p_help_module );
+        vlc_object_destroy( p_help_module );
         /*module_EndBank( p_vlc );*/
         if( i_object ) vlc_object_release( p_vlc );
         return VLC_EEXIT;
@@ -416,9 +458,12 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
     /*
      * Override default configuration with config file settings
      */
-    p_vlc->psz_homedir = config_GetHomeDir();
     config_LoadConfigFile( p_vlc, NULL );
 
+    /* Hack: insert the help module here */
+    vlc_object_attach( p_help_module, libvlc.p_module_bank );
+    /* End hack */
+
     /*
      * Override configuration with command line settings
      */
@@ -431,31 +476,39 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
                  "that they are valid.\nPress the RETURN key to continue..." );
         getchar();
 #endif
+        vlc_object_detach( p_help_module );
+        config_Free( p_help_module );
+        vlc_object_destroy( p_help_module );
         /*module_EndBank( p_vlc );*/
         if( i_object ) vlc_object_release( p_vlc );
         return VLC_EGENERIC;
     }
 
+    /* Hack: remove the help module here */
+    vlc_object_detach( p_help_module );
+    config_Free( p_help_module );
+    vlc_object_destroy( p_help_module );
+    /* End hack */
+
     /*
      * System specific configuration
      */
-    system_Configure( p_vlc );
+    system_Configure( p_vlc, &i_argc, ppsz_argv );
 
     /*
      * Message queue options
      */
+
+    var_Create( p_vlc, "verbose", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
     if( config_GetInt( p_vlc, "quiet" ) )
     {
-        libvlc.i_verbose = -1;
-    }
-    else
-    {
-        int i_tmp = config_GetInt( p_vlc, "verbose" );
-        if( i_tmp >= 0 )
-        {
-            libvlc.i_verbose = __MIN( i_tmp, 2 );
-        }
+        vlc_value_t val;
+        val.i_int = -1;
+        var_Set( p_vlc, "verbose", val );
     }
+    var_AddCallback( p_vlc, "verbose", VerboseCallback, NULL );
+    var_Change( p_vlc, "verbose", VLC_VAR_TRIGGER_CALLBACKS, NULL, NULL );
+
     libvlc.b_color = libvlc.b_color && config_GetInt( p_vlc, "color" );
 
     /*
@@ -464,7 +517,6 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
     msg_Flush( p_vlc );
 
     /* p_vlc initialization. FIXME ? */
-    p_vlc->i_desync = config_GetInt( p_vlc, "desync" ) * (mtime_t)1000;
 
 #if defined( __i386__ )
     if( !config_GetInt( p_vlc, "mmx" ) )
@@ -475,6 +527,8 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
         libvlc.i_cpu &= ~CPU_CAPABILITY_MMXEXT;
     if( !config_GetInt( p_vlc, "sse" ) )
         libvlc.i_cpu &= ~CPU_CAPABILITY_SSE;
+    if( !config_GetInt( p_vlc, "sse2" ) )
+        libvlc.i_cpu &= ~CPU_CAPABILITY_SSE2;
 #endif
 #if defined( __powerpc__ ) || defined( SYS_DARWIN )
     if( !config_GetInt( p_vlc, "altivec" ) )
@@ -497,6 +551,7 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
     PRINT_CAPABILITY( CPU_CAPABILITY_3DNOW, "3DNow!" );
     PRINT_CAPABILITY( CPU_CAPABILITY_MMXEXT, "MMXEXT" );
     PRINT_CAPABILITY( CPU_CAPABILITY_SSE, "SSE" );
+    PRINT_CAPABILITY( CPU_CAPABILITY_SSE2, "SSE2" );
     PRINT_CAPABILITY( CPU_CAPABILITY_ALTIVEC, "AltiVec" );
     PRINT_CAPABILITY( CPU_CAPABILITY_FPU, "FPU" );
     msg_Dbg( p_vlc, "CPU has capabilities %s", p_capabilities );
@@ -516,6 +571,14 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
         p_vlc->pf_memset = memset;
     }
 
+    /*
+     * Initialize hotkey handling
+     */
+    var_Create( p_vlc, "key-pressed", VLC_VAR_INTEGER );
+    p_vlc->p_hotkeys = malloc( sizeof(p_hotkeys) );
+    /* Do a copy (we don't need to modify the strings) */
+    memcpy( p_vlc->p_hotkeys, p_hotkeys, sizeof(p_hotkeys) );
+
     /*
      * Initialize playlist and get commandline files
      */
@@ -560,10 +623,26 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
         free( psz_modules );
     }
 
+    /*
+     * Allways load the hotkeys interface if it exists
+     */
+    VLC_AddIntf( 0, "hotkeys,none", VLC_FALSE );
+
     /*
      * FIXME: kludge to use a p_vlc-local variable for the Mozilla plugin
      */
     var_Create( p_vlc, "drawable", VLC_VAR_INTEGER );
+    var_Create( p_vlc, "drawableredraw", VLC_VAR_INTEGER );
+    var_Create( p_vlc, "drawablet", VLC_VAR_INTEGER );
+    var_Create( p_vlc, "drawablel", VLC_VAR_INTEGER );
+    var_Create( p_vlc, "drawableb", VLC_VAR_INTEGER );
+    var_Create( p_vlc, "drawabler", VLC_VAR_INTEGER );
+    var_Create( p_vlc, "drawablex", VLC_VAR_INTEGER );
+    var_Create( p_vlc, "drawabley", VLC_VAR_INTEGER );
+    var_Create( p_vlc, "drawablew", VLC_VAR_INTEGER );
+    var_Create( p_vlc, "drawableh", VLC_VAR_INTEGER );
+    var_Create( p_vlc, "drawableportx", VLC_VAR_INTEGER );
+    var_Create( p_vlc, "drawableporty", VLC_VAR_INTEGER );
 
     /*
      * Get input filenames given as commandline arguments
@@ -586,9 +665,7 @@ int VLC_AddIntf( int i_object, char const *psz_module, vlc_bool_t b_block )
 {
     int i_err;
     intf_thread_t *p_intf;
-    vlc_t *p_vlc;
-
-    p_vlc = i_object ? vlc_object_get( p_libvlc, i_object ) : p_static_vlc;
+    vlc_t *p_vlc = vlc_current_object( i_object );
 
     if( !p_vlc )
     {
@@ -628,9 +705,7 @@ int VLC_AddIntf( int i_object, char const *psz_module, vlc_bool_t b_block )
  *****************************************************************************/
 int VLC_Destroy( int i_object )
 {
-    vlc_t *p_vlc;
-
-    p_vlc = i_object ? vlc_object_get( p_libvlc, i_object ) : p_static_vlc;
+    vlc_t *p_vlc = vlc_current_object( i_object );
 
     if( !p_vlc )
     {
@@ -652,6 +727,18 @@ int VLC_Destroy( int i_object )
         p_vlc->psz_homedir = NULL;
     }
 
+    if( p_vlc->psz_configfile )
+    {
+        free( p_vlc->psz_configfile );
+        p_vlc->psz_configfile = NULL;
+    }
+
+    if( p_vlc->p_hotkeys )
+    {
+        free( p_vlc->p_hotkeys );
+        p_vlc->p_hotkeys = NULL;
+    }
+
     /*
      * XXX: Free module bank !
      */
@@ -685,13 +772,11 @@ int VLC_Destroy( int i_object )
  * VLC_Die: ask vlc to die.
  *****************************************************************************
  * This function sets p_vlc->b_die to VLC_TRUE, but does not do any other
- * task. It is your duty to call vlc_end and VLC_Destroy afterwards.
+ * task. It is your duty to call VLC_End and VLC_Destroy afterwards.
  *****************************************************************************/
 int VLC_Die( int i_object )
 {
-    vlc_t *p_vlc;
-
-    p_vlc = i_object ? vlc_object_get( p_libvlc, i_object ) : p_static_vlc;
+    vlc_t *p_vlc = vlc_current_object( i_object );
 
     if( !p_vlc )
     {
@@ -716,9 +801,7 @@ int VLC_AddTarget( int i_object, char const *psz_target,
 {
     int i_err;
     playlist_t *p_playlist;
-    vlc_t *p_vlc;
-
-    p_vlc = i_object ? vlc_object_get( p_libvlc, i_object ) : p_static_vlc;
+    vlc_t *p_vlc = vlc_current_object( i_object );
 
     if( !p_vlc )
     {
@@ -741,8 +824,8 @@ int VLC_AddTarget( int i_object, char const *psz_target,
         vlc_object_yield( p_playlist );
     }
 
-    i_err = playlist_Add( p_playlist, psz_target, ppsz_options, i_options,
-                          i_mode, i_pos );
+    i_err = playlist_AddExt( p_playlist, psz_target, psz_target,
+                             i_mode, i_pos, -1, ppsz_options, i_options);
 
     vlc_object_release( p_playlist );
 
@@ -757,11 +840,9 @@ int VLC_AddTarget( int i_object, char const *psz_target,
  *****************************************************************************/
 int VLC_Set( int i_object, char const *psz_var, vlc_value_t value )
 {
-    vlc_t *p_vlc;
+    vlc_t *p_vlc = vlc_current_object( i_object );
     int i_ret;
 
-    p_vlc = i_object ? vlc_object_get( p_libvlc, i_object ) : p_static_vlc;
-
     if( !p_vlc )
     {
         return VLC_ENOOBJ;
@@ -811,11 +892,9 @@ int VLC_Set( int i_object, char const *psz_var, vlc_value_t value )
  *****************************************************************************/
 int VLC_Get( int i_object, char const *psz_var, vlc_value_t *p_value )
 {
-    vlc_t *p_vlc;
+    vlc_t *p_vlc = vlc_current_object( i_object );
     int i_ret;
 
-    p_vlc = i_object ? vlc_object_get( p_libvlc, i_object ) : p_static_vlc;
-
     if( !p_vlc )
     {
         return VLC_ENOOBJ;
@@ -829,15 +908,84 @@ int VLC_Get( int i_object, char const *psz_var, vlc_value_t *p_value )
 
 /* FIXME: temporary hacks */
 
+
+/*****************************************************************************
+ * VLC_IsPlaying: Query for Playlist Status
+ *****************************************************************************/
+vlc_bool_t VLC_IsPlaying( int i_object )
+{
+
+    playlist_t * p_playlist;
+    vlc_bool_t   playing;
+
+    vlc_t *p_vlc = vlc_current_object( i_object );
+
+    /* Check that the handle is valid */
+    if( !p_vlc )
+    {
+        return VLC_ENOOBJ;
+    }
+
+    p_playlist = vlc_object_find( p_vlc, VLC_OBJECT_PLAYLIST, FIND_CHILD );
+
+    if( !p_playlist )
+    {
+        if( i_object ) vlc_object_release( p_vlc );
+        return VLC_ENOOBJ;
+    }
+
+    playing = playlist_IsPlaying( p_playlist );
+
+    vlc_object_release( p_playlist );
+
+    if( i_object ) vlc_object_release( p_vlc );
+
+    return playing;
+
+}
+
+
+/*****************************************************************************
+ * VLC_ClearPlaylist: Query for Playlist Status
+ *
+ * return: 0
+ *****************************************************************************/
+int VLC_ClearPlaylist( int i_object )
+{
+
+    playlist_t * p_playlist;
+    vlc_t *p_vlc = vlc_current_object( i_object );
+
+    /* Check that the handle is valid */
+    if( !p_vlc )
+    {
+        return VLC_ENOOBJ;
+    }
+
+    p_playlist = vlc_object_find( p_vlc, VLC_OBJECT_PLAYLIST, FIND_CHILD );
+
+    if( !p_playlist )
+    {
+        if( i_object ) vlc_object_release( p_vlc );
+        return VLC_ENOOBJ;
+    }
+
+    playlist_Clear(p_playlist);
+
+    vlc_object_release( p_playlist );
+
+    if( i_object ) vlc_object_release( p_vlc );
+    return 0;
+}
+
+
 /*****************************************************************************
  * VLC_Play: play
  *****************************************************************************/
 int VLC_Play( int i_object )
 {
     playlist_t * p_playlist;
-    vlc_t *p_vlc;
-
-    p_vlc = i_object ? vlc_object_get( p_libvlc, i_object ) : p_static_vlc;
+    vlc_t *p_vlc = vlc_current_object( i_object );
 
     /* Check that the handle is valid */
     if( !p_vlc )
@@ -879,9 +1027,7 @@ int VLC_Stop( int i_object )
     playlist_t    *   p_playlist;
     vout_thread_t *   p_vout;
     aout_instance_t * p_aout;
-    vlc_t *p_vlc;
-
-    p_vlc = i_object ? vlc_object_get( p_libvlc, i_object ) : p_static_vlc;
+    vlc_t *p_vlc = vlc_current_object( i_object );
 
     /* Check that the handle is valid */
     if( !p_vlc )
@@ -893,6 +1039,7 @@ int VLC_Stop( int i_object )
      * Ask the interfaces to stop and destroy them
      */
     msg_Dbg( p_vlc, "removing all interfaces" );
+
     while( (p_intf = vlc_object_find( p_vlc, VLC_OBJECT_INTF, FIND_CHILD )) )
     {
         intf_StopThread( p_intf );
@@ -904,6 +1051,7 @@ int VLC_Stop( int i_object )
     /*
      * Free playlists
      */
+
     msg_Dbg( p_vlc, "removing all playlists" );
     while( (p_playlist = vlc_object_find( p_vlc, VLC_OBJECT_PLAYLIST,
                                           FIND_CHILD )) )
@@ -945,9 +1093,7 @@ int VLC_Stop( int i_object )
 int VLC_Pause( int i_object )
 {
     input_thread_t *p_input;
-    vlc_t *p_vlc;
-
-    p_vlc = i_object ? vlc_object_get( p_libvlc, i_object ) : p_static_vlc;
+    vlc_t *p_vlc = vlc_current_object( i_object );
 
     if( !p_vlc )
     {
@@ -975,9 +1121,7 @@ int VLC_Pause( int i_object )
 int VLC_FullScreen( int i_object )
 {
     vout_thread_t *p_vout;
-    vlc_t *p_vlc;
-
-    p_vlc = i_object ? vlc_object_get( p_libvlc, i_object ) : p_static_vlc;
+    vlc_t *p_vlc = vlc_current_object( i_object );
 
     if( !p_vlc )
     {
@@ -1063,10 +1207,7 @@ static void SetLanguage ( char const *psz_lang )
     /* Set the default domain */
     textdomain( PACKAGE );
 
-#if defined( SYS_BEOS ) || defined ( SYS_DARWIN ) || \
-    ( defined( WIN32 ) && !defined( HAVE_INCLUDED_GETTEXT ) )
-    /* BeOS only support UTF8 strings */
-    /* Mac OS X prefers UTF8 */
+#if defined( ENABLE_UTF8 )
     bind_textdomain_codeset( PACKAGE, "UTF-8" );
 #endif
 
@@ -1138,6 +1279,7 @@ static void Usage( vlc_t *p_this, char const *psz_module_name )
     char psz_short[4];
     int i_index;
     int i_width = ConsoleWidth() - (PADDING_SPACES+LINE_START+1);
+    vlc_bool_t b_advanced = config_GetInt( p_this, "advanced" );
 
     memset( psz_spaces, ' ', PADDING_SPACES+LINE_START );
     psz_spaces[PADDING_SPACES+LINE_START] = '\0';
@@ -1170,6 +1312,23 @@ static void Usage( vlc_t *p_this, char const *psz_module_name )
             continue;
         }
 
+        /* Ignore modules with only advanced config options if requested */
+        if( !b_advanced )
+        {
+            for( p_item = p_parser->p_config;
+                 p_item->i_type != CONFIG_HINT_END;
+                 p_item++ )
+            {
+                if( (p_item->i_type & CONFIG_ITEM) &&
+                    !p_item->b_advanced ) break;
+            }
+            if( p_item->i_type == CONFIG_HINT_END ) continue;
+        }
+
+        /* Print name of module */
+        if( strcmp( "main", p_parser->psz_object_name ) )
+        fprintf( stdout, "\n %s\n", p_parser->psz_longname );
+
         b_help_module = !strcmp( "help", p_parser->psz_object_name );
 
         /* Print module options */
@@ -1181,19 +1340,24 @@ static void Usage( vlc_t *p_this, char const *psz_module_name )
             char *psz_bra = NULL, *psz_type = NULL, *psz_ket = NULL;
             char *psz_suf = "", *psz_prefix = NULL;
             int i;
-            if ( p_item->b_advanced && !config_GetInt( p_this, "advanced" ))
+
+            /* Skip advanced options if requested */
+            if( p_item->b_advanced && !b_advanced )
             {
                 continue;
             }
+
             switch( p_item->i_type )
             {
             case CONFIG_HINT_CATEGORY:
             case CONFIG_HINT_USAGE:
-                fprintf( stdout, " %s\n", p_item->psz_text );
+                if( !strcmp( "main", p_parser->psz_object_name ) )
+                fprintf( stdout, "\n %s\n", p_item->psz_text );
                 break;
 
             case CONFIG_ITEM_STRING:
             case CONFIG_ITEM_FILE:
+            case CONFIG_ITEM_DIRECTORY:
             case CONFIG_ITEM_MODULE: /* We could also have "=<" here */
                 if( !p_item->ppsz_list )
                 {
@@ -1214,6 +1378,7 @@ static void Usage( vlc_t *p_this, char const *psz_module_name )
                     break;
                 }
             case CONFIG_ITEM_INTEGER:
+            case CONFIG_ITEM_KEY: /* FIXME: do something a bit more clever */
                 psz_bra = " <"; psz_type = _("integer"); psz_ket = ">";
                 break;
             case CONFIG_ITEM_FLOAT:
@@ -1416,7 +1581,7 @@ static void Version( void )
       _("This program comes with NO WARRANTY, to the extent permitted by "
         "law.\nYou may redistribute it under the terms of the GNU General "
         "Public License;\nsee the file named COPYING for details.\n"
-        "Written by the VideoLAN team at Ecole Centrale, Paris.\n") );
+        "Written by the VideoLAN team; see the AUTHORS file.\n") );
 
 #ifdef WIN32        /* Pause the console because it's destroyed when we exit */
     fprintf( stdout, _("\nPress the RETURN key to continue...\n") );
@@ -1481,3 +1646,15 @@ static int ConsoleWidth( void )
 
     return i_width;
 }
+
+static int VerboseCallback( vlc_object_t *p_this, const char *psz_variable,
+                     vlc_value_t old_val, vlc_value_t new_val, void *param)
+{
+    vlc_t *p_vlc = (vlc_t *)p_this;
+
+    if( new_val.i_int >= -1 )
+    {
+        p_vlc->p_libvlc->i_verbose = __MIN( new_val.i_int, 2 );
+    }
+    return VLC_SUCCESS;
+}
