@@ -1,8 +1,8 @@
 /*****************************************************************************
  * configuration.c management of the modules configuration
  *****************************************************************************
- * Copyright (C) 2001 VideoLAN
- * $Id: configuration.c,v 1.60 2003/07/23 01:13:48 gbazin Exp $
+ * Copyright (C) 2001-2004 VideoLAN
+ * $Id: configuration.c,v 1.76 2004/01/29 17:04:01 gbazin Exp $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -22,6 +22,7 @@
  *****************************************************************************/
 
 #include <vlc/vlc.h>
+#include "vlc_keys.h"
 
 #include <stdio.h>                                              /* sprintf() */
 #include <stdlib.h>                                      /* free(), strtol() */
@@ -52,10 +53,17 @@
 #if defined( HAVE_SYS_TYPES_H )
 #   include <sys/types.h>
 #endif
-#if defined( WIN32 ) && !defined( UNDER_CE )
-#   include <direct.h>
+#if defined( WIN32 )
+#   if !defined( UNDER_CE )
+#       include <direct.h>
+#   endif
+#include <tchar.h>
 #endif
 
+
+static int ConfigStringToKey( char * );
+static char *ConfigKeyToString( int );
+
 /*****************************************************************************
  * config_GetType: get the type of a variable (bool, int, float, string)
  *****************************************************************************
@@ -90,6 +98,9 @@ int __config_GetType( vlc_object_t *p_this, const char *psz_name )
         break;
 
     case CONFIG_ITEM_MODULE:
+        i_type = VLC_VAR_MODULE;
+        break;
+
     case CONFIG_ITEM_STRING:
         i_type = VLC_VAR_STRING;
         break;
@@ -130,6 +141,7 @@ int __config_GetInt( vlc_object_t *p_this, const char *psz_name )
         return -1;
     }
     if( (p_config->i_type!=CONFIG_ITEM_INTEGER) &&
+        (p_config->i_type!=CONFIG_ITEM_KEY) &&
         (p_config->i_type!=CONFIG_ITEM_BOOL) )
     {
         msg_Err( p_this, "option %s does not refer to an int", psz_name );
@@ -218,6 +230,7 @@ void __config_PutPsz( vlc_object_t *p_this,
                       const char *psz_name, const char *psz_value )
 {
     module_config_t *p_config;
+    vlc_value_t oldval, val;
 
     p_config = config_FindConfig( p_this, psz_name );
 
@@ -238,18 +251,24 @@ void __config_PutPsz( vlc_object_t *p_this,
 
     vlc_mutex_lock( p_config->p_lock );
 
-    /* free old string */
-    if( p_config->psz_value ) free( p_config->psz_value );
+    /* backup old value */
+    oldval.psz_string = p_config->psz_value;
 
     if( psz_value && *psz_value ) p_config->psz_value = strdup( psz_value );
     else p_config->psz_value = NULL;
 
+    val.psz_string = p_config->psz_value;
+
     vlc_mutex_unlock( p_config->p_lock );
 
     if( p_config->pf_callback )
     {
-        p_config->pf_callback( p_this );
+        p_config->pf_callback( p_this, psz_name, oldval, val,
+                               p_config->p_callback_data );
     }
+
+    /* free old string */
+    if( oldval.psz_string ) free( oldval.psz_string );
 }
 
 /*****************************************************************************
@@ -262,6 +281,7 @@ void __config_PutPsz( vlc_object_t *p_this,
 void __config_PutInt( vlc_object_t *p_this, const char *psz_name, int i_value )
 {
     module_config_t *p_config;
+    vlc_value_t oldval, val;
 
     p_config = config_FindConfig( p_this, psz_name );
 
@@ -272,12 +292,16 @@ void __config_PutInt( vlc_object_t *p_this, const char *psz_name, int i_value )
         return;
     }
     if( (p_config->i_type!=CONFIG_ITEM_INTEGER) &&
+        (p_config->i_type!=CONFIG_ITEM_KEY) &&
         (p_config->i_type!=CONFIG_ITEM_BOOL) )
     {
         msg_Err( p_this, "option %s does not refer to an int", psz_name );
         return;
     }
 
+    /* backup old value */
+    oldval.i_int = p_config->i_value;
+
     /* if i_min == i_max == 0, then do not use them */
     if ((p_config->i_min == 0) && (p_config->i_max == 0))
     {
@@ -296,9 +320,12 @@ void __config_PutInt( vlc_object_t *p_this, const char *psz_name, int i_value )
         p_config->i_value = i_value;
     }
 
+    val.i_int = p_config->i_value;
+
     if( p_config->pf_callback )
     {
-        p_config->pf_callback( p_this );
+        p_config->pf_callback( p_this, psz_name, oldval, val,
+                               p_config->p_callback_data );
     }
 }
 
@@ -312,6 +339,7 @@ void __config_PutFloat( vlc_object_t *p_this,
                         const char *psz_name, float f_value )
 {
     module_config_t *p_config;
+    vlc_value_t oldval, val;
 
     p_config = config_FindConfig( p_this, psz_name );
 
@@ -327,6 +355,9 @@ void __config_PutFloat( vlc_object_t *p_this,
         return;
     }
 
+    /* backup old value */
+    oldval.f_float = p_config->f_value;
+
     /* if f_min == f_max == 0, then do not use them */
     if ((p_config->f_min == 0) && (p_config->f_max == 0))
     {
@@ -345,9 +376,12 @@ void __config_PutFloat( vlc_object_t *p_this,
         p_config->f_value = f_value;
     }
 
+    val.f_float = p_config->f_value;
+
     if( p_config->pf_callback )
     {
-        p_config->pf_callback( p_this );
+        p_config->pf_callback( p_this, psz_name, oldval, val,
+                               p_config->p_callback_data );
     }
 }
 
@@ -395,6 +429,34 @@ module_config_t *config_FindConfig( vlc_object_t *p_this, const char *psz_name )
     return NULL;
 }
 
+/*****************************************************************************
+ * config_FindModule: find a specific module structure.
+ *****************************************************************************/
+module_t *config_FindModule( vlc_object_t *p_this, const char *psz_name )
+{
+    vlc_list_t *p_list;
+    module_t *p_module, *p_result = NULL;
+    int i_index;
+
+    if( !psz_name ) return NULL;
+
+    p_list = vlc_list_find( p_this, VLC_OBJECT_MODULE, FIND_ANYWHERE );
+
+    for( i_index = 0; i_index < p_list->i_count; i_index++ )
+    {
+        p_module = (module_t *)p_list->p_values[i_index].p_object;
+        if( !strcmp( p_module->psz_object_name, psz_name ) )
+        {
+             p_result = p_module;
+             break;
+        }
+    }
+
+    vlc_list_release( p_list );
+
+    return p_result;
+}
+
 /*****************************************************************************
  * config_Duplicate: creates a duplicate of a module's configuration data.
  *****************************************************************************
@@ -438,18 +500,10 @@ void config_Duplicate( module_t *p_module, module_config_t *p_orig )
     /* Do the duplication job */
     for( i = 0; i < i_lines ; i++ )
     {
-        p_module->p_config[i].i_type = p_orig[i].i_type;
-        p_module->p_config[i].i_short = p_orig[i].i_short;
-        p_module->p_config[i].i_value = p_orig[i].i_value;
+        p_module->p_config[i] = p_orig[i];
+
         p_module->p_config[i].i_value_orig = p_orig[i].i_value;
-        p_module->p_config[i].i_min = p_orig[i].i_min;
-        p_module->p_config[i].i_max = p_orig[i].i_max;
-        p_module->p_config[i].f_value = p_orig[i].f_value;
         p_module->p_config[i].f_value_orig = p_orig[i].f_value;
-        p_module->p_config[i].f_min = p_orig[i].f_min;
-        p_module->p_config[i].f_max = p_orig[i].f_max;
-        p_module->p_config[i].b_dirty = p_orig[i].b_dirty;
-        p_module->p_config[i].b_advanced = p_orig[i].b_advanced;
 
         p_module->p_config[i].psz_type = p_orig[i].psz_type ?
                                    strdup( p_orig[i].psz_type ) : NULL;
@@ -467,18 +521,63 @@ void config_Duplicate( module_t *p_module, module_config_t *p_orig )
         p_module->p_config[i].p_lock = &p_module->object_lock;
 
         /* duplicate the string list */
-        p_module->p_config[i].ppsz_list = NULL;
-        if( p_orig[i].ppsz_list )
+        if( p_orig[i].i_list )
         {
-            for( j = 0; p_orig[i].ppsz_list[j]; j++ );
-            p_module->p_config[i].ppsz_list = malloc( (j+1) *sizeof(char *) );
-            if( p_module->p_config[i].ppsz_list )
+            if( p_orig[i].ppsz_list )
             {
-                for( j = 0; p_orig[i].ppsz_list[j]; j++ )
-                    p_module->p_config[i].ppsz_list[j] =
-                        strdup( p_orig[i].ppsz_list[j] );
+                p_module->p_config[i].ppsz_list =
+                    malloc( (p_orig[i].i_list + 1) * sizeof(char *) );
+                if( p_module->p_config[i].ppsz_list )
+                {
+                    for( j = 0; j < p_orig[i].i_list; j++ )
+                        p_module->p_config[i].ppsz_list[j] =
+                            strdup( p_orig[i].ppsz_list[j] );
+                    p_module->p_config[i].ppsz_list[j] = NULL;
+                }
+            }
+            if( p_orig[i].ppsz_list_text )
+            {
+                p_module->p_config[i].ppsz_list_text =
+                    malloc( (p_orig[i].i_list + 1) * sizeof(char *) );
+                if( p_module->p_config[i].ppsz_list_text )
+                {
+                    for( j = 0; j < p_orig[i].i_list; j++ )
+                        p_module->p_config[i].ppsz_list_text[j] =
+                            strdup( _(p_orig[i].ppsz_list_text[j]) );
+                    p_module->p_config[i].ppsz_list_text[j] = NULL;
+                }
+            }
+            if( p_orig[i].pi_list )
+            {
+                p_module->p_config[i].pi_list =
+                    malloc( (p_orig[i].i_list + 1) * sizeof(int) );
+                if( p_module->p_config[i].pi_list )
+                {
+                    for( j = 0; j < p_orig[i].i_list; j++ )
+                        p_module->p_config[i].pi_list[j] =
+                            p_orig[i].pi_list[j];
+                }
+            }
+        }
+
+        /* duplicate the actions list */
+        if( p_orig[i].i_action )
+        {
+            int j;
+
+            p_module->p_config[i].ppf_action =
+                malloc( p_orig[i].i_action * sizeof(void *) );
+            p_module->p_config[i].ppsz_action_text =
+                malloc( p_orig[i].i_action * sizeof(char *) );
+
+            for( j = 0; j < p_orig[i].i_action; j++ )
+            {
+                p_module->p_config[i].ppf_action[j] =
+                    p_orig[i].ppf_action[j];
+                p_module->p_config[i].ppsz_action_text[j] =
+                    p_orig[i].ppsz_action_text[j] ?
+                    strdup( p_orig[i].ppsz_action_text[j] ) : NULL;
             }
-            p_module->p_config[i].ppsz_list[j] = NULL;
         }
 
         p_module->p_config[i].pf_callback = p_orig[i].pf_callback;
@@ -520,11 +619,29 @@ void config_Free( module_t *p_module )
         if( p_item->psz_value_orig )
             free( p_item->psz_value_orig );
 
-        if( p_item->ppsz_list )
+        if( p_item->i_list )
         {
-            for( i = 0; p_item->ppsz_list[i]; i++ )
-                free(p_item->ppsz_list[i]);
-            free( p_item->ppsz_list );
+            for( i = 0; i < p_item->i_list; i++ )
+            {
+                if( p_item->ppsz_list && p_item->ppsz_list[i] )
+                    free( p_item->ppsz_list[i] );
+                if( p_item->ppsz_list_text && p_item->ppsz_list_text[i] )
+                    free( p_item->ppsz_list_text[i] );
+            }
+            if( p_item->ppsz_list ) free( p_item->ppsz_list );
+            if( p_item->ppsz_list_text ) free( p_item->ppsz_list_text );
+            if( p_item->pi_list ) free( p_item->pi_list );
+        }
+
+        if( p_item->i_action )
+        {
+            for( i = 0; i < p_item->i_action; i++ )
+            {
+                if( p_item->ppsz_action_text[i] )
+                    free( p_item->ppsz_action_text[i] );
+            }
+            if( p_item->ppf_action ) free( p_item->ppf_action );
+            if( p_item->ppsz_action_text ) free( p_item->ppsz_action_text );
         }
     }
 
@@ -612,23 +729,34 @@ int __config_LoadConfigFile( vlc_object_t *p_this, const char *psz_module_name )
     FILE *file;
     char line[1024];
     char *p_index, *psz_option_name, *psz_option_value;
-    char *psz_filename, *psz_homedir;
+    char *psz_filename, *psz_homedir, *psz_configfile;
     int i_index;
 
-    psz_homedir = p_this->p_vlc->psz_homedir;
-    if( !psz_homedir )
+    psz_configfile = p_this->p_vlc->psz_configfile;
+    if( !psz_configfile || !psz_configfile )
     {
-        msg_Err( p_this, "psz_homedir is null" );
-        return -1;
+        psz_homedir = p_this->p_vlc->psz_homedir;
+        if( !psz_homedir )
+        {
+            msg_Err( p_this, "psz_homedir is null" );
+            return -1;
+        }
+        psz_filename = (char *)malloc( sizeof("/" CONFIG_DIR "/" CONFIG_FILE) +
+                                       strlen(psz_homedir) );
+        if( psz_filename )
+            sprintf( psz_filename, "%s/" CONFIG_DIR "/" CONFIG_FILE,
+                     psz_homedir );
     }
-    psz_filename = (char *)malloc( strlen("/" CONFIG_DIR "/" CONFIG_FILE) +
-                                   strlen(psz_homedir) + 1 );
+    else
+    {
+        psz_filename = strdup( psz_configfile );
+    }
+
     if( !psz_filename )
     {
         msg_Err( p_this, "out of memory" );
         return -1;
     }
-    sprintf( psz_filename, "%s/" CONFIG_DIR "/" CONFIG_FILE, psz_homedir );
 
     msg_Dbg( p_this, "opening config file %s", psz_filename );
 
@@ -724,7 +852,7 @@ int __config_LoadConfigFile( vlc_object_t *p_this, const char *psz_module_name )
                     case CONFIG_ITEM_INTEGER:
                         if( !*psz_option_value )
                             break;                    /* ignore empty option */
-                        p_item->i_value = atoi( psz_option_value);
+                        p_item->i_value = strtol( psz_option_value, 0, 0 );
 #if 0
                         msg_Dbg( p_this, "option \"%s\", value %i",
                                  p_item->psz_name, p_item->i_value );
@@ -740,6 +868,11 @@ int __config_LoadConfigFile( vlc_object_t *p_this, const char *psz_module_name )
                                  p_item->psz_name, (double)p_item->f_value );
 #endif
                         break;
+                    case CONFIG_ITEM_KEY:
+                        if( !*psz_option_value )
+                            break;                    /* ignore empty option */
+                        p_item->i_value = ConfigStringToKey( psz_option_value );
+                        break;
 
                     default:
                         vlc_mutex_lock( p_item->p_lock );
@@ -805,60 +938,77 @@ int __config_SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name )
     int i_sizebuf = 0;
     char *p_bigbuffer, *p_index;
     vlc_bool_t b_backup;
-    char *psz_filename, *psz_homedir;
+    char *psz_filename, *psz_homedir, *psz_configfile;
     int i_index;
 
     /* Acquire config file lock */
     vlc_mutex_lock( &p_this->p_vlc->config_lock );
 
-    psz_homedir = p_this->p_vlc->psz_homedir;
-    if( !psz_homedir )
+    psz_configfile = p_this->p_vlc->psz_configfile;
+    if( !psz_configfile || !psz_configfile )
     {
-        msg_Err( p_this, "psz_homedir is null" );
-        vlc_mutex_unlock( &p_this->p_vlc->config_lock );
-        return -1;
-    }
-    psz_filename = (char *)malloc( strlen("/" CONFIG_DIR "/" CONFIG_FILE) +
-                                   strlen(psz_homedir) + 1 );
-    if( !psz_filename )
-    {
-        msg_Err( p_this, "out of memory" );
-        vlc_mutex_unlock( &p_this->p_vlc->config_lock );
-        return -1;
-    }
-    sprintf( psz_filename, "%s/" CONFIG_DIR, psz_homedir );
+        psz_homedir = p_this->p_vlc->psz_homedir;
+        if( !psz_homedir )
+        {
+            msg_Err( p_this, "psz_homedir is null" );
+            vlc_mutex_unlock( &p_this->p_vlc->config_lock );
+            return -1;
+        }
+        psz_filename = (char *)malloc( sizeof("/" CONFIG_DIR "/" CONFIG_FILE) +
+                                       strlen(psz_homedir) );
+
+        if( psz_filename )
+            sprintf( psz_filename, "%s/" CONFIG_DIR, psz_homedir );
+
+        if( !psz_filename )
+        {
+            msg_Err( p_this, "out of memory" );
+            vlc_mutex_unlock( &p_this->p_vlc->config_lock );
+            return -1;
+        }
 
 #if defined( UNDER_CE )
-    {
-        wchar_t psz_new[ MAX_PATH ];
-        MultiByteToWideChar( CP_ACP, 0, psz_filename, -1, psz_new, MAX_PATH );
-        if( CreateDirectory( psz_new, NULL ) )
         {
-            msg_Err( p_this, "could not create %s", psz_filename );
+            wchar_t psz_new[ MAX_PATH ];
+            MultiByteToWideChar( CP_ACP, 0, psz_filename, -1, psz_new,
+                                 MAX_PATH );
+            if( CreateDirectory( psz_new, NULL ) )
+            {
+                msg_Err( p_this, "could not create %s", psz_filename );
+            }
         }
-    }
 
 #elif defined( HAVE_ERRNO_H )
 #   if defined( WIN32 )
-    if( mkdir( psz_filename ) && errno != EEXIST )
+        if( mkdir( psz_filename ) && errno != EEXIST )
 #   else
-    if( mkdir( psz_filename, 0755 ) && errno != EEXIST )
+        if( mkdir( psz_filename, 0755 ) && errno != EEXIST )
 #   endif
-    {
-        msg_Err( p_this, "could not create %s (%s)",
-                         psz_filename, strerror(errno) );
-    }
+        {
+            msg_Err( p_this, "could not create %s (%s)",
+                             psz_filename, strerror(errno) );
+        }
 
 #else
-    if( mkdir( psz_filename ) )
-    {
-        msg_Err( p_this, "could not create %s", psz_filename );
-    }
+        if( mkdir( psz_filename ) )
+        {
+            msg_Err( p_this, "could not create %s", psz_filename );
+        }
 
 #endif
 
-    strcat( psz_filename, "/" CONFIG_FILE );
-
+        strcat( psz_filename, "/" CONFIG_FILE );
+    }
+    else
+    {
+        psz_filename = strdup( psz_configfile );
+        if( !psz_filename )
+        {
+            msg_Err( p_this, "out of memory" );
+            vlc_mutex_unlock( &p_this->p_vlc->config_lock );
+            return -1;
+        }
+    }
 
     msg_Dbg( p_this, "opening config file %s", psz_filename );
 
@@ -986,6 +1136,7 @@ int __config_SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name )
              p_item->i_type != CONFIG_HINT_END;
              p_item++ )
         {
+            char *psz_key;
             if( p_item->i_type & CONFIG_HINT )
                 /* ignore hints */
                 continue;
@@ -1002,6 +1153,17 @@ int __config_SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name )
                     fprintf( file, "#" );
                 fprintf( file, "%s=%i\n", p_item->psz_name, p_item->i_value );
                 break;
+            case CONFIG_ITEM_KEY:
+                if( p_item->psz_text )
+                    fprintf( file, "# %s (%s)\n", p_item->psz_text,
+                             _("key") );
+                if( p_item->i_value == p_item->i_value_orig )
+                    fprintf( file, "#" );
+                psz_key = ConfigKeyToString( p_item->i_value );
+                fprintf( file, "%s=%s\n", p_item->psz_name,
+                         psz_key ? psz_key : "" );
+                if ( psz_key ) free( psz_key );
+                break;
 
             case CONFIG_ITEM_FLOAT:
                 if( p_item->psz_text )
@@ -1071,8 +1233,6 @@ int __config_LoadCmdLine( vlc_object_t *p_this, int *pi_argc, char *ppsz_argv[],
     p_this->p_vlc->i_argc    = *pi_argc;
     p_this->p_vlc->ppsz_argv = ppsz_argv;
 
-    p_this->p_vlc->p_channel = NULL;
-
 #ifdef SYS_DARWIN
     /* When vlc.app is run by double clicking in Mac OS X, the 2nd arg
      * is the PSN - process serial number (a unique PID-ish thingie)
@@ -1266,11 +1426,14 @@ int __config_LoadCmdLine( vlc_object_t *p_this, int *pi_argc, char *ppsz_argv[],
                 config_PutPsz( p_this, psz_name, optarg );
                 break;
             case CONFIG_ITEM_INTEGER:
-                config_PutInt( p_this, psz_name, atoi(optarg));
+                config_PutInt( p_this, psz_name, strtol(optarg, 0, 0));
                 break;
             case CONFIG_ITEM_FLOAT:
                 config_PutFloat( p_this, psz_name, (float)atof(optarg) );
                 break;
+            case CONFIG_ITEM_KEY:
+                config_PutInt( p_this, psz_name, ConfigStringToKey( optarg ) );
+                break;
             case CONFIG_ITEM_BOOL:
                 config_PutInt( p_this, psz_name, !flag );
                 break;
@@ -1319,7 +1482,7 @@ int __config_LoadCmdLine( vlc_object_t *p_this, int *pi_argc, char *ppsz_argv[],
                 else
                 {
                     config_PutInt( p_this, pp_shortopts[i_cmd]->psz_name,
-                                           atoi(optarg) );
+                                           strtol(optarg, 0, 0) );
                 }
                 break;
             case CONFIG_ITEM_BOOL:
@@ -1379,7 +1542,7 @@ char *config_GetHomeDir( void )
 
 #if defined(WIN32) || defined(UNDER_CE)
     typedef HRESULT (WINAPI *SHGETFOLDERPATH)( HWND, int, HANDLE, DWORD,
-                                               LPTSTR );
+                                               LPSTR );
 #   define CSIDL_FLAG_CREATE 0x8000
 #   define CSIDL_APPDATA 0x1A
 #   define SHGFP_TYPE_CURRENT 0
@@ -1388,10 +1551,10 @@ char *config_GetHomeDir( void )
     SHGETFOLDERPATH SHGetFolderPath ;
 
     /* load the shfolder dll to retrieve SHGetFolderPath */
-    if( ( shfolder_dll = LoadLibrary("shfolder.dll") ) != NULL )
+    if( ( shfolder_dll = LoadLibrary( _T("SHFolder.dll") ) ) != NULL )
     {
         SHGetFolderPath = (void *)GetProcAddress( shfolder_dll,
-                                                  "SHGetFolderPathA" );
+                                                  _T("SHGetFolderPathA") );
         if ( SHGetFolderPath != NULL )
         {
             p_homedir = (char *)malloc( MAX_PATH );
@@ -1438,3 +1601,65 @@ char *config_GetHomeDir( void )
 
     return p_homedir;
 }
+
+
+static int ConfigStringToKey( char *psz_key )
+{
+    int i_key = 0;
+    unsigned int i;
+    char *psz_parser = strchr( psz_key, '-' );
+    while( psz_parser && psz_parser != psz_key )
+    {
+        for( i = 0; i < sizeof(vlc_modifiers) / sizeof(key_descriptor_t); i++ )
+        {
+            if( !strncasecmp( vlc_modifiers[i].psz_key_string, psz_key,
+                              strlen( vlc_modifiers[i].psz_key_string ) ) )
+            {
+                i_key |= vlc_modifiers[i].i_key_code;
+            }
+        }
+        psz_key = psz_parser + 1;
+        psz_parser = strchr( psz_key, '-' );
+    }
+    for( i = 0; i < sizeof(vlc_keys) / sizeof( key_descriptor_t ); i++ )
+    {
+        if( !strcasecmp( vlc_keys[i].psz_key_string, psz_key ) )
+        {
+            i_key |= vlc_keys[i].i_key_code;
+            break;
+        }
+    }
+    return i_key;
+}
+
+static char *ConfigKeyToString( int i_key )
+{
+    char *psz_key = malloc( 100 );
+    char *p;
+    size_t index;
+
+    if ( !psz_key )
+    {
+        return NULL;
+    }
+    *psz_key = '\0';
+    p = psz_key;
+    for( index = 0; index < (sizeof(vlc_modifiers) / sizeof(key_descriptor_t));
+         index++ )
+    {
+        if( i_key & vlc_modifiers[index].i_key_code )
+        {
+            p += sprintf( p, "%s-", vlc_modifiers[index].psz_key_string );
+        }
+    }
+    for( index = 0; index < (sizeof(vlc_keys) / sizeof( key_descriptor_t));
+         index++)
+    {
+        if( (int)( i_key & ~KEY_MODIFIER ) == vlc_keys[index].i_key_code )
+        {
+            p += sprintf( p, "%s", vlc_keys[index].psz_key_string );
+            break;
+        }
+    }
+    return psz_key;
+}
