@@ -1,8 +1,8 @@
 /*****************************************************************************
  * threads.c : threads implementation for the VideoLAN client
  *****************************************************************************
- * Copyright (C) 1999, 2000, 2001, 2002, 2003 VideoLAN
- * $Id: threads.c,v 1.44 2003/11/22 00:41:07 titer Exp $
+ * Copyright (C) 1999-2004 VideoLAN
+ * $Id: threads.c,v 1.47 2004/02/22 15:41:27 massiot Exp $
  *
  * Authors: Jean-Marc Dressler <polux@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -568,7 +568,9 @@ int __vlc_thread_create( vlc_object_t *p_this, char * psz_file, int i_line,
     {
         int i_error, i_policy;
         struct sched_param param;
+
         memset( &param, 0, sizeof(struct sched_param) );
+        i_priority += config_GetInt( p_this, "rt-offset" );
         if ( i_priority < 0 )
         {
             param.sched_priority = (-1) * i_priority;
@@ -645,13 +647,17 @@ int __vlc_thread_set_priority( vlc_object_t *p_this, char * psz_file,
     }
 
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
-
-#ifdef SYS_DARWIN
-    if ( i_priority )
+    if ( i_priority
+#ifndef SYS_DARWIN
+            && config_GetInt( p_this, "rt-priority" )
+#endif
+        )
     {
         int i_error, i_policy;
         struct sched_param param;
+
         memset( &param, 0, sizeof(struct sched_param) );
+        i_priority += config_GetInt( p_this, "rt-offset" );
         if ( i_priority < 0 )
         {
             param.sched_priority = (-1) * i_priority;
@@ -662,7 +668,9 @@ int __vlc_thread_set_priority( vlc_object_t *p_this, char * psz_file,
             param.sched_priority = i_priority;
             i_policy = SCHED_RR;
         }
-        if ( (i_error = pthread_setschedparam( pthread_self(),
+        if ( !p_this->thread_id )
+            p_this->thread_id = pthread_self();
+        if ( (i_error = pthread_setschedparam( p_this->thread_id,
                                                i_policy, &param )) )
         {
             msg_Warn( p_this, "couldn't set thread priority (%s:%d): %s",
@@ -670,8 +678,6 @@ int __vlc_thread_set_priority( vlc_object_t *p_this, char * psz_file,
             i_priority = 0;
         }
     }
-#endif
-
 #endif
 
     return 0;
